//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework;

/**
 * This is a simple logging utility for use within the client framework.
 * It may, of course, be used in client applications as well.
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1998.11.16  MB              Created
 */
public class FWLog {



    // Singleton logger instance
    private static Logger logger = null;

    /**
     * Inner class providing the most basic of logging facilities: System.out
     */
    private static class DefaultLogger implements Logger {
        private boolean debugging = false;

        DefaultLogger(boolean debug) {
            debugging = debug;
        }

        public void log(String info) {
            System.out.println("LOG: " + info);
        }

        public void debug(String info) {
            if (debugging) {
                log("[DEBUG]" + info);
            }
        }

        public void debug(String info, Exception ex) {
            if (debugging) {
                log("[DEBUG]" + info);
            }
            if (ex instanceof RuntimeException)
            {
                // Keine neue Exception um die alte herum legen.
                throw (RuntimeException)ex;
            }
            throw new RuntimeException(ex);
        }

        public void setDebugging(boolean on) {
            this.debugging = on;
        }

        public void error(String info, Exception ex) {
            if (ex instanceof RuntimeException)
            {
                // Keine neue Exception um die alte herum legen.
                throw (RuntimeException)ex;
            }
            throw new RuntimeException(ex);
        }

        public void error(Exception ex) {
            if (ex instanceof RuntimeException)
            {
                // Keine neue Exception um die alte herum legen.
                throw (RuntimeException)ex;
            }
            throw new RuntimeException(ex);
        }
    }


    /**
     * Write some debugging information
     *
     * @param info the information to be written
     * @author Marcel Baumann
     */
    public static void debug(String info) {
        getLogger().debug(info);
    }

    /**
     * Write some debugging information
     */
    public static void debug(String info, Exception ex) {
        getLogger().debug(info, ex);
    }

    public static void error(String info, Exception ex) {
        getLogger().error(info, ex);
    }

    public static void error(Exception ex) {
        getLogger().error(ex);
    }

    /**
     * Write some logging information
     *
     * @param info the information to be written
     * @author Marcel Baumann
     */
    public static void log(String info) {
        getLogger().log(info);
    }


    /**
     * Set whether or not debugging onfo should be written
     *
     * @param on true if debugging info should be written
     */
    public static void setDebugging(boolean on) {
        getLogger().setDebugging(on);
    }


    /**
     * Set the logger implementation
     *
     * @param logger the logger implementation
     * @author Marcel Baumann
     */
    public static void setLogger(Logger logger) {
        FWLog.logger = logger;
    }


    /**
     * Get the logger implementation
     *
     * @return com.nexirius.framework.Logger
     * @author Marcel Baumann
     */
    public static Logger getLogger() {
        if (FWLog.logger == null) {
            boolean debug = false;

            try {
                debug = (System.getProperty("debug") != null);
            } catch (SecurityException ex) {
            }

            FWLog.logger = new DefaultLogger(debug);
        }

        return FWLog.logger;
    }
}
