//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.application;

import com.nexirius.framework.dataeditor.PopupEditorAdaptor;
import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.DataModelEnumeration;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.framework.gadgets.ErrorAdaptor;
import com.nexirius.framework.swing.JnexFileChooser;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;

@SuppressWarnings("serial")
public class DialogManager {
    static boolean defaultRememberDialogBounds = true;
    static PopupEditorAdaptor popupEditorAdaptor;
    static AskAdaptor askAdaptor;
    static ErrorAdaptor errorAdaptor;
    static ProgressAdaptor progressAdaptor;
    static int errorWindowCount = 0;
    static JFileChooser chooser;
    static Hashtable<String, DialogPropertiesModel> dialogProperties = new Hashtable<String, DialogPropertiesModel>();
    private static JFrame applicationFrame;

    public static boolean isDefaultRememberDialogBounds() {
        return defaultRememberDialogBounds;
    }

    public static void setDefaultRememberDialogBounds(boolean defaultRememberDialogBounds) {
        DialogManager.defaultRememberDialogBounds = defaultRememberDialogBounds;
    }

    public static ArrayModel<DialogPropertiesModel> getDialogPropertiesModels() {
        ArrayModel<DialogPropertiesModel> array = new ArrayModel<DialogPropertiesModel>(new DialogPropertiesModel(), "DialogPropertyArray");
        Enumeration<String> en = dialogProperties.keys();

        while (en.hasMoreElements()) {
            DialogPropertiesModel dialogPropertiesModel = dialogProperties.get(en.nextElement());

            dialogPropertiesModel.setParentDataModelContainer(null);
            array.append(dialogPropertiesModel);
        }

        return array;
    }

    public static void setDialogPropertiesModels(ArrayModel<DialogPropertiesModel> array) {
        DataModelEnumeration<DialogPropertiesModel> en = array.getEnumeration();

        while (en.hasMore()) {
            DialogPropertiesModel dialogPropertiesModel = en.next();
            dialogProperties.put(dialogPropertiesModel.getName(), dialogPropertiesModel);
        }
    }

    public static void setPopupEditorAdaptor(PopupEditorAdaptor ea) {
        popupEditorAdaptor = ea;
    }

    public static PopupEditorAdaptor getPopupEditorAdaptor() {
        return popupEditorAdaptor;
    }

    public static void setAskAdaptor(AskAdaptor aa) {
        askAdaptor = aa;
    }

    public static void setProgressAdaptor(ProgressAdaptor pa) {
        progressAdaptor = pa;
    }

    public static ProgressAdaptor getProgressAdaptor() {
        return progressAdaptor;
    }

    public static AskAdaptor getAskAdaptor() {
        return askAdaptor;
    }

    public static void setErrorAdaptor(ErrorAdaptor ea) {
        errorAdaptor = ea;
    }

    public static ErrorAdaptor getErrorAdaptor() {
        if (errorAdaptor == null) {
            errorAdaptor = new DefaultErrorAdaptor();
        }

        return errorAdaptor;
    }

    public static ErrorMessageException getErrorMessageException(Throwable e) {
        if (e == null) {
            return null;
        }

        if (e instanceof ErrorMessageException) {
            return (ErrorMessageException) e;
        }

        return getErrorMessageException(e.getCause());
    }

    public static void error(Throwable ex) {
        String message = ex.getMessage();

        if (message == null) {
            message = ex.toString();
        }

        ErrorMessageException errorMessageException = getErrorMessageException(ex);

        if (errorMessageException != null) {
            message = errorMessageException.getMessage();
            String cause = "";

            if (errorMessageException.getCause() != null) {
                cause = " (" + errorMessageException.getCause().toString() + ")";
            }
            error(ViewerFactory.getInstance().getText(message) + cause, errorMessageException.getAttributes());
        } else {
            error(message, (Object[]) null);
        }
    }

    public static void error(String message) {
        error(message, (Object[]) null);
    }

    public static void error(String message, Object param[]) {
        SwingUtilities.invokeLater(new ErrorRunnable(getErrorAdaptor(), message, param, true));
    }

    public static void error(String message, Object param1) {
        Object param[] = new Object[1];

        param[0] = param1;

        error(message, param);
    }

    public static void warning(String message) {
        warning(message, null);
    }

    public static void warning(String message, Object param[]) {
        SwingUtilities.invokeLater(new ErrorRunnable(getErrorAdaptor(), message, param, false));
    }

    public static JFrame getApplicationFrame() {
        return applicationFrame;
    }

    public static void setApplicationFrame(JFrame mainFrame) {
        DialogManager.applicationFrame = mainFrame;
    }

    static class ErrorRunnable implements Runnable {
        ErrorAdaptor errorAdaptor;
        String message;
        Object[] param;
        boolean isError;

        public ErrorRunnable(ErrorAdaptor errorAdaptor, String message, Object[] param, boolean isError) {
            this.errorAdaptor = errorAdaptor;
            this.message = message;
            this.param = param;
            this.isError = isError;
        }

        @Override
        public void run() {
            if (isError) {
                errorAdaptor.error(message, param);
            } else {
                errorAdaptor.warning(message, param);
            }
        }
    }

    public static boolean ask(String question, String answerYes, String answerNo, boolean defaultIsYes) {
        return askAdaptor.ask(question, answerYes, answerNo, defaultIsYes);
    }

    /**
     * Modal window
     *
     * @param question
     * @param answerYes
     * @param answerNo
     * @param defaultIsYes
     * @param hasCancel    if true, throws RuntimeException on Cancel
     */
    public static boolean ask(String question, String answerYes, String answerNo, boolean defaultIsYes, boolean hasCancel, Object[] parameters) {
        return askAdaptor.ask(question, answerYes, answerNo, defaultIsYes, hasCancel, parameters);
    }

    /**
     * Throws RuntimeException on Cancel
     *
     * @param question
     * @param answerYes
     * @param answerNo
     * @param defaultIsYes
     * @param hasCancel
     */
    public static boolean ask(String question, String answerYes, String answerNo, boolean defaultIsYes, boolean hasCancel) {
        return askAdaptor.ask(question, answerYes, answerNo, defaultIsYes, hasCancel);
    }

    /**
     * Sets the Default Cursor.
     * @param c null (for toplevelFrame) or the component which changes its cursor
     */
    public static void setDefaultCursor(Component c) {
        if (c == null) {
            c = getToplevelFrame();
        }

        if (c != null) {
            c.setCursor(Cursor.getDefaultCursor());
        }
    }

    /**
     * Sets the Default Cursor as soon as all AWT events have been processed!
     * @param c null (for toplevelFrame) or the component which changes its cursor
     */
    public static void setDefaultCursorLater(Component c) {
        setDefaultCursorLater(c, 1);
    }

    protected static void setDefaultCursorLater(final Component c, final int callCount) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                if (callCount >= 5) {
                    setDefaultCursor(c);
                }
                else {
                    // Give other tasks / threads time to do their work.
                    // Only after that we set the cursor back to its default look
                    Thread.yield();
                    try {
                        Thread.sleep(50 / callCount);
                    }
                    catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    setDefaultCursorLater(c, callCount + 1);
                }
            }
        };
        SwingUtilities.invokeLater(runnable);
    }

    /**
     * set the components cursor (hourglass)
     *
     * @param c null (for toplevelFrame) or the component which changes its cursor
     * @return the component which has changed its cursor
     */
    public static Component setWaitCursor(Component c) {
        if (c == null) {
            c = getToplevelFrame();
        }

        if (c != null) {
            c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }

        return c;
    }

    public static void showError() {
        ++errorWindowCount;
    }

    public static void hideError() {
        --errorWindowCount;

        if (errorWindowCount < 0) {
            new Exception("ErrorWindowCount < 0").printStackTrace();
            errorWindowCount = 0;
        }
    }

    public static boolean hasErrorWindow() {
        return errorWindowCount > 0;
    }

    public static JDialog createJDialog(String title, boolean modal, String dialogPropertiesName) {
        return createJDialog(title, modal, dialogPropertiesName, true);
    }

    public static JDialog createJDialog(String title, boolean modal, String dialogPropertiesName, boolean closeOnEscapeKey) {
        final JDialog ret = createJDialog(title, modal);
        ret.setName(dialogPropertiesName);

        DialogPropertiesModel dialogPropertiesModel = getDialogProperties(dialogPropertiesName);

        if (dialogPropertiesModel == null) {
            Rectangle bounds = ret.getBounds();
            createDialogProperties(dialogPropertiesName, bounds);
        } else {
            dialogPropertiesModel.apply(ret);
        }

        if (closeOnEscapeKey) {
            closeDialogOnEscape(ret);
        }

        return ret;
    }

    public static void closeDialogOnEscape(final JDialog ret) {
        KeyStroke keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);

        Container contentPane = ret.getContentPane();

        if (contentPane instanceof JComponent) {
            JComponent comp = (JComponent) contentPane;
            String mapKey = "close";
            comp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(keyStroke, mapKey);
            comp.getActionMap().put(mapKey, new AbstractAction() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    ret.setVisible(false);
                    ret.dispose();
                }
            });
        }
    }

    public static DialogPropertiesModel getDialogProperties(String dialogPropertiesName) {
        return dialogProperties.get(dialogPropertiesName);
    }

    public static void createDialogProperties(String dialogPropertiesName, Rectangle bounds) {
        DialogPropertiesModel dialogPropertiesModel;
        dialogPropertiesModel = new DialogPropertiesModel(dialogPropertiesName, bounds);
        dialogProperties.put(dialogPropertiesName, dialogPropertiesModel);
    }

    public static boolean saveDialogProperties(JDialog dialog) {
        String dialogPropertiesName = dialog.getName();

        if (dialogPropertiesName == null || dialogPropertiesName.startsWith("dialog")) {
            return false;
        }

        DialogPropertiesModel dialogPropertiesModel = new DialogPropertiesModel(dialogPropertiesName, dialog.getBounds());
        dialogProperties.put(dialogPropertiesName, dialogPropertiesModel);

        return true;
    }

    public static boolean applyDialogProperties(JDialog dialog) {
        String dialogPropertiesName = dialog.getName();

        if (dialogPropertiesName == null || dialogPropertiesName.startsWith("dialog")) {
            return false;
        }

        DialogPropertiesModel dialogPropertiesModel = getDialogProperties(dialogPropertiesName);

        if (dialogPropertiesModel != null) {
            return dialogPropertiesModel.apply(dialog);
        }

        return false;
    }

    /**
     * create a new JDialog window which is based on the current toplevelFrame
     */
    public static JDialog createJDialog(String title, boolean modal) {
        return createJDialog(title, modal, true);
    }

    public static JDialog createJDialog(String title, boolean modal, boolean closeOnEscapeKey) {
        Component owner;

        if (modal) {
            owner = getToplevelFrame();
        } else {
            owner = applicationFrame;

            /**
             * avoid problem with modal dialogs which start a non modal dialog
             */
            if (getToplevelFrame() instanceof JDialog) {
                if (((JDialog) getToplevelFrame()).isModal()) {
                    owner = getToplevelFrame();
                }
            }
        }

        return createJDialog(owner, title, modal, closeOnEscapeKey);
    }

    public static JDialog createJDialog(Component owner, String title, boolean modal, boolean closeOnEscapeKey) {
        JDialog ret;

        if (owner instanceof Frame) {
            ret = new JDialog((Frame) owner, title, modal);
        } else if (owner instanceof Dialog) {
            ret = new JDialog((Dialog) owner, title, modal);
        } else {
            ret = new JDialog(new Frame(), title, modal);
        }

        if (closeOnEscapeKey) {
            closeDialogOnEscape(ret);
        }

        return ret;
    }

    /**
     * Create a JComboBox with all available look and feel
     *
     * @return a look and feel selector combo box
     */
    public static JComboBox createLookAndFeelSelector() {
        final JComboBox jComboBox1 = new JComboBox();
        int cnt;

        UIManager.LookAndFeelInfo installed_laf[];
        installed_laf = UIManager.getInstalledLookAndFeels();

        for (cnt = 0; cnt < installed_laf.length; cnt++) {
            jComboBox1.addItem(installed_laf[cnt].getName());
        }

        jComboBox1.setSelectedItem(UIManager.getLookAndFeel().getName());

        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(ActionEvent evt) {

                UIManager.LookAndFeelInfo inst_laf[];
                inst_laf = UIManager.getInstalledLookAndFeels();
                try {
                    UIManager.setLookAndFeel(inst_laf[jComboBox1.getSelectedIndex()].getClassName());
                    SwingUtilities.updateComponentTreeUI(DialogManager.getToplevelFrame());
                    SwingUtilities.updateComponentTreeUI(DialogManager.getApplicationFrame());
                }
                catch (Throwable e) {
                    throw new RuntimeException(e);
                }
            }
        });

        return jComboBox1;
    }

    /**
     * show the dialog on top of all other windows and center it against
     * the current toplevel component
     */
    public static void popup(JDialog dialog) {
        popup(dialog, defaultRememberDialogBounds);
    }

    /**
     * Show dialog
     * @param dialog
     * @param rememberDialogBounds next time a dialog with the same name is generated it apears at the
     * original position and with the original size on the screen
     */
    public static void popup(JDialog dialog, boolean rememberDialogBounds) {
        try {
            new PopupRunnable(dialog, rememberDialogBounds).run();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public static void popdown(JDialog dialog) {
        if (dialog != null) {
            dialog.setVisible(false);
            dialog.dispose();
        }
    }


    static class PopupRunnable implements Runnable {
        JDialog dialog;
        boolean rememberDialogBounds;

        public PopupRunnable(JDialog dialog, boolean rememberDialogBounds) {
            this.dialog = dialog;
            this.rememberDialogBounds = rememberDialogBounds;
        }

        @Override
        public void run() {
            if (!rememberDialogBounds || !applyDialogProperties(dialog)) {
                if (dialog.getOwner() == getApplicationFrame()) {
                    center(dialog, getApplicationFrame().getBounds());
                } else {
                    center(dialog, false);
                }
            } else if (dialog.getOwner() != null && dialog.getOwner().getName() != null && dialog.getOwner().getName().equals(dialog.getName())) {
                Point location = dialog.getOwner().getLocation();
                dialog.setLocation(location.x + 20, location.y + 20);
                saveDialogProperties(dialog);
            }
            dialog.addWindowListener(new DialogListener(dialog));

            try {
                dialog.setVisible(true);
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }

            //            if (!dialog.isModal()) {
            //                setAlwaysOnTop(dialog, true);
            //                setAlwaysOnTop(dialog, false);
            //            }
        }
    }

    //    private static Method setAlwaysOnTopMethod;
    //
    //    protected static void initAlwaysOnTopMethods() {
    //        // These methods is only needed for Gtk, Reflection is used to allow compilation
    //        // against JDK 1.4
    //        if (setAlwaysOnTopMethod == null) {
    //            try {
    //                setAlwaysOnTopMethod = Window.class.getMethod("setAlwaysOnTop", new Class[]{boolean.class});
    //            } catch (NoSuchMethodException e) {
    //                throw new RuntimeException(e);
    //            }
    //        }
    //    }
    //
    //    public static void setAlwaysOnTop(Window window, boolean onTop) {
    //        initAlwaysOnTopMethods();
    //        if (setAlwaysOnTopMethod == null) {
    //            return;
    //        }
    //        try {
    //            setAlwaysOnTopMethod.invoke(window, new Object[]{new Boolean(onTop)});
    //        } catch (IllegalAccessException e) {
    //            throw new RuntimeException(e);
    //        } catch (InvocationTargetException e) {
    //            throw new RuntimeException(e);
    //        }
    //    }


    /**
     * put the component in the center of the current toplevel window
     * or in the center of the screen
     */
    public static void center(Component comp, boolean centerOfScreen) {
        Rectangle bounds = null;

        if (centerOfScreen || getToplevelFrame() == null) {
            Rectangle screenSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();

            bounds = screenSize;
        } else {
            bounds = getToplevelFrame().getBounds();
        }

        center(comp, bounds);
    }

    public static void center(Component comp, Rectangle bounds) {
        Dimension dim = comp.getSize();
        Point center = new Point(bounds.x + bounds.width / 2, bounds.y + bounds.height / 2);
        Point p = new Point(center.x - dim.width / 2, center.y - dim.height / 2);

        //Problem with dual moniors with secondary monitor on the left
        //p.x = Math.max(p.x, 0);
        //p.y = Math.max(p.y, 0);
        ensureUppeLeftCornerVisibility(p, dim);

        comp.setLocation(p);
    }

    /*
     * checks if the new window rectangle would be completly visible on one of the
     * screens. If it would be shared move it until it is visible on the main window.
     *
     * created by Joachim Schlienger
     */
    private static void ensureUppeLeftCornerVisibility(Point p, Dimension dim) {

        Rectangle visibleRectangle = new Rectangle(p, dim);
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice[] gs = ge.getScreenDevices();

        int primaryWidth = 0;
        int primaryHeight = 0;

        boolean isVisible = false;

        for(GraphicsDevice gd : gs) {
            if(gd.getDefaultConfiguration() != null) {
                Rectangle bnds = gd.getDefaultConfiguration().getBounds();

                if(bnds.x == 0 && bnds.y == 0) {
                    primaryWidth = bnds.width;
                    primaryHeight = bnds.height;
                }
                if(bnds.contains(visibleRectangle)) {
                    isVisible = true;
                }
            }

        }

        if(!isVisible) {
            p.x = Math.max(p.x, 0);
            p.y = Math.max(p.y, 0);
            p.x = Math.min(p.x, primaryWidth - dim.width);
            p.y = Math.min(p.y, primaryHeight - dim.height);
        }
    }

    /**
     * @param toplevelFrame
     * @deprecated
     */
    @Deprecated
    public static void setToplevelFrame(Component toplevelFrame) {
        //DialogManager.toplevelFrame = toplevelFrame;
    }

    public static Component getToplevelFrame() {
        return KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusedWindow();
    }

    private static class DialogListener extends WindowAdapter {
        JDialog dialog;

        private void close() {
            saveDialogProperties(dialog);
            setDefaultCursor(null);
        }

        public DialogListener(JDialog d) {
            this.dialog = d;
        }

        @Override
        public void windowClosed(WindowEvent e) {
            close();
        }
    }


    static class DefaultErrorAdaptor implements ErrorAdaptor {
        public void show(String kat, String message) {
            System.err.println("[" + kat + "]: " + message);
        }

        @Override
        public void error(String message) {
            show("error", message);
        }

        @Override
        public void error(String message, Object param[]) {
            MessageFormat f = new MessageFormat(message);

            show("error", f.format(param));
        }

        @Override
        public void warning(String message) {
            show("warning", message);
        }

        @Override
        public void warning(String message, Object param[]) {
            MessageFormat f = new MessageFormat(message);

            show("warning", f.format(param));
        }
    }

    /**
     * @param directoryName null is System.getProperty("user.dir")
     */
    public static JnexFileChooser getFileChooser(String directoryName) {
        return JnexFileChooser.getInstance(directoryName);
    }

    public static void main(String[] args) {
        JFrame mainFrame = new JFrame("Application");
        mainFrame.setBounds(100, 100, 100, 100);
        mainFrame.setVisible(true);
        DialogManager.setApplicationFrame(mainFrame);
        JDialog modal1 = DialogManager.createJDialog("Modal1", true);

        JButton button = new JButton("press");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JDialog notModalPopup = DialogManager.createJDialog("NotModalPopup", false);
                notModalPopup.setBounds(100, 100, 100, 100);
                DialogManager.popup(notModalPopup);
            }
        });
        modal1.getContentPane().add(button);
        modal1.pack();
        DialogManager.popup(modal1);
        JDialog notModal = DialogManager.createJDialog("NotModal1", false);
        DialogManager.popup(notModal);
        JDialog modal2 = DialogManager.createJDialog("Modal2", true);
        DialogManager.popup(modal2);
    }

}

