//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

/**
 * This type represents a context - a set of commands which are possibly
 * available to the end-user at some point in the life-time of an application.
 * Of course, whether or not these commands really become avbailable through
 * GUI widgets depends on the actions of the end-user.
 * Implementations of this interface may manage commands in whatever way
 * they choose.  For example, one context implementation may indeed represent
 * a simple list of commands.  Whilst another implementation may represent
 * a composite of multiple contexts.
 *
 * @author Marcel Baumann
 * @version Date        Chng.Rq.   Author    Changes/Enhancements
 *          1999.01.04             MB       Created
 */
public interface Context {
    /**
     * Returns whether of not a command of the given name is contained in
     * this context
     *
     * @param commandName The command name of interest
     * @return boolean
     * @author Marcel Baumann
     * @see com.nexirius.framework.command.Command#getCommandName()
     */
    boolean containsCommand(String commandName);

    /**
     * Returns a command corresponding to the given name
     *
     * @param commandName The command name of interest
     * @return com.nexirius.framework.command.Command
     * @author Marcel Baumann
     */
    Command getCommand(String commandName);

    /**
     * Returns a complete list of all the commands in this context
     *
     * @return com.nexirius.framework.command.Command[]
     * @author Marcel Baumann
     */
    Command[] getCommandList();

    /**
     * Returns the name of this context
     *
     * @return java.lang.String
     * @author Marcel Baumann
     */
    String getName();

    /**
     * Returns the processor associated with this context for the given
     * command
     *
     * @return com.nexirius.framework.command.Processor
     * @author Marcel Baumann
     */
    Processor getProcessor(String commandName);
}
