//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import java.awt.*;

/**
 * This is the event multicaster class to support the ContextChangeListener
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1998.11.16  MB              Created
 */
public class ContextChangeEventMulticaster
        extends AWTEventMulticaster
        implements ContextChangeListener {




    /**
     * Constructor to support multicast events.
     *
     * @param a A context change listener
     * @param b A context change listener
     */
    protected ContextChangeEventMulticaster(ContextChangeListener a,
                                            ContextChangeListener b) {
        super(a, b);
    }


    /**
     * Implement ContextChangeListener interface
     *
     * @param event A context change event
     * @see com.nexirius.framework.ContextChangeListener
     */
    public void contextChange(ContextChangeEvent event) {
        ((ContextChangeListener) a).contextChange(event);
        ((ContextChangeListener) b).contextChange(event);
    }


    /**
     * Add new listener to support multicast events.
     *
     * @param a A context change listener
     * @param b A context change listener
     * @return ContextChangeListener
     */
    public static ContextChangeListener add(ContextChangeListener a,
                                            ContextChangeListener b) {
        if (a == null) return b;
        if (b == null) return a;
        return new ContextChangeEventMulticaster(a, b);
    }


    /**
     * Remove listener to support multicast events
     *
     * @param a A context change listener
     * @param b A context change listener
     * @return ContextChangeListener
     */
    public static ContextChangeListener remove(ContextChangeListener a,
                                               ContextChangeListener b) {
        return (ContextChangeListener) removeInternal(a, b);
    }
}
