//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.util.assertion.Assert;

import java.awt.*;

/**
 * This type is a simple extension of the DefaultProcessor class.
 * Its sole purpose in life is to execute commands and to modify the mouse
 * pointer in a way that reflects that a command is being executed.
 * In order to do this it needs a reference to a parent component. This is
 * typically the applications main window, and it is given in the constructor
 * of the class.
 * !! This strategy for modifying the mouse point through a subclass of
 * !! DefaultProcessor does not seem like a particularly good idea. Processors
 * !! should have nothing to do with GUIs, let alone mouse pointers.
 * !! More preferable would be to have processors fire ProcessorEvents.
 * !! Listeners can therefore be notified of processor status e.g. synchronous
 * !! command execution. One such listener could be responsible for controlling
 * !! the mouse pointer, or progress bar etc.
 * <p/>
 *
 * @author Marcel Baumann
 * @deprecated for the reasons spelled out in these class comments
 *             <p/>
 *             Date        Chng.Rq.   Author    Changes/Enhancements
 *             1999.01.04             MB       Created
 */
@Deprecated
public class GUIProcessor extends DefaultProcessor {



    // A parent component is required in order to make changes to the mouse
    // pointer.
    private Component parentComponent;

    /**
     * Constructor a GUIProcessor
     *
     * @param parentComponent Component The GUI component relative to which
     *                        the cursor will change its appearance whilst
     *                        commands are being executed
     */
    public GUIProcessor(Component parentComponent) {
        super();
        Assert.pre(parentComponent != null,
                "Parameter parentComponent is not null");
        this.parentComponent = parentComponent;
    }


    /**
     * Execute a command
     *
     * @param command The command to be executed
     * @param param   A parameter which is passed to the initialiseAction()
     *                method of the command.
     */
    @Override
    public synchronized void execute(Command command, Object param) {
        Assert.pre(command != null, "Parameter command is not null");

        // Change the current cursor
        DialogManager.setWaitCursor(parentComponent);
        try {
            // Set wait cursor
            Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
            this.parentComponent.setCursor(waitCursor);

            // Do as the superclass does
            super.execute(command, param);
        } finally {
            // Restore the orignal cursor

            DialogManager.setDefaultCursorLater(parentComponent);
        }
    }
}
