//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.util.assertion.Assert;

/**
 * Provide a simple implementation of a context which contains no commands.
 * Null object pattern.
 *
 * @see com.nexirius.framework.command.Context
 * @see com.nexirius.framework.command.DefaultContextManager
 */
public class NullContext implements Context {




    /**
     * The string returned by getName()
     *
     * @see #getName()
     */
    public static final String NULL_CONTEXT_NAME = "NULL_CONTEXT";

    /**
     * The empty list of commands returned by getCommandList()
     *
     * @see #getCommandList()
     */
    private static Command[] nullList = null;


    /**
     * The processor returned by getProcessor()
     *
     * @see #getProcessor()
     */
    private static Processor nullProcessor = null;

    /**
     * Construct a null context
     */
    public NullContext() {
    }


    /**
     * Return whether the context contains the command with the given name.
     * In this implementation, always return false, since the null context
     * contains no commands
     *
     * @param commandName Does nothing
     * @return boolean
     */
    public boolean containsCommand(String commandName) {
        return false;
    }


    /**
     * Get the command with the given command name.
     * Since context contains no commands, this method cannot return a
     * command under any circumstances. Calling this method will therefore
     * result in a precondition violation.
     *
     * @param String A commandName (the command we are looking for)
     * @return A command
     */
    public Command getCommand(String commandName) {
        Assert.pre(false, "Command not in context");
        return null;
    }


    /**
     * Return a list of commands in this context.
     * In this implementation the list is of kength zero.
     *
     * @return An empty command list
     */
    public Command[] getCommandList() {
        if (nullList == null) {
            nullList = new Command[0];
        }
        return nullList;
    }


    /**
     * Return the name of this context.
     * return NULL_CONTEXT_NAME
     */
    public String getName() {
        return NULL_CONTEXT_NAME;
    }


    /**
     * Returns the processor associated with this context for the given
     * command
     *
     * @return com.nexirius.framework.command.Processor
     * @author Marcel Baumann
     */
    public Processor getProcessor(String commandName) {
        if (nullProcessor == null) {
            nullProcessor = new DefaultProcessor();
        }
        return nullProcessor;
    }
}
