//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.util.assertion.Assert;

import javax.swing.*;

/**
 * This type represents a basic implementation of the Processor interface.
 *
 * @author Marcel Baumann
 */
public class SimpleProcessor implements Processor {




    /**
     * Execute the command
     *
     * @param command The command to be executed
     * @param param   The paramater to be passed to the command's
     *                initialiseAction() method.
     * @author Marcel Baumann
     */
    public synchronized void execute(Command command, Object param) {
        Assert.pre(command != null, "Parameter command is not null");

        SwingUtilities.invokeLater(new SwingExecutor(command, param));
    }

    class SwingExecutor implements Runnable {
        private Command command;
        private Object param;

        SwingExecutor(Command command, Object param) {
            this.command = command;
            this.param = param;
        }

        public void run() {
            try {
                command.initialiseAction(param);
                command.doAction();
                command.finaliseAction();
            } catch (Exception e) {
                // Invoke the exception action on the command
                e.printStackTrace();
                command.exceptionAction(e);
            }
        }
    }
}
