//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.Viewable;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.ViewerCreator;


/**
 * This class creates an array editor
 * It is a single vertical column of all the children editors. The associated
 * layout is used to layout the array items
 */
public class ArrayEditorCreator implements ViewerCreator {




    int maxConcurrentElements = 1;
    ViewerCreator elementCreator = null;

    public ArrayEditorCreator() {
    }

    public ArrayEditorCreator(int maxConcurrentElements) {
        this.maxConcurrentElements = maxConcurrentElements;
    }

    public ArrayEditorCreator(ViewerCreator elementCreator, int maxConcurrentElements) {
        this.elementCreator = elementCreator;
        this.maxConcurrentElements = maxConcurrentElements;
    }

    public void setElementCreator(ViewerCreator elementCreator) {
        this.elementCreator = elementCreator;
    }

    /**
     * return true
     */
    public boolean isEditor() {
        return true;
    }

    /**
     * creates a new instance of type ArrayEditor
     *
     * @param model must be of type ArrayModel
     * @return the newly created ArrayViewer which is assotiated with the given array model
     */
    public DataViewer createViewer(Viewable model) {
        if (model == null) {
            throw new IllegalArgumentException("model may not be null");
        }

        if (!(model instanceof ArrayModel)) {
            throw new IllegalArgumentException("model must be of type ArrayModel but have: " + model.getClass().getName());
        }

        return new ArrayEditor((ArrayModel) model, elementCreator, maxConcurrentElements);
    }
}
