//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.BooleanModel;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.swing.CFJCheckBox;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * This viewer is designed to edit BooleanModel data in a Swing JCheckBox
 *
 * @author Marcel Baumann
 */

public class BooleanEditor extends DataViewer {
    public static final String editorName = "BooleanEditor";

    /**
     * Creates an instance and initialises its associated model
     */
    public BooleanEditor(BooleanModel model) {
        super(model);
    }

    /**
     * Returns true
     */
    public boolean isEditor() {
        return true;
    }

    /**
     * Returns the associated model as BooleanModel
     */
    public BooleanModel getModel() {
        return (BooleanModel) getDataModel();
    }

    /**
     * Returns the associated check box component
     */
    public CFJCheckBox getCheckBox() {
        return (CFJCheckBox) getJComponent();
    }

    /**
     * Creates the actual check box component
     */
    public void create() {
        CFJCheckBox checkBox = new CFJCheckBox(factory.getClientResource(), getModel().getFieldName());
        setJComponent(checkBox);

        checkBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean b = ((JCheckBox) getJComponent()).isSelected();
                if (getModel().getBoolean() != b) {
                    getModel().setBoolean(b);
                    getModel().fireEditEvent(getModel(), DataModelEvent.VALUE_EDIT);
                }
            }
        });

        update();
    }

    /**
     * Set the checkbox state to the associated boolean model value.
     */
    public void update() {
        updateUI();
    }

    /**
     * this method is called when the boolean model has changed
     */
    public void updateUI() {
        if (isCreated()) {
            super.updateUI();

            ((JCheckBox)getJComponent()).setSelected(getModel().getBoolean());

            if (getModel().isGray() || getModel().getFlag(ModelFlag.NOT_EDITABLE)) {
                getCheckBox().setEnabled(false);
            } else {
                getCheckBox().setEnabled(true);
            }
        }
    }

    /**
     * Only for debugging
     */
    public String getViewerName() {
        return editorName;
    }
}
