//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.ColorModel;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelAdaptor;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.framework.swing.SwingViewer;

import javax.swing.*;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class ColorEditor implements SwingViewer {

    JColorChooser colorChooser = new JColorChooser();
    ColorListener colorListener = new ColorListener();
    ColorModel colorModel;
    ColorSelectionModel selectionModel;

    public ColorEditor() {
    }

    public JComponent createJComponent(ViewerFactory factory) {
        colorModel.addSoftDataModelListener(colorListener);
        colorChooser.setColor(colorModel.getColor());

        selectionModel = colorChooser.getSelectionModel();

        selectionModel.addChangeListener(new ColorChangeListener());

        return colorChooser;
    }

    public boolean isEditor() {
        return true;
    }

    public void setDataModel(DataModel model) {
        colorModel = (ColorModel) model;
    }

    public DataModel getDataModel() {
        return colorModel;
    }

    class ColorChangeListener implements ChangeListener {
        public void stateChanged(ChangeEvent arg0) {
            colorModel.setColor(selectionModel.getSelectedColor());
        }
    }

    class ColorListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent ev) {
            colorChooser.setColor(colorModel.getColor());
        }
    }
}
