//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.framework.swing.SwingViewer;
import com.nexirius.framework.swing.SwingViewerCreator;
import com.nexirius.framework.application.DialogManager;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class DateEditor implements SwingViewer {
    DateModel dateModel;
    private JButton button;
    private DateEditor.DateListener dateListener;

    public DateEditor() {
    }

    public JComponent createJComponent(ViewerFactory factory) {
        JPanel panel = new JPanel(new BorderLayout());

        panel.add(factory.createViewer(new SimpleEditorCreator(), dateModel).getJComponent(), BorderLayout.CENTER);

        if (!dateModel.getFlag(ModelFlag.NOT_EDITABLE)) {
            button = new JButton("...");

            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    DialogManager.getPopupEditorAdaptor().popupEdit(dateModel, new SwingViewerCreator(DateChooserEditor.class));
                }
            });
            
            button.setMargin(new Insets(1, 3, 1, 3));
            panel.add(button, BorderLayout.EAST);
        }

        update();

        return panel;
    }

    public boolean isEditor() {
        return true;
    }

    public void setDataModel(DataModel model) {
        dateModel = (DateModel) model;
        dateListener = new DateListener();
        dateModel.addSoftDataModelListener(dateListener);
    }

    public DataModel getDataModel() {
        return dateModel;
    }

    public void update() {
        if (button != null) {
            button.setEnabled(!dateModel.getFlag(ModelFlag.GRAY));
        }
    }

    class DateListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            update();
        }
    }
}
