//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER

package com.nexirius.framework.dataeditor;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.swing.JnexFileChooser;

import javax.swing.*;
import java.awt.*;
import java.io.File;

/**
 * This viewer is designed to edit a file name string
 *
 * @author Marcel Baumann
 */

public class FileNameEditor extends DataViewer {

    public static final String editorName = "FileNameEditor";
    private DefaultDataModelCommand editCommand;

    public FileNameEditor(StringModel model) {
        super(model);
    }

    public void showDialog() {
        javax.swing.SwingUtilities.invokeLater(new ShowFileDialog());
    }

    public class ShowFileDialog implements Runnable {
        public void run() {
            JnexFileChooser chooser = DialogManager.getFileChooser(null);

            if (getModel() instanceof DirNameModel) {
                chooser.getJFileChooser().setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            } else {
                chooser.getJFileChooser().setFileSelectionMode(JFileChooser.FILES_ONLY);
            }

            String fileName = null;

            if (getModel().getText().length() > 2) {
                fileName = new File(getModel().getFullName()).getPath();
            } else if (getModel().getBrowseDirectory() != null) {
                fileName = getModel().getBrowseDirectory().getText();
            }

            File theFile = chooser.popupOpenDialog(fileName);
            if (theFile != null) {
                getModel().setText(theFile.getAbsolutePath());
                getModel().fireEditEvent(getModel(), DataModelEvent.VALUE_EDIT);
            }
        }
    }

    /**
     * Return true
     */
    public boolean isEditor() {
        return true;
    }

    /**
     * Returns the associated data model
     */
    public FileNameModel getModel() {
        return (FileNameModel) getDataModel();
    }

    /**
     * Returns the text field component or null (if not yet created)
     */
    public JPanel getJPanel() {
        return (JPanel) getJComponent();
    }

    /**
     * Create the actual text field
     */
    public void create() {
        JPanel panel = new JPanel();

        panel.setLayout(new BorderLayout());
        setJComponent(panel);
        editCommand = new EditCommand();

        getModel().appendMethod(editCommand);

        try {
            panel.add(factory.createViewer(new SimpleEditorCreator(), getModel()).getJComponent(), BorderLayout.CENTER);
            panel.add(factory.createDefaultEditor(editCommand).getJComponent(), BorderLayout.EAST);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        update();
    }

    public String getViewerName() {
        return editorName;
    }

    public void update() {
        if (editCommand != null && getModel() != null) {
            editCommand.setEnabled(!getModel().getFlag(ModelFlag.GRAY));
        }
    }

    class EditCommand extends DefaultDataModelCommand {
        public EditCommand() {
            super("...");
        }

        public boolean doAction() {
            showDialog();
            return(true);
        }
    }

}

