//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.ComboBoxModel;
import com.nexirius.framework.dataviewer.DataViewer;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;

/**
 * This editor is designed to edit a ComboBox in a  Swing JList
 *
 * @author Marcel Baumann
 */
public class SimpleArrayEditor extends DataViewer {



    public static final String s_viewername = "SimpleArrayEditor";

    JList list = null;

    /**
     * Creates a list viewer based on an array of any simple type
     */
    public SimpleArrayEditor(ComboBoxModel model) {
        super(model);
    }

    /**
     * Returns the assotiated data model
     */
    public ComboBoxModel getModel() {
        return (ComboBoxModel) getDataModel();
    }

    /**
     * Creates the JList
     */
    @Override
    public void create() {
        list = new JList(getModel().getValueArray().getListModel());

        JScrollPane scrollPane = new JScrollPane(list);
        scrollPane.setFocusable(false);
        scrollPane.getViewport().setFocusable(false);

        scrollPane.setPreferredSize(new Dimension(200, 100));
        setJComponent(scrollPane);

        list.addListSelectionListener(new MyListSelectionListener());
        update();
    }

    class MyListSelectionListener implements ListSelectionListener {
        @Override
        public void valueChanged(ListSelectionEvent e) {
            int newValue = list.getSelectedIndex();

            if (newValue >= 0) {
                getModel().setInt(newValue);
            }
        }
    }

    /**
     * No specific implementation
     */
    @Override
    public void update() {
        list.setSelectedIndex(getModel().getInt());
    }

    /**
     * Only for debugging
     */
    @Override
    public String getViewerName() {
        return s_viewername;
    }
}
