//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.command.Command;
import com.nexirius.framework.datamodel.StepperModel;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.swing.CommandJButton;
import com.nexirius.framework.swing.paintbutton.OpenArrowLeftButton;
import com.nexirius.framework.swing.paintbutton.OpenArrowRightButton;
import com.nexirius.framework.swing.paintbutton.OpenEndArrowLeftButton;
import com.nexirius.framework.swing.paintbutton.OpenEndArrowRightButton;

import javax.swing.*;
import java.awt.*;


public class StepperEditor extends StructEditor {



    public static final String editorName = "StepperEditor";

    public StepperEditor(StepperModel stepper) {
        super(stepper);
    }

    public StepperModel getStepperModel() {
        return (StepperModel) getModel();
    }

    /**
     * Only for debugging
     */
    public String getViewerName() {
        return editorName;
    }

    /**
     * Creates the actual JPanel and all the children components (depends on the asssociated layout status)
     */
    public void create() {
        if (isCreated()) {

            return;
        }

        JPanel arrayPanel = new JPanel(new GridBagLayout());

        setJComponent(arrayPanel);

        try {
            DataViewer from = getFactory().createDefaultEditor(getStepperModel().getViewableChild(StepperModel.STEPPER_MODEL_FROM));
            DataViewer count = getFactory().createDefaultEditor(getStepperModel().getViewableChild(StepperModel.STEPPER_MODEL_COUNT));
            DataViewer info = getFactory().createDefaultViewer(getStepperModel().getViewableChild(StepperModel.STEPPER_MODEL_INFO));

            CommandJButton beginButton = new OpenEndArrowLeftButton(factory.getClientResource(), (Command) getStepperModel().getViewableChild(StepperModel.BEGIN_STEPPER_COMMAND), factory.getCommandProcessor());
            CommandJButton prevButton = new OpenArrowLeftButton(factory.getClientResource(), (Command) getStepperModel().getViewableChild(StepperModel.DECREMENT_STEPPER_COMMAND), factory.getCommandProcessor());
            CommandJButton nextButton = new OpenArrowRightButton(factory.getClientResource(), (Command) getStepperModel().getViewableChild(StepperModel.INCREMENT_STEPPER_COMMAND), factory.getCommandProcessor());
            CommandJButton endButton = new OpenEndArrowRightButton(factory.getClientResource(), (Command) getStepperModel().getViewableChild(StepperModel.END_STEPPER_COMMAND), factory.getCommandProcessor());

            JTextField fromComp = (JTextField) from.getJComponent();
            JTextField countComp = (JTextField) count.getJComponent();

            fromComp.setPreferredSize(new Dimension(50, 20));
            fromComp.setHorizontalAlignment(JTextField.RIGHT);
            fromComp.setForeground(Color.blue);

            countComp.setPreferredSize(new Dimension(24, 20));
            countComp.setHorizontalAlignment(JTextField.RIGHT);
            countComp.setForeground(Color.blue);

            JComponent infoComp = info.getJComponent();

            infoComp.setForeground(Color.blue);

            int x = 0;

            arrayPanel.add(beginButton, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(prevButton, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(fromComp, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(nextButton, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(endButton, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(infoComp, new GridBagConstraints(x++, 0, 1, 1, 1.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
            arrayPanel.add(countComp, new GridBagConstraints(x++, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void update() {
    }
}
