//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.framework.command.Command;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * This class is the used to define an alias to another command.
 *
 * @author Marcel Baumann
 */
public class AliasDataModelCommand extends DefaultDataModelCommand {



    static String resources[] = {Command.ABORT_AFTER_TIMEOUT, Command.ENABLED, Command.SYNCHRONOUS, Command.TIMEOUT, Command.RESOURCE_KEY};
    Command twin = null;
    PropertyChangeListener propertyChangeListener;

    public AliasDataModelCommand(String name) {
        super(name);
        updateProperties();
    }

    public AliasDataModelCommand(String name, Command twin) {
        this(name);
        setTwin(twin);
    }

    /**
     * return an exact copy
     */
    public DataModelCommand duplicate() {
        return new AliasDataModelCommand(getCommandName(), twin);
    }

    public void setTwin(Command twin) {
        if (this.twin == twin) {
            return;
        }

        if (this.twin != null) {
            this.twin.removePropertyChangeListener(getPropertyChangeListener());
        }

        this.twin = twin;

        if (twin != null) {
            twin.addPropertyChangeListener(getPropertyChangeListener());
        }

        updateProperties();
    }

    public Command getTwin() {
        return twin;
    }

    public void removeTwin() {
        if (twin != null) {
            twin.removePropertyChangeListener(getPropertyChangeListener());
            twin = null;
        }

        updateProperties();
    }

    private PropertyChangeListener getPropertyChangeListener() {
        if (propertyChangeListener == null) {
            propertyChangeListener = new MyPropertyChangeListener();
        }

        return propertyChangeListener;
    }

    /**
     * This function calls the same function on the twin command
     */
    public boolean doAction() {
        boolean ret = true;
        if (twin != null) {
            ret = twin.doAction();
        }
        return(ret);
    }

    /**
     * This function calls the same function on the twin command
     */
    public void exceptionAction(Throwable e) {
        if (twin != null) {
            twin.exceptionAction(e);
        }
    }

    /**
     * This function calls the same function on the twin command
     */
    public void initialiseAction(Object parameter) {
        if (twin != null) {
            twin.initialiseAction(parameter);
        }
    }

    /**
     * This function calls the same function on the twin command
     */
    public void finaliseAction() {
        if (twin != null) {
            twin.finaliseAction();
        }
    }

    /**
     * This function calls the same function on the twin command
     */
    public void timeoutAction() {
        if (twin != null) {
            twin.timeoutAction();
        }
    }

    public void updateProperties() {
        if (twin == null) {
            setEnabled(false);
        } else {

            for (int i = 0; i < resources.length; ++i) {
                String key = resources[i];
                Object val = twin.getValue(key);

                if (val != null) {
                    putValue(key, val);
                }
            }
        }
    }

    class MyPropertyChangeListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            putValue(evt.getPropertyName(), evt.getNewValue());
        }
    }
}
