//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;



public class AskModel extends StructModel {



    public static final String FIELDNAME_QUESTION = "Question";
    public static final String FIELDNAME_ANSWER_YES = "Yes";
    public static final String FIELDNAME_ANSWER_NO = "No";

    public static final int ANSWER_CANCEL2 = -2;
    public static final int ANSWER_CANCEL = -1;
    public static final int ANSWER_NO = 0;
    public static final int ANSWER_YES = 1;

    StringModel question;
    String answerYes;
    String answerNo;
    String answerCancel = "Cancel";
    DataModelCommand yesCommand;
    DataModelCommand noCommand;
    DataModelCommand cancelCommand;
    boolean defaultIsYes = false;
    boolean hasCancel = false;
    IntModel answer = new IntModel(ANSWER_CANCEL2, "Answer");

    public AskModel(String fieldName, String question, String answerYes, String answerNo, boolean defaultIsYes) {
        this.defaultIsYes = defaultIsYes;
        init(question, answerYes, answerNo);
        setFieldName(fieldName);
    }

    private void init(String question, String answerYes, String answerNo) {
        this.question = new StringModel(question, FIELDNAME_QUESTION);
        this.answerYes = answerYes;
        this.answerNo = answerNo;

        append(this.question);

        yesCommand = new YesMethod(answerYes);
        noCommand = new NoMethod(answerNo);
        cancelCommand = new CancelMethod(answerCancel);

        appendMethod(yesCommand);
        appendMethod(noCommand);
        appendMethod(cancelCommand);
    }

    public void setHasCancel(boolean on) {
        hasCancel = on;
    }

    public boolean getHasCancel() {
        return hasCancel;
    }

    public void setAnswerListener(DataModelListener listener) {
        answer.addDataModelListener(listener);
    }

    public StringModel getQuestionModel() {
        return question;
    }

    public DataModelCommand getYesCommand() {
        return yesCommand;
    }

    public DataModelCommand getNoCommand() {
        return noCommand;
    }

    public DataModelCommand getCancelCommand() {
        return cancelCommand;
    }

    public int getAnswer() {
        return answer.getInt();
    }

    public boolean isYes() {
        return getAnswer() == ANSWER_YES;
    }

    public boolean isCancel() {
        return getAnswer() == ANSWER_CANCEL;
    }

    public boolean getDefaultIsYes() {
        return defaultIsYes;
    }

    class YesMethod extends DefaultDataModelCommand {
        YesMethod(String name) {
            super(name);
            setSynchronous(true);
        }

        @Override
        public boolean doAction() {
            answer.setInt(ANSWER_YES);
            return(true);
        }
    }

    class NoMethod extends DefaultDataModelCommand {
        NoMethod(String name) {
            super(name);
            setSynchronous(true);            
        }
        
        @Override
        public boolean doAction() {
            answer.setInt(ANSWER_NO);
            return(true);
        }
    }

    class CancelMethod extends DefaultDataModelCommand {
        CancelMethod(String name) {
            super(name);
        }

        @Override
        public boolean doAction() {
            answer.setInt(ANSWER_CANCEL);
            return(true);
        }
    }
}


