//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.framework.FWLog;
import com.nexirius.util.statemachine.State;
import com.nexirius.util.statemachine.StateMachine;
import com.nexirius.util.statemachine.StateMachineAdaptor;

/**
 * This class is a StructModel which fires grabFocus events based on the states
 * of a state machine. The names of the states in the state machine are related
 * to the field names of the members of this StructModel.
 *
 * @author Marcel Baumann
 */
public class AssistantModel extends StructModel {



    StateMachine stateMachine;

    /**
     * Initialize an assistant model based on the given state machine
     *
     * @param machine The state machine which defined the act model
     * @param name    the field name of this model
     */
    public AssistantModel(StateMachine machine, String name) {
        super(name);
        setStateMachine(machine);
    }

    /**
     * Initialize an assistant model based on the given state machine
     *
     * @param machine The state machine which defined the act model
     */
    public void setStateMachine(StateMachine m) {
        stateMachine = m;

        if (stateMachine != null) {
            stateMachine.addStateMachineListener(new MyStateMachineListener());
        }
    }

    /**
     * Access the current state machine
     *
     * @return
     */
    public StateMachine getStateMachine() {
        return stateMachine;
    }

    /**
     * Access the model (child) which has the same name as the current state of the state machine
     *
     * @return a child with the name (field name9 of the current state or a StringModel("unknown state " + actState)
     */
    public DataModel getActModel() {
        String actState = getStateMachine().getActState().getName();
        DataModel model = null;

        try {
            model = getChild(actState);
        } catch (Exception ex) {
            throw new RuntimeException("unknown state " + actState, ex);
        }

        return model;
    }

    /**
     * Access the current State of the associated state machine
     *
     * @return
     */
    public State getActState() {
        return getStateMachine().getActState();
    }

    /**
     * This class fires grab focus events on the child which has the same name as the current
     * state in the associated state machine. Grab focus is fired just after each time the state
     * machine has changed its state.
     */
    class MyStateMachineListener extends StateMachineAdaptor {
        public void stateMachineAfterTransition(StateMachine m, State prev, String event, State act) {
            FWLog.debug("***" + toString(prev, event, act));
            DataModel model = null;

            try {
                model = getChild(act.getName());
                model.grabFocus();
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
    }
}

