//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.swing.JnexFileChooser;
import com.nexirius.util.MagicNumberException;
import com.nexirius.util.MagicNumberInputStream;
import com.nexirius.util.MagicNumberOutputStream;
import com.nexirius.util.MagicNumberTextFile;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;

public class DataModelFile implements DataModelListener {

    static final int OPEN = 1;
    static final int SAVE = 2;

    public static String openText = "Open";
    public static String saveText = "Save";

    String file_id;
    int format_version;

    String directoryName;
    String fileName;
    FileNameModel fileNameModel;
    String extension;
    MagicNumberTextFile file;
    StringModel magicNumber;
    DataModel model;
    boolean needSaving = false;
    boolean readOnly = false;
    boolean checkMagicNumber = true;
//    JFileChooser chooser = null;

    public static int counter = 0;
    protected JDialog dialog;
    protected int actionOption;

    public DataModelFile(DataModel m) {
        model = m;

        model.addDataModelListener(this);

        magicNumber = new StringModel("", "MagicNumber");
    }

    public void setFileNameModel(FileNameModel fileNameModel) {
        this.fileNameModel = fileNameModel;
    }

    public void setFileName(String fileName) {
        this.fileNameModel.setText(fileName);
    }

    public void setButtonTexts(String open, String save) {
        openText = open;
        saveText = save;
    }

    /**
     * by default magic number exceptions result in an error.
     * When this method is called with false then the magic number exception is ignored
     *
     * @param check (default is true)
     */
    public void setCheckMagicNumber(boolean check) {
        checkMagicNumber = check;
    }

    public String getFileName() {
        if (file == null) {

            return "";
        }

        return file.getAbsolutePath();
    }

    public Component getParentComponent() {
        return DialogManager.getToplevelFrame();
    }

    public void setExtension(String e) {
        extension = e;
    }

    public void setType(String file_id, int format_version) {
        this.file_id = file_id;
        this.format_version = format_version;
    }

    public void setDirectory(String d) {
        directoryName = d;

        if (d != null) {
            File dir = new File(d);

            if (!dir.exists()) {
                dir.mkdirs();
            }
        }
    }

    public void setReadOnly(boolean on) {
        readOnly = on;

        if (readOnly) {
            needSaving = false;
        }
    }

    public void setNeedSaving(boolean on) {
        if (!readOnly) {
            needSaving = on;
        }
    }

    public boolean getNeedSaving() {
        return needSaving;
    }

    public void setMagicNumber(long v) {
        if (v == 0) {
            magicNumber.setText("");
        } else {
            magicNumber.setText(new Long(v).toString());
        }
    }

    public StringModel getMagicNumberModel() {
        return magicNumber;
    }

    public long getMagicNumber() {
        if (file != null) {
            return file.getMagicNumber();
        }

        return 0;
    }

    public void updateMagicNumber() {
        if (getNeedSaving()) {
            setMagicNumber(0);
        } else {
            setMagicNumber(getMagicNumber());
        }
    }

    public void clear() {
        model.clear();
        setNeedSaving(false);
        setReadOnly(false);
    }

    public synchronized boolean newCommand() {
        if (getNeedSaving()) {
            if (!DialogManager.ask("DatenUeberschreiben", "OK", "Cancel", false)) {

                return false;
            }
        }

        clear();

        fileName = null;

        if (fileNameModel != null) {
            fileNameModel.setText("");
        }

        file = null;

        return true;
    }

    public void save()
            throws Exception {
        save(fileName != null);
    }

    public void save(boolean exists)
            throws Exception {
        if (readOnly) {
            DialogManager.error("KannReadOnlyNichtSpeichern");

            return;
        }

        if (model == null) {

            return;
        }

        if (file == null) {
            saveAs();

            return;
        }

        if (exists) {
            file.openExistingFileForWrite(file_id, format_version);
        } else {
            file.openNewFileForWrite(file_id, format_version);
        }

        MagicNumberOutputStream out = file.getOut();

        showProgress(out);

        try {
            model.writeDataTo(out);
        } catch (Exception ex) {
            DialogManager.error("KannNichtSpeichern");
        }

        file.close();

        setNeedSaving(false);

        updateMagicNumber();

        hideProgress();
    }

    public void saveAs()
            throws Exception {
        if (readOnly) {
            DialogManager.error("KannReadOnlyNichtSpeichern");

            return;
        }

        fileName = generateFileName();

        if (fileNameModel != null) {
            fileNameModel.setText(fileName);
        }

        File newFile = null;

        newFile = JnexFileChooser.getInstance(directoryName).popupSaveDialog(fileName);
//        initChooser(SAVE, fileName);
//        dialog.setVisible(true);
//        int retval = actionOption;
//
//        if (retval == JFileChooser.APPROVE_OPTION) {
//            newFile = chooser.getSelectedFile();
//
//            if (newFile == null) {
//                DialogManager.error("Bitte geben Sie beim speichern einen Dateinamen an");
//
//                return;
//            }
//        } else {
//
//            return;
//        }

        if (newFile == null) {
            return;
        }

        String dir = "";
        boolean exists = false;

        if (newFile.exists()) {

            if (file == null || !newFile.getPath().equals(file.getPath())) {

                if (!DialogManager.ask("BestehendeDateiUeberschreiben", "OK", "Cancel", false)) {

                    return;
                }
            }

            if (newFile.isFile() && newFile.canWrite()) {
                dir = getDir(newFile);

                exists = true;

            } else {
                Object p[] = new Object[1];
                p[0] = newFile;

                DialogManager.error("KannNichtSchreiben{0}", p);

                return;
            }

        } else {
            // check whether the directory exists and is writable

            dir = getDir(newFile);

            if (dir.length() > 0) {
                File directory = new File(dir);

                if (!directory.exists() || !directory.canWrite() || !directory.isDirectory()) {
                    Object p[] = new Object[1];
                    p[0] = dir;

                    DialogManager.error("KannNichtSchreibenVerzeichnis{0}", p);

                    return;
                }
            }
        }

        // append extension

        String newName = newFile.getPath();

        if (!exists && extension != null && !newName.endsWith(extension)) {
            newName = newName + extension;
        }

        fileName = newName;

        if (fileNameModel != null) {
            fileNameModel.setText(fileName);
        }

        directoryName = dir;

        file = new MagicNumberTextFile(fileName);

        save(exists);
    }

    String getDir(File f) {
        // calculate dir
        String name = f.getAbsolutePath();

        int index = name.lastIndexOf(File.separatorChar);

        if (index > 1) {

            return name.substring(0, index);
        }

        return "";
    }

//    public void initChooser(int type, String filename) {
//        actionOption = JFileChooser.CANCEL_OPTION;
//
//        if (chooser == null) {
//            if (directoryName == null) {
//                directoryName = File.separator;	// fix
//            }
//
//            chooser = new JFileChooser(new File(directoryName));
//            chooser.addActionListener(new ActionListener() {
//                public void actionPerformed(ActionEvent e) {
//                    System.out.println("e.getActionCommand() = " + e.getActionCommand());
//
//                    if (e.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)) {
//                        actionOption = JFileChooser.APPROVE_OPTION;
//                    }
//                    dialog.setVisible(false);
//                }
//            });
//
//        } else {
//            DialogManager.setDefaultCursor(chooser);
//            chooser.rescanCurrentDirectory();
//        }
//
//        if (dialog != null) {
//            dialog.getContentPane().removeAll();
//            dialog.dispose();
//        }
//
//        dialog = DialogManager.createJDialog("", true);
//        dialog.getContentPane().setLayout(new BorderLayout());
//        dialog.getContentPane().add(chooser);
//
//        dialog.pack();
//
//        DialogManager.center(dialog, false);
//
//        String appr;
//
//        switch (type) {
//            case SAVE:
//                appr = saveText;
//                break;
//
//            case OPEN:
//                appr = openText;
//                break;
//
//            default:
//                appr = "ERROR";
//        }
//
//        chooser.setApproveButtonText(appr);
//
//        if (file != null) {
//            chooser.setSelectedFile(file);
//        } else if (filename != null) {
//            chooser.setSelectedFile(new File(chooser.getCurrentDirectory().getAbsolutePath(), filename));
//        }
//    }

    public boolean open()
            throws Exception {

        File theFile = JnexFileChooser.getInstance(directoryName).popupOpenDialog(fileName);

        if (theFile != null) {

            return open(theFile.getAbsolutePath());
        }

        return false;
    }

    public boolean open(String f)
            throws Exception {
        MagicNumberTextFile oldFile = file;
        String oldFileName = fileName;
        String oldDirectoryName = directoryName;

        file = new MagicNumberTextFile(f);
        fileName = file.getAbsolutePath();

        if (fileNameModel != null) {
            fileNameModel.setText(fileName);
        }

        directoryName = getDir(file);

        try {
            boolean ret = reOpen();

            if (!ret) {
                file = oldFile;
                fileName = oldFileName;

                if (fileNameModel != null) {
                    fileNameModel.setText(fileName);
                }

                directoryName = oldDirectoryName;
            }

            return ret;
        } catch (Exception ex) {
            file = null;
            fileName = null;

            if (fileNameModel != null) {
                fileNameModel.setText("");
            }

            throw ex;
        }
    }

    public void reset() {
        file = null;
        fileName = null;

        if (fileNameModel != null) {
            fileNameModel.setText("");
        }
    }

    public boolean reOpen()
            throws Exception {

        if (getNeedSaving()) {
            if (!DialogManager.ask("EingabenUeberschreiben", "OK", "Cancel", false)) {

                return false;
            }
        }

        setReadOnly(false);

        boolean ret = readFileDataIntoModel();

        setNeedSaving(false);

        updateMagicNumber();

        hideProgress();

        return ret;
    }

    public boolean readFileDataIntoModel() throws IOException {
        boolean ret = false;

        file.openForRead(file_id);

        checkFormatVersion(file.getFileVersion());

        MagicNumberInputStream in = file.getIn();

        showProgress(in);

        try {
            model.readDataFrom(in);

            ret = true;

            while (in.read() >= 0)
                ;
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        try {
            file.close();
        } catch (MagicNumberException ex) {
            if (checkMagicNumber) {
                throw ex;
            }
        }

        return ret;
    }

    public boolean checkFormatVersion(int version) {
        if (version != format_version) {
            // error
        }

        return true;
    }

    public void prepareExit()
            throws Exception {
        if (getNeedSaving()) {
            save();
        }
    }

    public String generateFileName() {
        return getModelName() + (++counter) + (extension == null ? "" : extension);
    }

    public String getModelName() {
        return model.getFieldName();
    }

    public void showProgress(Object stream) {
//        double progress = 0.0;
//        DialogManager.setWaitCursor(chooser);
//        DialogManager.setWaitCursor(getParentComponent());
    }

    public void hideProgress() {
//        DialogManager.setDefaultCursor(chooser);
//        DialogManager.setDefaultCursor(getParentComponent());
    }

    public void dataModelChangeValue(DataModelEvent e) {
        setNeedSaving(true);
//new Exception().printStackTrace();
//System.out.println(e);
//		updateMagicNumber();
    }

    @Override
    public void dataModelWantToChangeValue(VetoableDataModelEvent event)
    {
    }
    public void dataModelChangeStructure(DataModelEvent e) {
        setNeedSaving(true);
    }

    public void dataModelGrabFocus(DataModelEvent event) {
    }

    public static void main(String argv[])
            throws Exception {
        javax.swing.JFrame f = new javax.swing.JFrame();
        DataModel model = new StringModel("Hallo", "M");
        DataModelFile mfile = new DataModelFile(model);
        mfile.setType("Test", 0);
        mfile.setDirectory("C:\\temp");

        javax.swing.JButton b1 = new javax.swing.JButton("save as");
        javax.swing.JButton b2 = new javax.swing.JButton("save");
        javax.swing.JButton b3 = new javax.swing.JButton("open");

        b1.addActionListener(new SaveAsListener(mfile));
        b2.addActionListener(new SaveListener(mfile));
        b3.addActionListener(new OpenListener(mfile));

        f.getContentPane().setLayout(new javax.swing.BoxLayout(f.getContentPane(), javax.swing.BoxLayout.Y_AXIS));
        f.getContentPane().add(b1);
        f.getContentPane().add(b2);
        f.getContentPane().add(b3);

        f.pack();
        f.setVisible(true);
    }
}

class SaveAsListener implements java.awt.event.ActionListener {
    DataModelFile mfile;

    public SaveAsListener(DataModelFile f) {
        mfile = f;
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
        try {
            mfile.saveAs();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }
}

class SaveListener implements java.awt.event.ActionListener {
    DataModelFile mfile;

    public SaveListener(DataModelFile f) {
        mfile = f;
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
        try {
            mfile.save();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }
}

class OpenListener implements java.awt.event.ActionListener {
    DataModelFile mfile;

    public OpenListener(DataModelFile f) {
        mfile = f;
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
        try {
            mfile.open();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }
}
