//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import javax.swing.tree.*;

/**
 * A class which helps to display a hierarchical data model in a swing tree
 *
 * @author Marcel Baumann
 */
public class DataModelTree extends DefaultTreeModel implements SynchronizedDataModelListener {

    private static final long serialVersionUID = 5962989270911985182L;
    
    TreeSelectionModel selectionModel;

    public DataModelTree(DataModel model) {
        this(model, new DefaultTreeSelectionModel());
    }

    public DataModelTree(DataModel model, TreeSelectionModel selection_model) {
        super(model.getTreeHook());

        if (model == null || selection_model == null) {
            throw new IllegalArgumentException("model and selection_model must both not be null");
        }

        this.selectionModel = selection_model;

        model.addSoftDataModelListener(this);
    }

    public DataModel getDataModel() {
        return ((TreeHook) getRoot()).getDataModel();
    }

    public TreeSelectionModel getTreeSelectionModel() {
        return this.selectionModel;
    }

    Object[] calculatePath(DataModelEvent event) {
        TreePath treepath = event.getTreePath();
        Object path[] = treepath.getPath();
        int root_index = -1;

        // find the root in the path
        for (int i = 0; i < path.length; ++i) {

            if (path[i] == getRoot()) {
                root_index = i;

                break;
            }
        }

        if (root_index >= 0) {
            Object actual_path[] = new Object[path.length - root_index];

            for (int i = root_index; i < path.length; ++i) {
                actual_path[i - root_index] = path[i];
            }

            return actual_path;
        }
        Object ret[] = {getRoot()};

//for (int j = 0 ; j < path.length; ++j) { System.out.println(j + " " + ((TreeHook)path[j]).getDataModel().getFieldName());}

        return ret;
    }

    public void dataModelChangeValue(DataModelEvent event) {
        if (event.fromChild()) {
            TreeNode source = event.getDataModel().getTreeHook();
            Object path[] = calculatePath(event);
            int childIndices[] = null;
            Object children[] = null;

            fireTreeStructureChanged(source, path, childIndices, children);
        } else if (event.getSource() instanceof ArrayModel) {
            dataModelChangeStructure(event);
        }
    }

    public void dataModelChangeStructure(DataModelEvent event) {
        reload(event.getDataModel().getTreeHook());
    }

 @Override
    public void dataModelWantToChangeValue(VetoableDataModelEvent event)
    {
    }
 
 public void dataModelGrabFocus(DataModelEvent event) {
    }

    public void dataModelEdit(DataModelEvent event) {
    }
}
