//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

import java.io.File;

/**
 * A class that represents a directory name.
 *
 * @author Marcel Baumann
 */
public class DirNameModel extends FileNameModel {




    public DirNameModel() {
        this("");
    }

    /**
     * @param value The initial value.
     */
    public DirNameModel(String value) {
        super(value);
    }

    /**
     * @param value     The initial value.
     * @param fieldName The initial field name
     */
    public DirNameModel(String value, String fieldName) {
        super(value, fieldName);
    }

    /**
     * Creates an exact copy of the dir name model
     */
    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new DirNameModel(getText());
        }

        return super.duplicate(instance, copyPairs);
    }

    /**
     * appends a File.separator to the end unless the text is empty
     */
    public void setText(String text) {
        if (text.length() > 0) {
            int sepIndex = text.lastIndexOf(File.separatorChar);

            // check whether last character is no separator
            if (sepIndex >= 0 && sepIndex < text.length() - 1) {
                text = text + File.separatorChar;
            }
        }

        super.setText(text);
    }
}
