//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

import javax.swing.text.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

public class DocumentModel extends DataModel {

    public DocumentModel() {
        this(new DefaultStyledDocument(), "");
    }

    public DocumentModel(String fieldName) {
        this(new DefaultStyledDocument(), fieldName);
    }

    public DocumentModel(Document document, String fieldName) {
        super(document, fieldName);
        document.addDocumentListener(new LocalDocumentListener());
    }

    public Document getDocument()
    {
        return (Document)getValue();
    }

    public void setText(String text)
    {
        try {
            getDocument().remove(0, getDocument().getLength());
        } catch (BadLocationException e) {
            throw new RuntimeException(e);  //TODO
        }
        appendText(text);
    }

    public void appendText(String text)
    {
        try {
            getDocument().insertString(getDocument().getLength(), text, null);
        } catch (BadLocationException e) {
            throw new RuntimeException(e);  //TODO
        }
    }

    public void clear() {
        try {
            getDocument().remove(0, getDocument().getLength());
        } catch (BadLocationException e) {
            throw new RuntimeException(e);  //TODO
        }
    }

    public String getCaption() {
        return "";
    }

    public Object cloneValue(CopyPairs copyPairs) {
        // TODO
        return getDocument();
    }

    class LocalDocumentListener implements DocumentListener {
        public void insertUpdate(DocumentEvent e) {
            fireValueChange(DocumentModel.this);
        }

        public void removeUpdate(DocumentEvent e) {
            fireValueChange(DocumentModel.this);
        }

        public void changedUpdate(DocumentEvent e) {
            fireValueChange(DocumentModel.this);
        }

    }
}

