//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;



/**
 * This interface is used for the definition of simple field names as well as for
 * hierarchical (tree) lists of all the field names from a container.
 *
 * @author Marcel Baumann
 */
public interface FieldName {
    /**
     * Returs a string representation
     */
    public String toString();

    /**
     * Returns the parent field name if this element is part of a hierarchical field name.
     */
    public FieldName getParent();

    /**
     * Returns true if this is a hierarchical field name
     */
    public boolean hasChildren();

    /**
     * If this is a hierarchical field name then a list containig all direct children is returned.
     */
    public FieldNameVector getChildren();

    /**
     * Returns all the fieldnames up to the to parent field name separated by the given separator.
     *
     * @param separator The string which is put between the field names (e.g. ";")
     */
    public String getFullName(String separator);

    /**
     * Set the parent field name
     *
     * @param fn The parent field name
     */
    public void setParent(FieldName fn);

    /**
     * Appends all the leaves in the field name tree recursively to the given vector
     *
     * @param v The list which the children field names are appended to.
     */
    public void getFieldNameVector(FieldNameVector v);

    /**
     * Recursively traverses the hierarchical field name tree and returns the number of leaves.
     */
    public int getCount();
}
