//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.SortedVector;

import java.util.Enumeration;

/**
 * This class holds a list a of FieldName references.
 *
 * @author Marcel Baumann
 */
public class FieldNameVector extends SortedVector {



    /**
     * Adds an status to the end of the list.
     */
    public void append(FieldName fn) {
        super.addElement(fn);
    }

    /**
     * Returns an element at the specified index.
     */
    public FieldName getFieldName(int index) {
        return (FieldName) getObject(index);
    }

    /**
     * Appends all the leaves in the field name tree recursively to the given vector
     */
    public void getFieldNameVector(FieldNameVector v) {
        Enumeration e = elements();

        while (e.hasMoreElements()) {
            ((FieldName) e.nextElement()).getFieldNameVector(v);
        }
    }

    /**
     * Recursively traverses the hierarchical field name tree and returns the number of leaves.
     */
    public int getCount() {
        int ret = 0;
        Enumeration e = elements();

        while (e.hasMoreElements()) {
            ret += ((FieldName) e.nextElement()).getCount();
        }

        return ret;
    }
}
