//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;



/**
 * A class that holds a hierarchical field name (tree).
 *
 * @author Marcel Baumann
 */
public class FullFieldName extends BasicFieldName {



    FieldNameVector children = new FieldNameVector();

    /**
     * Creates a new field name (with no children)
     */
    public FullFieldName(String name) {
        super(name);
    }

    /**
     * Returns wheter this field name has one or more children
     */
    public boolean hasChildren() {
        return this.children.size() > 0;
    }

    /**
     * Returns the children vector.
     *
     * @return Never returns null
     */
    public FieldNameVector getChildren() {
        return this.children;
    }

    /**
     * Returns all the fieldnames up to the to parent field name separated by the given separator.
     *
     * @param separator The string which is put between the field names (e.g. ";")
     */
    public String getFullName(String separator) {
        String ret = toString();

        if (getParent() != null) {
            ret = getParent().getFullName(separator) + separator + ret;
        }

        return ret;
    }

    /**
     * Appends a child field name to this field name.
     *
     * @param fn The child which is appeded (the child gets a new parent)
     */
    public void append(FieldName fn) {
        fn.setParent(this);
        this.children.append(fn);
    }

    /**
     * Appends all the leaves in the field name tree recursively to the given vector
     *
     * @param v The list which the children field names are appended to.
     */
    public void getFieldNameVector(FieldNameVector v) {
        if (hasChildren()) {
            this.children.getFieldNameVector(v);
        } else {
            super.getFieldNameVector(v);
        }
    }

    /**
     * Recursively traverses the hierarchical field name tree and returns the number of leaves.
     */
    public int getCount() {
        return this.children.getCount();
    }
}
