package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

/**
 * A class which holds an instance name which can be selected from an ArrayModel
 * the getCaption() method of the array elements is used as the text in the combo box
 */
public class InstanceNameComboBoxModel extends StringModel {
    private ArrayModel selectorArray;
    private ArrayListener arrayListener = new ArrayListener();

    public InstanceNameComboBoxModel(String value, ArrayModel selectorArray, String fieldName) {
        super(value, fieldName);
        if (value == null || value.length() == 0) {
            setFlag(ModelFlag.EMPTY, true);
        }
        this.selectorArray = selectorArray;
        selectorArray.addSoftDataModelListener(arrayListener);
    }

    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new InstanceNameComboBoxModel(getText(), selectorArray, getFieldName());
        }

        return super.duplicate(instance, copyPairs);
    }

    public DataModel getSelectedDataModel() {
        DataModelEnumeration en = selectorArray.getEnumeration();
        String text = getText();

        while (en.hasMore()) {
            DataModel dataModel = en.next();

            if (text.equals(dataModel.getInstanceName())) {
                return dataModel;
            }
        }

        return null;
    }

    public String getCaption() {
        DataModel selectedDataModel = getSelectedDataModel();
        return selectedDataModel == null ? "" : selectedDataModel.getCaption();
    }

    public String toString() {
        DataModel selectedDataModel = getSelectedDataModel();
        return selectedDataModel == null ? "" : selectedDataModel.toString();
    }

    public ArrayModel getArrayModel() {
        return selectorArray;
    }

    class ArrayListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            fireValueChange(InstanceNameComboBoxModel.this);
        }
    }
}
