//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

/**
 * A specific runtime exception which is thrown when a DataModel is
 * finishing its edit phase and the new value is invalid and allowInvalid is false.
 *
 * @author Marcel Baumann
 */
public class InvalidValueException extends RuntimeException {



    Object value = null;
    DataModel source = null;

    /**
     * Create invalid value exception.
     *
     * @param value The invalid value.
     */
    public InvalidValueException(Object value) {
        super();
        this.value = value;
    }

    /**
     * Create an exception with message and value
     *
     * @param message Reason
     * @param value   The invalid value.
     */
    public InvalidValueException(String message, Object value) {
        super(message);
        this.value = value;
    }

    /**
     * Create an exception with message and value and source
     *
     * @param message Reason
     * @param value   The invalid value.
     */
    public InvalidValueException(String message, Object value, DataModel source) {
        super(message);
        this.value = value;
        this.source = source;
    }

    /**
     * Returns the invalid value which caused the exception
     */
    public Object getValue() {
        return this.value;
    }

    /**
     * Returns the data model which trew exception
     */
    public DataModel getSource() {
        return this.source;
    }
}
