//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.simpletype.SimpleType_long;

/**
 * A class that represents a long information which can be displayed in a text field.
 *
 * @author Marcel Baumann
 */
public class LongModel extends SimpleModel {




    public LongModel() {
        this(0L);
    }

    /**
     * Creates a new instance and initializes its value
     *
     * @param value The initial value
     */
    public LongModel(long value) {
        super(new SimpleType_long(value));
    }

    /**
     * Creates a new instance and initializes its value and field name
     *
     * @param value     The initial value
     * @param fieldName The initial field name
     */
    public LongModel(long value, String fieldName) {
        super(new SimpleType_long(value), fieldName);
    }

    /**
     * Returns the current value as int
     */
    public long getLong() {
        return getSimpleType().getLong();
    }

    /**
     * Set the new value as long
     * 
     * @param i
     *            The new value
     * @return true wenn der Wert übernommen wurde; false wenn irgend jemand sein Veto einlegte.
     */
    public boolean setLong(long i)
    {
        SimpleType_long newValue = new SimpleType_long(i);
        
        boolean ret = true;
        if (newValue.equals(value) == false) 
        {
            // Mal schauen ob jemand das Veto gegen die gewünschte Änderung einlegt!
            ret = fireWantToChangeValue(newValue);
        }
        
        if (ret)
        {
            setEmpty(false);
            setValue(newValue);
        }
        return (ret);
    }

    /**
     * Set the new value as string
     *
     * @param text The string which is parsed to produce the new integer value
     * @throws Exception If the text cannot be parsed into an integer.
     */
    public void setText(String text)
            throws Exception {
        if (text.length() == 0) {
            if (!isEmpty()) {
                value = new SimpleType_long(0L);
                setEmpty();
            }
        } else {
            if (isEmpty()) {
                setEmpty(false);
            }
            setValue(new SimpleType_long(text));
        }
    }

    /**
     * Returns an exact copy of this instance
     *
     * @param instance  null (only set by subclasses dupicate method)
     * @param copyPairs null (only needed with container classes)
     */
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs) {
        if (instance == null) {
            instance = new LongModel(getLong());
        }

        return super.duplicate(instance, copyPairs);
    }
}
