//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

/**
 * This class combines a ModelFlag with a boolean status.
 *
 * @author Marcel Baumann
 */
public final class ModelFlagContainer extends Object {
    ModelFlag flag;
    Boolean status;

    /**
     * Creates a container which combines a flag with a boolean status
     *
     * @param flag   The flag (identifier)
     * @param status The initial state of the flag
     */
    public ModelFlagContainer(ModelFlag flag, boolean status) {
        this.flag = flag;
        this.status = new Boolean(status);
    }

    /**
     * Creates a container which combines a flag with a boolean status
     *
     * @param flag   The flag (identifier)
     * @param status The initial state of the flag
     */
    public ModelFlagContainer(ModelFlag flag, Boolean status) {
        if (flag == null || status == null) {
            throw new IllegalArgumentException("flag and status must be != null");
        }

        this.flag = flag;
        this.status = status;
    }

    /**
     * Returns the flag
     */
    public ModelFlag getFlag() {
        return this.flag;
    }

    /**
     * Returns th current status as Boolean
     */
    public Boolean getStatus() {
        return this.status;
    }

    /**
     * Two containers are equal if the assotiated flag is the same
     */
    public boolean equals(Object o) {
        if (o == null || !(o instanceof ModelFlagContainer)) {
            return false;
        }

        ModelFlagContainer fc = (ModelFlagContainer) o;

        return fc.getFlag().equals(getFlag());
    }

    /**
     * Returns the flags generateFileInfo code
     */
    public int hashCode() {
        return this.flag.hashCode();
    }

    /**
     * Returns the flag name and status (for debugging purpses)
     */
    public String toString() {
        return flag + "(" + status + ")";
    }
}
