//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.resource.ClientResource;
import com.nexirius.util.resource.ResourceChangeEvent;
import com.nexirius.util.resource.ResourceChangeListener;

/**
 * A class which helps to define a LimitIntModel or a ComboboxModel or a BooleanModel to display
 * language dependend list entries.
 * <PRE>
 * String labels[] = {"male", "female"};
 * MultiLanguageLabels genderLabels = new MultiLanguageLabels(clientResource, labels);
 * boolean MALE = false;
 * boolean FEMALE = true;
 * BooleanModel gender = new BooleanModel(MALE, "gender");
 * gender.setTrueFalseLabelModel(genderLabels);
 * </PRE>
 *
 * @author Marcel Baumann
 */
public class MultiLanguageLabels extends SimpleArrayModel implements ResourceChangeListener {



    String labels[];
    ClientResource resource;

    /**
     * Creates a new model which changes its value on resource change events.
     *
     * @param The    resource where this model gets listener
     * @param labels The array which defines the resource keys for this array.
     */
    public MultiLanguageLabels(ClientResource resource, String labels[]) {
        super((Object[])labels);
        this.resource = resource;
        this.labels = labels;
        newLabels();
        this.resource.addResourceChangeListener(this);
    }

    /**
     * Set a new basis array for the labels.
     *
     * @param labels The array which defines the resource keys for this array.
     */
    public void setLabels(String labels[]) {
        this.labels = labels;
        newLabels();
    }

    /**
     * Implementation of resource change behaviour
     */
    @Override
    public void resourceChange(ResourceChangeEvent event) {
        newLabels();
    }

    /**
     * When the labels array has changed it gets translated into the current language.
     * Labels which are not defined in the resources are showing the resource key.
     */
    public void newLabels() {
        String new_labels[] = new String[labels.length];

        for (int i = 0; i < labels.length; ++i) {
            new_labels[i] = resource.getLabel(labels[i]);
        }

        setArray((Object[])new_labels);
    }
}
