//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

/**
 * A class that represents a password.
 *
 * @author Marcel Baumann
 */
public class PasswordModel extends StringModel {



    protected char echoChar = '*';
    protected PasswordValidator validator; // only validates the format (length and ev. illegal characters)

    public PasswordModel() {
        this("", null);
    }

    /**
     * @param value     The initial value.
     * @param validator The initial validator (only validates the format)
     */
    public PasswordModel(String value, PasswordValidator validator) {
        super(value);
        setPasswordValidator(validator);
    }

    /**
     * @param value     The initial value.
     * @param validator The initial validator (only validates the format)
     * @param fieldName The initial field name
     */
    public PasswordModel(String value, PasswordValidator validator, String fieldName) {
        super(value, fieldName);
        setPasswordValidator(validator);
    }

    /**
     * Set a validator.
     *
     * @param validator The validator (only validates the format)
     */
    public void setPasswordValidator(PasswordValidator validator) {
//		Assert.pre(validator != null, "PasswordValidator is not null");
        this.validator = validator;
    }

    /**
     * Returns the string representation which is displayed in the GUI '*****'
     *
     * @see #getPassword
     */
    public String getText() {
        StringBuffer b = new StringBuffer();

        for (int i = 0; i < super.getText().length(); ++i) {
            b.append(echoChar);
        }

        return b.toString();
    }

    /**
     * Returns the actual password
     */
    public String getPassword() {
        return super.getText();
    }

    /**
     * Returns the character which is displayed instead of password letters
     */
    public char getEchoChar() {
        return echoChar;
    }

    /**
     * Sets the character which is displayed instead of password letters.
     * The default echoChar is '*'.
     */
    public void setEchoChar(char echoChar) {
        this.echoChar = echoChar;
    }

    /**
     * Sets the password to a new value
     */
    public void setText(String text) {
        if (validator == null) {
            super.setText(text);
        } else {
            if (validator.validate(text)) {
                super.setText(text);
            } else {
                throw new RuntimeException(validator.getValidationError(text));
            }
        }
    }

    /**
     * Creates an exact copy of the password model
     */
    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new PasswordModel(getText(), validator);
        }

        ((PasswordModel) instance).setPasswordValidator(validator);
        ((PasswordModel) instance).echoChar = echoChar;

        return super.duplicate(instance, copyPairs);
    }
}
