//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.DoubleRounder;

/**
 * A class that represents a double information which displays its value after rounding it.
 * The actual value of the double is not (!) modified by the rounding (it is only the textual output which is affected).
 *
 * @author Marcel Baumann
 */
public class RoundedDoubleModel extends DoubleModel {



    DoubleRounder rounder = null;

    /**
     * @param value   The initial value
     * @param rounder The double rounder (or null)
     */
    public RoundedDoubleModel(double value, DoubleRounder rounder) {
        super(value);
        this.rounder = rounder;
    }

    public RoundedDoubleModel(double value, DoubleRounder rounder, String fieldName) {
        super(value, fieldName);
        this.rounder = rounder;
    }

    public void setRounder(DoubleRounder rounder) {
        this.rounder = rounder;
        fireValueChange();
    }

    public String getText() {
        if (rounder == null || isEmpty()) {
            return super.getText();
        }
        if (format == null || isExceptional()) {
            return Double.toString(rounder.round(getDouble()));
        }

        return format.format(rounder.round(getDouble())).toString();
    }

    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new RoundedDoubleModel(getDouble(), rounder);
        }

        ((RoundedDoubleModel) instance).setRounder(rounder);

        return super.duplicate(instance, copyPairs);
    }
}
