package com.nexirius.framework.datamodel;

/**
 * Eine Erweiterung des DataModelEvents welche v.a. VOR geplanten Änderungen an einem Model
 * verschickt wird.
 * DIe Listener erhalten den geplanen neuen Wert und haben die Möglichkeit ihr Veto gegen die
 * Änderung einzulegen.
 */
public class VetoableDataModelEvent extends DataModelEvent
{
    private static final long serialVersionUID = -9036947483488113683L;
    
    protected boolean veto = false;
    protected String vetoCause = null;
    protected Object newValue;
    
    public VetoableDataModelEvent(DataModel source, Object newValue)
    {
        super(source, WANT_TO_CHANGE);
        this.newValue = newValue;
    }

    public boolean isVeto()
    {
        return veto;
    }

    public void setVeto(boolean veto)
    {
        this.veto = veto;
    }
    
    public String getVetoCause()
    {
        return vetoCause;
    }

    public void setVetoCause(String vetoCause)
    {
        this.vetoCause = vetoCause;
    }

    public Object getNewValue()
    {
        return newValue;
    }

    /**
     * Liefert true wenn dieses Event direkt und unmittelbar an alle Listener gesendet werden muss.
     * @return true
     */
    @Override
    public boolean needImmediateCalls()
    {
        return(true);
    }

}
