package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.BigDecimalModel;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.StringModel;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;
import java.math.BigDecimal;

public class BigDecimalJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        BigDecimalModel bigDecimalModel = (BigDecimalModel) target;
        TextToken token = TextToken.nextToken(in);

        if (token.isString()) {
            try {
                bigDecimalModel.setBigDecimal(new BigDecimal(token.getString()));
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        } else if (token.isIdentifier("null")) {
            bigDecimalModel.setEmpty();
        } else {
            throw new JsonParseException(token, "string literal (BigDecimal)", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        BigDecimalModel bigDecimalModel = (BigDecimalModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(TextToken.quoteString(bigDecimalModel.getBigDecimal().toPlainString()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}