package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.DateModel;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;
import java.util.Date;

public class DateJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        DateModel dateModel = (DateModel)target;
        TextToken token = TextToken.nextToken(in);

        if (token.isInt()) {
            dateModel.setDate(new Date(token.getLong()));
        } else if (token.isIdentifier("null")) {
            dateModel.setEmpty();
        } else {
            throw new JsonParseException(token, "date value", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        DateModel dateModel = (DateModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(Long.toString(dateModel.getDate().getTime()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}