package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DoubleModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class DoubleJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        DoubleModel doubleModel = (DoubleModel)target;
        TextToken token = TextToken.nextToken(in);

        if (token.isDouble()) {
            doubleModel.setDouble(token.getDouble());
        } else if (token.isInt() || token.isLong()) {
            doubleModel.setDouble(token.getLong());
        } else if (token.isIdentifier("null")) {
            doubleModel.setEmpty();
        } else {
            throw new JsonParseException(token, "double value", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        DoubleModel doubleModel = (DoubleModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(Double.toString(doubleModel.getDouble()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}