package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.BooleanModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.LongModel;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class LongJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        LongModel longModel = (LongModel)target;
        TextToken token = TextToken.nextToken(in);

        if (token.isInt() || token.isLong()) {
            longModel.setLong(token.getLong());
        } else if (token.isIdentifier("null")) {
            longModel.setEmpty();
        } else {
            throw new JsonParseException(token, "long value", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        LongModel longModel = (LongModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(Long.toString(longModel.getLong()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}