//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel.parser;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DefaultDataModelCommand;
import com.nexirius.framework.datamodel.StructModel;

import java.util.Enumeration;
import java.util.Vector;

/**
 * A class which represents a DataModel class
 *
 * @author Marcel Baumann
 */
public class DataModelTypeClass extends DataModelType {



    String implementedBy;
    Vector members;

    public DataModelTypeClass(String name) {
        super(name);
        members = new Vector();
    }

    public void setImplementedBy(String c) {
        implementedBy = c;
    }

    public DataModel newInstance(DataModelClasses classes, String init)
            throws ParserException {
        if (implementedBy == null) {
            return createDefaultInstance(classes, init);
        } else {
            try {
                StructModel ret = (StructModel) Class.forName(implementedBy).newInstance();

                initDefaultInstance(classes, ret, init);

                return ret;
            } catch (ClassNotFoundException ex) {
                throw new ParserException("Class " + implementedBy + " not found in CLASSPATH");
            } catch (IllegalAccessException ex1) {
                throw new ParserException("Class constructor of " + implementedBy + " not found in CLASSPATH");
            } catch (InstantiationException ex2) {
                throw new ParserException("Class constructor failed of " + implementedBy + " not found in CLASSPATH");
            }
        }
    }

    public void initDefaultInstance(DataModelClasses classes, StructModel cl, String init)
            throws ParserException {
        Enumeration e = members.elements();

        while (e.hasMoreElements()) {
            DataModelType t = (DataModelType) e.nextElement();

            if (t instanceof DataModelTypeMethod) {
                DefaultDataModelCommand meth = new DefaultDataModelCommand(t.getMemberName());

                cl.appendMethod(meth);
            } else {
                DataModel instance = t.newInstance(classes);

                instance.setFieldName(t.getMemberName());

                cl.append(instance);
            }
        }

        if (init != null) {
            try {
                cl.dropData(init);
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    public StructModel createDefaultInstance(DataModelClasses classes, String init)
            throws ParserException {
        StructModel ret = new StructModel(getName());

        initDefaultInstance(classes, ret, init);

        return ret;
    }

    public void append(DataModelType m)
            throws Exception {
        members.addElement(m);
    }

    public DataModelType getMember(String name) {
        Enumeration e = members.elements();

        while (e.hasMoreElements()) {
            DataModelType m = (DataModelType) e.nextElement();

            if (name.equals(m.getMemberName())) {

                return m;
            }
        }

        return null;
    }

    /**
     * returns all non-method members
     */
    public Vector getMembers() {
        Vector ret = new Vector();
        Enumeration e = members.elements();

        while (e.hasMoreElements()) {
            DataModelType m = (DataModelType) e.nextElement();

            if (m.isMember()) {
                ret.addElement(m);
            }
        }

        return ret;
    }

    public Vector getMemberNames() {
        Vector ret = new Vector();
        Enumeration e = members.elements();

        while (e.hasMoreElements()) {
            DataModelType m = (DataModelType) e.nextElement();

            if (m.isMember()) {
                ret.addElement(m.getMemberName());
            }
        }

        return ret;
    }

    public void appendDTD(StringBuffer sb) {
        sb.append("<!ELEMENT ");
        sb.append(getName());

        Vector members = getMembers();

        if (members.size() > 1) {
            Enumeration e = members.elements();
            sb.append(" (");

            while (e.hasMoreElements()) {
                DataModelType t = (DataModelType) e.nextElement();

                if (t.isArray()) {
                    sb.append(t.getName());
                    sb.append("*");
                } else {
                    sb.append(t.getMemberName());
                }

                if (e.hasMoreElements()) {
                    sb.append(", ");
                }
            }

            sb.append(")>\n");

            e = members.elements();

            while (e.hasMoreElements()) {
                DataModelType t = (DataModelType) e.nextElement();

                t.appendDTD(sb);
            }
        }
    }
}
