//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel.persistence;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelEnumeration;
import com.nexirius.framework.datamodel.DataModelVector;

import java.util.Collection;
import java.util.Hashtable;

/**
 * This class is used a for storing a table of PersistentModels together with
 * the Environment and an additional object
 *
 * @author Marcel Baumann
 */
public class PersistentModelTable {



    private Hashtable table = new Hashtable();
    private DataModelVector vector = new DataModelVector();

    public PersistentModelTable() {
    }

    public PersistentModelTableEntry put(DataModel model, PersistenceEnvironment environment) {
        PersistentModelTableEntry ret = new PersistentModelTableEntry(model, environment);
        String instanceName = model.getInstanceName();

        if (instanceName == null) {
            throw new RuntimeException("Cannot introduce instance with null instance identifier");
        }

        if (null == table.put(environment.getName() + '.' + instanceName, ret)) {
            vector.append(model);
        }

        return ret;
    }

    public PersistentModelTableEntry get(String environmentName, String identifier) {
        return (PersistentModelTableEntry) table.get(environmentName + '.' + identifier);
    }

    public void remove(String environmentName, String identifier) {
        PersistentModelTableEntry e = (PersistentModelTableEntry) table.remove(environmentName + '.' + identifier);

        if (e != null) {
            vector.remove(e.model);
        }
    }

    public DataModelEnumeration getEnumeration() {
        return vector.getEnumeration();
    }

    public synchronized Collection getAllEntries() {
        return table.values();
    }
}
