package com.nexirius.framework.datamodel.persistence.generic;

import com.nexirius.util.StringVector;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.jdbc.JdbcConnectionHandler;

import java.util.Iterator;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;

public class DataModelSearchAttributes {
    private String databaseTableName;
    private String dataModelClassName;
    private StringVector searchableFields;

    public DataModelSearchAttributes(String databaseTableName, String dataModelClassName, StringVector searchableFields) {
        this.databaseTableName = databaseTableName;
        this.dataModelClassName = dataModelClassName;
        this.searchableFields = searchableFields;
    }

    public String getDatabaseTableName() {
        return databaseTableName;
    }

    public String getDataModelClassName() {
        return dataModelClassName;
    }

    public StringVector getSearchableFields() {
        return searchableFields;
    }

    public void setSearchableFields(StringVector searchableFields) {
        this.searchableFields = searchableFields;
    }

    public StringBuffer createAndLikeWhereClause(DataModel searchAttributesStructure) {
        StringVector values = new StringVector();

        for (String field = searchableFields.firstItem(); field != null; field = searchableFields.nextItem()) {
            values.append(searchAttributesStructure.getChildText(field));
        }

        return createAndLikeWhereClause(values);
    }

    public StringBuffer createAndLikeWhereClause(StringVector values) {
        StringBuffer ret = new StringBuffer();

        if (values.size() != searchableFields.size()) {
            throw new RuntimeException("Expect equal number of values: have " + values.size() + " instead of " + searchableFields.size());
        }

        int i = 0;
        for (String field = searchableFields.firstItem(); field != null; field = searchableFields.nextItem()) {
            String value = values.getItem(i++);

            if (value != null)  {
                value = value.trim();

                if (value.trim().length() > 0) {
                    WhereClause.appendAnd(ret);
                    ret.append(field);
                    ret.append(" LIKE '%");
                    ret.append(value);
                    ret.append("%' ");
                }
            }
        }

        return ret;
    }

    public void insertAttributes(JdbcConnectionHandler jdbcConnectionHandler, DataModel model, IGenericNamespace namespace) throws SQLException {
        StringVector searchableFields = getSearchableFields();

        StringBuffer attributes = new StringBuffer();
        StringBuffer questionmarks = new StringBuffer();
        boolean first = true;

        Iterator it = searchableFields.iterator();

        while (it.hasNext()) {
            String field = (String) it.next();

            if (first) {
                first = false;
            } else {
                attributes.append(',');
                questionmarks.append(',');
            }

            attributes.append(field);
            questionmarks.append('?');
        }

        it = searchableFields.iterator();

        StringBuffer sql = new StringBuffer("INSERT INTO ");

        sql.append(getDatabaseTableName());

        if (namespace == null) {
            sql.append(" (UUID,");
        } else {
            sql.append(" (UUID,NAMESPACE_UUID,");
        }

        sql.append(attributes.toString());
        if (namespace == null) {
            sql.append(") VALUES (?,");
        } else {
            sql.append(") VALUES (?,?,");
        }
        sql.append(questionmarks.toString());

        if (namespace == null) {
            sql.append(");");
        } else {
            sql.append(",?);");
        }

        System.out.println("sql.toString() = " + sql.toString());

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        it = searchableFields.iterator();

        statement.setString(i++, model.getInstanceName());

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        while (it.hasNext()) {
            String field = (String) it.next();

            statement.setString(i++, model.getChildText(field));
        }

        statement.execute();

        statement.close();
    }

    public boolean hasSearchAttributes(JdbcConnectionHandler jdbcConnectionHandler, String instanceName, IGenericNamespace namespace) throws SQLException {
        StringBuffer sql = new StringBuffer("SELECT UUID FROM ");

        sql.append(getDatabaseTableName());

        if (namespace == null) {
            sql.append(" WHERE UUID=?;");
        } else {
            sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
        }

        System.out.println("sql.toString() = " + sql.toString());
        System.out.println("instanceName = " + instanceName);
        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        statement.setString(i++, instanceName);

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();

        ResultSet result = statement.getResultSet();

        boolean ret = false;

        if (result.first()) {
            return true;
        }

        statement.close();

        return ret;
    }

    public void updateAttributes(JdbcConnectionHandler jdbcConnectionHandler, DataModel model, IGenericNamespace namespace) throws SQLException {
        StringVector searchableFields = getSearchableFields();

        StringBuffer attributes = new StringBuffer();
        boolean first = true;

        Iterator it = searchableFields.iterator();

        while (it.hasNext()) {
            String field = (String) it.next();

            if (first) {
                first = false;
            } else {
                attributes.append(',');
            }

            attributes.append(field);
            attributes.append("=?");
        }

        it = searchableFields.iterator();

        StringBuffer sql = new StringBuffer("UPDATE ");

        sql.append(getDatabaseTableName());

        sql.append(" SET ");

        sql.append(attributes.toString());

        if (namespace == null) {
            sql.append(" WHERE UUID=?;");
        } else {
            sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
        }

        System.out.println("sql.toString() = " + sql.toString());

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        it = searchableFields.iterator();

        while (it.hasNext()) {
            String field = (String) it.next();

            statement.setString(i++, model.getChildText(field));
        }

        statement.setString(i++, model.getInstanceName());

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();

        statement.close();
    }

//    StringBuffer sql = new StringBuffer("UPDATE ");
//
//    sql.append(TABLE_JNEX_INSTANCE);
//    sql.append(" SET TYPE=?,VALUE=?");
//
//    if (namespace == null) {
//        sql.append(" WHERE UUID=?;");
//    } else {
//        sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
//    }

}
