package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.*;

import javax.swing.ComboBoxModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import java.util.Hashtable;

/**
 * ArrayModel wrapper to act as SwingCombobox Model
 */
public class ArrayModelComboBoxModel implements ComboBoxModel {
    private ArrayModel arrayModel;
    private Hashtable listenerInstances = new Hashtable();
    private ModelFlag selectedFlag;

    public ArrayModelComboBoxModel(ArrayModel arrayModel, ModelFlag selectedFlag) {
        this.arrayModel = arrayModel;
        this.selectedFlag = selectedFlag;
    }

    public int getSize() {
        return arrayModel.getSize();
    }

    public Object getElementAt(int index) {
        return arrayModel.getDataModel(index);
    }

    public void addListDataListener(ListDataListener l) {
        ListDataListenerModelListener listener = new ListDataListenerModelListener(l);
        listenerInstances.put(l, listener);
        arrayModel.addSoftDataModelListener(listener);
    }

    public void removeListDataListener(ListDataListener l) {
        ListDataListenerModelListener listener = (ListDataListenerModelListener) listenerInstances.get(l);
        if(listener != null) {
            arrayModel.removeDataModelListener(listener);
        }
    }

    public void setSelectedItem(Object anItem) {
        if (selectedFlag == null) {
            arrayModel.setHighlightedItem((DataModel) anItem);
        } else {
            DataModelEnumeration en = arrayModel.getEnumeration();

            // remove all flags
            while(en.hasMore()) {
                DataModel dataModel = en.next();

                if (dataModel.getFlag(selectedFlag)) {
                    if (dataModel != anItem) {
                        dataModel.setFlag(selectedFlag, false);
                    }
                }
            }

            if (anItem instanceof DataModel) {
                ((DataModel)anItem).setFlag(selectedFlag, true);
            }
        }
    }

    public Object getSelectedItem() {
        if (selectedFlag == null) {
            return arrayModel.getHighlightedItem();
        } else {
            DataModelEnumeration en = arrayModel.getEnumeration();

            // remove all flags
            while(en.hasMore()) {
                DataModel dataModel = en.next();

                if (dataModel.getFlag(selectedFlag)) {
                    return dataModel;
                }
            }
        }

        return null;
    }

    public class ListDataListenerModelListener extends DataModelAdaptor {
        private ListDataListener listener;

        public ListDataListenerModelListener(ListDataListener listener) {
            this.listener = listener;
        }

        public void dataModelChangeValue(DataModelEvent event) {
            listener.contentsChanged(new ListDataEvent(arrayModel, ListDataEvent.CONTENTS_CHANGED, 0, getSize() - 1));
        }
    }
}
