//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.DataModelCommand;
import com.nexirius.framework.swing.CommandJButton;
import com.nexirius.framework.swing.CommandJMenuItem;

import javax.swing.*;

/**
 * This class creates a button or a menu status and is associated with a DataModelCommand instance
 *
 * @author Marcel Baumann
 * @see com.nexirius.framework.datamodel.DataModelCommand
 */
public class CommandViewer extends DataViewer {



    public static final String s_viewername = "CommandViewer";

    protected DataModelCommand command;

    /**
     * Creates a new CommandViewer which is attached to the given command
     */
    public CommandViewer(DataModelCommand command) {
        super(command.getDataModel());
        this.command = command;
    }

    /**
     * Creates the actual Swing JButton
     */
    public void create() {
        setJComponent(createJButton());
        getJComponent().setName(command.getCommandName());
    }

    public DataModelCommand getCommand() {
        return command;
    }

    /**
     * Empty implementation
     */
    public void update() {
    }

    /**
     * Creates a new Swing JMenuItem which is associated with the data model command.
     */
    public JMenuItem createJMenuItem() {
        CommandJMenuItem ret = new CommandJMenuItem(factory.getClientResource(), command, factory.getCommandProcessor());

//        ret.setResourceKey(command.getResourceKey());

        return ret;
    }

    /**
     * Creates a new Swing JButton which is associated with the data model command.
     */
    public JButton createJButton() {
        CommandJButton ret = new CommandJButton(factory.getClientResource(), command, factory.getCommandProcessor());

//		ret.setResourceKey(command.getResourceKey());

        return ret;
    }

    /**
     * Only for debugging
     */
    public String getViewerName() {
        return s_viewername;
    }

    /**
     * grabFocus is not showing any effect (because it would steal the focus from any subcomponent which
     * throws a DataModel.GRAB_FOCUS event
     */
    public void grabFocus() {

    }
}
