//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import javax.swing.JProgressBar;

import com.nexirius.framework.datamodel.*;

/**
 * This viewer is designed to show LimitIntModel or DoubleModel or IntModel data.
 * It is implemented as a Swing JProgressBar.
 * <br>
 * A DoubleModel is displayed as progress 0.0 to 1.0 (0%-100%) if < 0.0 indefinite if > 1.0 then 100%
 * <br>
 * An IntModel is displayed as progress 0 to 100 (0%-100%) if < 0 indefinite if > 100 then 100%
 * <br>
 * An LimitIntModel is displayed as progress min to max (0%-100%)
 *
 * @author Marcel Baumann
 */
public class ProgressViewer extends DataViewer {



    public static final String s_viewername = "ProgressViewer";

    /**
     * Creates a Progress viewer based on a simple model
     */
    public ProgressViewer(SimpleModel model) {
        super(model);
    }

    /**
     * Returns the actual percentage or -1 if indeterminate
     */
    public int getPercentage() {
        double percentage = -1.0;

        if (getDataModel() instanceof DoubleModel) {
            percentage = ((DoubleModel) getDataModel()).getDouble();
        } else if (getDataModel() instanceof LimitIntModel) {
            LimitIntModel lim = (LimitIntModel) getDataModel();

            percentage = (double) (lim.getInt() - lim.getLowLimit()) / (double) (lim.getHighLimit() - lim.getLowLimit());
        } else if (getDataModel() instanceof IntModel) {
            percentage = ((IntModel) getDataModel()).getInt() / 100.0;
        }

        if (percentage < 0.0) {

            return -1;
        }

        if (percentage > 1.0) {

            return 100;
        }

        return (int) (percentage * 100.0);
    }

    /**
     * Returns the created JProgressBar
     */
    public JProgressBar getJProgressBar() {
        return (JProgressBar) getJComponent();
    }

    /**
     * Creates the progress bar
     */
    public void create() {
        setJComponent(new JProgressBar());
        update();
    }

    /**
     * Set the actual value of the progress bar
     * data model.
     */
    public void update() {
        if (isCreated()) {
            int percentage = getPercentage();

            if (model.isExceptional() || percentage < 0) {
                getJProgressBar().setIndeterminate(true);
            } else {
                getJProgressBar().setMaximum(100);
                getJProgressBar().setValue(percentage);
                getJProgressBar().setIndeterminate(false);
            }

            getJProgressBar().setVisible(!getDataModel().isInvisible());
        }
    }

    /**
     * Only used for debugging
     */
    public String getViewerName() {
        return s_viewername;
    }

    /**
     * Not implemented for JProgressBar
     */
    public void grabFocus() {
    }
}
