//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.gadgets.ArrayLayout;
import com.nexirius.framework.swing.CFJRadioButton;
import com.nexirius.util.SortedVector;
import com.nexirius.util.StringVector;
import com.nexirius.util.assertion.Assert;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This viewer is designed to show a list of Objects in a radio button box.
 *
 * @author Marcel Baumann
 * @see com.nexirius.framework.datamodel.ComboBoxModel
 * @see com.nexirius.framework.datamodel.LimitIntModel
 */

@SuppressWarnings("serial")
public class RadioBoxViewer extends DataViewer {
    public static final String s_viewername = "RadioBoxViewer";
    CFJRadioButton buttons[] = null;
    JPanel buttonPanel;
    boolean swap = false;
    int hide[] = null;
    boolean horizontal = false;
    public static final String KEYSTROKE_UP = "Keystroke.up";
    public static final String KEYSTROKE_DOWN = "Keystroke.down";

    /**
     * Creates a viewer associated to the given model
     */
    public RadioBoxViewer(LimitIntModel model) {
        super(model);
    }

    /**
     * Creates a viewer associated to the given model
     */
    public RadioBoxViewer(LimitIntModel model, int hide[]) {
        super(model);
        this.hide = hide;
    }

    /**
     * Creates a viewer associated to the given model
     */
    public RadioBoxViewer(BooleanModel model, boolean swap) {
        super(model);
        this.swap = swap;
    }

    public void setHorizontal(boolean horizontal) {
        this.horizontal = horizontal;
    }

    /**
     * Returns the associated data model
     */
    public DataModel getModel() {
        return getDataModel();
    }

    boolean mustHide(int index) {
        if (hide == null) {
            return false;
        }

        for (int i = 0; i < hide.length; ++i) {
            if (hide[i] == index) {

                return true;
            }
        }

        return false;
    }

    protected boolean isGray(int i)
    {
        boolean ret = false;

        if (getModel() instanceof LimitIntModel)
        {
            ObjectList list = ((LimitIntModel) getModel()).getObjectList();
            Object o = list.getElementAt(i);
            if (o instanceof DataModel)
            {
                DataModel dataModel = (DataModel) o;
                ret = dataModel.isGray();
            }
        }
        return (ret);
    }

    /**
     * Creates the actual ArrayPanel
     */
    @Override
    public void create() {

        if (getModel() instanceof LimitIntModel) {
            if (!((LimitIntModel) getModel()).getObjectList().hasList()) {
                setVector(new StringVector("...", ""));
            }

            ObjectList list = ((LimitIntModel) getModel()).getObjectList();

            buttonPanel = createButtonPanel();
            Insets margin = getInsets();

            buttons = new CFJRadioButton[list.getSize()];
            ButtonGroup group = new ButtonGroup();
            MyActionListener myActionListener = new MyActionListener(group);

            for (int i = 0; i < list.getSize(); ++i) {
                Object o = list.getElementAt(i);
                CFJRadioButton button = new CFJRadioButton(getFactory().getClientResource(), o.toString());

                button.setMargin(margin);
                group.add(button);

                boolean mustHide = mustHide(i);
                if (o instanceof DataModel)
                {
                    DataModel dataModel = (DataModel)o;
                    if (dataModel.isInvisible())
                    {
                        // Diese Möglichkeit nicht darstellen wenn das Invisible Flag gesetzt ist
                        mustHide = true;
                    }
                    if (dataModel.isGray())
                    {
                        // Die Option  zwar darstellen, dem Benutzer aber nicht die Möglichkeit geben sie auszuwählen!
                        button.setEnabled(false);
                    }
                }

                if (!mustHide)
                {
                    addButton(buttonPanel, button);
                }

                buttons[i] = button;
                button.addActionListener(myActionListener);
            }

            addKeystrokes(buttonPanel);

            setJComponent(buttonPanel);
        } else if (getModel() instanceof BooleanModel) {
            ArrayLayout al = new ArrayLayout(true, ArrayLayout.FULL_SIZE);
            buttonPanel = new JPanel(al);
            ButtonGroup group = new ButtonGroup();

            al.setMargin(0);
            buttons = new CFJRadioButton[2];
            MyBooleanListener myBooleanListener = new MyBooleanListener(group);
            SimpleArrayModel labelArray = ((BooleanModel) getModel()).getTrueFalseLabelModel();
            String labels[];

            if (labelArray == null) {
                labels = BooleanModel.TRUE_FALSE;
            } else {
                labels = new String[2];
                labels[0] = labelArray.getItem(0).toString();
                labels[1] = labelArray.getItem(1).toString();
            }

            buttons[0] = new CFJRadioButton(getFactory().getClientResource(), labels[0]);
            buttons[1] = new CFJRadioButton(getFactory().getClientResource(), labels[1]);
            group.add(buttons[0]);
            group.add(buttons[1]);

            if (swap) {
                addButton(buttonPanel, buttons[0]);
                addButton(buttonPanel, buttons[1]);
            } else {
                addButton(buttonPanel, buttons[1]);
                addButton(buttonPanel, buttons[0]);
            }

            buttons[0].addActionListener(myBooleanListener);
            buttons[1].addActionListener(myBooleanListener);

            setJComponent(buttonPanel);
        }

        update();
    }

    private void addKeystrokes(JPanel targetComponent) {
        KeyStroke up;
        KeyStroke down;

        if (horizontal) {
            up = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0);
            down = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0);
        } else {
            up = KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);
            down = KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);
        }

        targetComponent.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(up, KEYSTROKE_UP);
        targetComponent.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(down, KEYSTROKE_DOWN);
        targetComponent.getActionMap().put(KEYSTROKE_UP, new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                keystrokeUp();
            }
        });
        targetComponent.getActionMap().put(KEYSTROKE_DOWN, new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                keystrokeDown();
            }
        });
    }

    protected int getNextEnabledItem(int index, int direction)
    {
        LimitIntModel intModel = (LimitIntModel) getModel();
        int newIndex = index + direction;

        while (newIndex != index)
        {
            if (newIndex < intModel.getLowLimit())
            {
                newIndex = intModel.getHighLimit();
            }
            if (newIndex > intModel.getHighLimit())
            {
                newIndex = intModel.getLowLimit();
            }
            if (isGray(newIndex) == false)
            {
                return (newIndex);
            }
            newIndex += direction;
        }
        return (index);
    }

    private void keystrokeDown() {
        keyStrokeDirection(false);
    }

    private void keystrokeUp() {
        keyStrokeDirection(true);
    }

    private void keyStrokeDirection(boolean up) {
        ArrayList<JRadioButton> actualButtons = new ArrayList<JRadioButton>();
        Component[] comps = buttonPanel.getComponents();
        int selected = -1;

        for (Component comp : comps) {
            if (comp instanceof JRadioButton) {
                JRadioButton button = (JRadioButton) comp;

                if (button.isEnabled() && button.isVisible()) {
                    actualButtons.add(button);
                    if (button.isSelected()) {
                        selected = actualButtons.size() - 1;
                    }
                }
            }
        }

        int numButtons = actualButtons.size();

        if (selected >= 0 && numButtons > 1) {
            selected += (up ? -1 : 1);

            if (selected >= numButtons) {
                selected = 0;
            } else if (selected < 0) {
                selected = numButtons - 1;
            }


            JRadioButton jRadioButton = actualButtons.get(selected);
            jRadioButton.requestFocus();
            jRadioButton.doClick();
        }
    }

    protected Insets getInsets() {
        return new Insets(0, 30, 0, 30);
    }

    protected void addButton(JPanel buttonPanel, CFJRadioButton button) {
        buttonPanel.add(button);
    }

    protected JPanel createButtonPanel() {
        return new JPanel(new ArrayLayout(horizontal, ArrayLayout.FULL_SIZE));
    }

    class MyActionListener implements ActionListener {
        ButtonGroup group;

        MyActionListener(ButtonGroup g) {
            group = g;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            for (int i = 0; i < buttons.length; ++i) {
                if (buttons[i].getModel() == group.getSelection()) {
                    ((LimitIntModel) getModel()).setInt(i);
                    break;
                }
            }
        }
    }

    class MyBooleanListener implements ActionListener {
        ButtonGroup group;

        MyBooleanListener(ButtonGroup g) {
            group = g;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            for (int i = 0; i < buttons.length; ++i) {
                if (buttons[i].getModel() == group.getSelection()) {
                    ((BooleanModel) getModel()).setBoolean(i != 0);
                    break;
                }
            }
        }
    }

    @Override
    public void update() {
        if (getModel() instanceof LimitIntModel) {
            int newValue = ((LimitIntModel) getModel()).getInt();

            if (buttons != null && newValue < buttons.length) {
                buttons[newValue].getModel().setSelected(true);
            }
        } else {
            int newValue = 0;

            if (((BooleanModel) getModel()).getBoolean()) {
                newValue = 1;
            }

            if (buttons != null && newValue < buttons.length) {
                buttons[newValue].getModel().setSelected(true);
            }
        }

        updateUI();
    }

    @Override
    public void updateUI()
    {
        super.updateUI();

        if (getModel().isGray())
        {
            // Alle Buttons Grau darstellen
            for (CFJRadioButton button : buttons)
            {
                button.setEnabled(false);
            }
        }
        else
        {
            // Die enabledten Buttons, und nur diese,  normal darstellen
            for (int i = 0; i < buttons.length; i++)
            {
                boolean enable = true;
                if (getModel() instanceof LimitIntModel)
                {
                    ObjectList list = ((LimitIntModel) getModel()).getObjectList();
                    Object o = list.getElementAt(i);
                    if (o instanceof DataModel)
                    {
                        enable = ((DataModel)o).isGray() == false;
                    }
                }
                buttons[i].setEnabled(enable);
            }
        }
    }

    /**
     * Used for debugging
     */
    @Override
    public String getViewerName() {
        return s_viewername;
    }

    /**
     * Assign an object list which is used with the associated LimitIntModel.
     *
     * @see com.nexirius.framework.datamodel.LimitIntModel
     */
    public void setVector(SortedVector<?> v) {
        Assert.pre(v != null && v.size() > 0, "The ComboBoxViewer needs a list with at least one element");

        ObjectList l = ((LimitIntModel) getModel()).getObjectList();

        l.setVector(v);
    }
}
