//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.SimpleModel;

import javax.swing.*;

/**
 * This viewer is designed to show SimpleModel data. It is implemented as a Swing JLabel (opaque == true).
 * Basically the getText method of the associated model is used to set the label text.
 *
 * @author Marcel Baumann
 */
public class SimpleViewer extends DataViewer {



    public static final String s_viewername = "SimpleViewer";

    /**
     * Creates a simple viewer based on a simple model
     */
    public SimpleViewer(SimpleModel model) {
        super(model);
    }

    /**
     * Returns the associated model
     */
    public SimpleModel getModel() {
        return (SimpleModel) getDataModel();
    }

    /**
     * Returns the created JLabel or null^(if called before creation.
     */
    public JLabel getJLabel() {
        return (JLabel) getJComponent();
    }

    /**
     * Creates the label
     */
    public void create() {
        setJComponent(new JLabel("", SwingConstants.LEFT));
        getJLabel().setOpaque(true);
        update();
    }

    /**
     * Set the text of the label to the string value which is returned by the getText method from the associated
     * data model.
     */
    public void update() {
        SimpleModel model = getModel();

        if (isCreated()) {
            if (model.isExceptional()) {
                getJLabel().setText(factory.getClientResource().getLabel(model.getStatus().getExceptionalStringId()));
            } else {
                String text = model.getText();

                if (model.getFlag(ModelFlag.TRANSLATE_VALUE)) {
                    text = factory.getClientResource().getLabel(text);
                }

                getJLabel().setText(text);
            }
        }
    }

    /**
     * Only used for debugging
     */
    public String getViewerName() {
        return s_viewername;
    }

    /**
     * Not implemented for JLabel
     */
    public void grabFocus() {
    }
}
