//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER

package com.nexirius.framework.gadgets;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;
import java.awt.*;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * JPanel which displays HTML Text content based on an URL
 */
public class HtmlPanel extends JPanel {



    JEditorPane html;
    HtmlPanelAdaptor adaptor = null;

    public HtmlPanel(URL url) {
        setLayout(new BorderLayout());
        getAccessibleContext().setAccessibleName("HTML panel");
        getAccessibleContext().setAccessibleDescription("A panel for viewing HTML documents, and following their links");

        try {
            if (url == null) {
                html = new JEditorPane();
            } else {
                html = new JEditorPane(url);
            }

            html.setEditable(false);
            html.addHyperlinkListener(new DefaultHyperlinkListener());

            JScrollPane scroller = new JScrollPane();
            scroller.setFocusable(false);
            scroller.getViewport().setFocusable(false);
            scroller.getHorizontalScrollBar().setUnitIncrement(30);
            scroller.getVerticalScrollBar().setUnitIncrement(30);

            JViewport vp = scroller.getViewport();
            vp.add(html);
            // vp.setBackingStoreEnabled(true); deprecated with 1.3
            vp.setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
            add(scroller, BorderLayout.CENTER);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * set notification instance
     */
    public void setAdaptor(HtmlPanelAdaptor a) {
        adaptor = a;
    }

    /**
     * Follows the reference in an
     * link.  The given url is the requested reference.
     * By default this calls <a href="#setPage">setPage</a>,
     * and if an exception is thrown the original previous
     * document is restored and a beep sounded.  If an
     * attempt was made to follow a link, but it represented
     * a malformed url, this method will be called with a
     * null argument.
     *
     * @param u the URL to follow
     */
    public void linkActivated(URL u) {
        if (html != null) {
            Cursor c = html.getCursor();
            Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
            html.setCursor(waitCursor);

            SwingUtilities.invokeLater(new PageLoader(u, c));
        }
    }

    /**
     * temporary class that loads synchronously (although
     * later than the request so that a cursor change
     * can be done).
     */
    class PageLoader implements Runnable {
        URL url;
        Cursor cursor;

        PageLoader(URL u, Cursor c) {
            url = u;
            cursor = c;
        }

        @Override
        public void run() {
            if (url == null) {
                // restore the original cursor
                html.setCursor(cursor);

                Container parent = html.getParent();
                parent.repaint();
            } else {
                Document doc = html.getDocument();

                try {
                    html.setPage(url);

                    if (adaptor != null) {
                        adaptor.notify(url);
                    }
                } catch (IOException ioe) {
                    html.setDocument(doc);
                    getToolkit().beep();
                } finally {
                    // schedule the cursor to revert after
                    // the paint has happended.
                    url = null;
                    SwingUtilities.invokeLater(this);
                }
            }
        }
    }

    class DefaultHyperlinkListener implements HyperlinkListener {
        /**
         * Notification of a change relative to a
         * hyperlink.
         */
        @Override
        public void hyperlinkUpdate(HyperlinkEvent e) {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                linkActivated(e.getURL());
            }
        }
    }
}
