//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets;

import com.nexirius.framework.datamodel.*;

import javax.swing.*;
import java.awt.*;

public class LayoutPanelComponentModel extends StructModel {

    public static final int HOTSPOT_SIZE = 6;

    DynamicJComponentHandler componentHandler;
    IntModel x;
    IntModel y;
    LimitIntModel w;
    LimitIntModel h;
    StringModel name;
    ColorModel color;
    int xOffset = 0;
    int yOffset = 0;

    boolean resizing = false;

    public LayoutPanelComponentModel() {
        super("LayoutPanelComponent");
        init(null, 0);
    }

    public LayoutPanelComponentModel(JComponent component, int index) {
        super("LayoutPanelComponent");
        init(component, index);
    }

    private void init(JComponent component, int index) {
        append(x = new IntModel(0, "x"));
        append(y = new IntModel(0, "y"));
        append(w = new LimitIntModel(0, 0, Integer.MAX_VALUE, "twidth"));
        append(h = new LimitIntModel(0, 0, Integer.MAX_VALUE, "theight"));
        append(color = new ColorModel("background", Color.gray));
        append(name = new StringModel("", "name"));
        name.setFlag(ModelFlag.NOT_EDITABLE, true);
        name.setFlag(ModelFlag.TRANSIENT, true);

        reInit(component, index);
    }

    public void reInit(JComponent component, int index) {
        if (component == null) {
            return;
        }

        Dimension size = component.getPreferredSize();
        Point location = component.getLocation();

        x.setInt(location.x);
        y.setInt(location.y);
        w.setInt(size.width);
        h.setInt(size.height);

        String n = component.getName();

        if (n == null || n.length() == 0) {
            n = "Component_" + index;
        }

        name.setText(n);
    }

    public void layout(JComponent component) {
        component.setBounds(x.getInt(), y.getInt(), w.getInt(), h.getInt());
        component.setBackground(color.getColor());
        handleAfterCreation(component);
        component.doLayout();
    }

    public void paint(Graphics g, Point offset) {
        int rx = x.getInt() + offset.x;
        int ry = y.getInt() + offset.y;
        int rw = w.getInt();
        int rh = h.getInt();
        g.setColor(Color.red);
        g.drawRect(rx, ry, rw, rh);
        g.setColor(Color.blue);
        g.fillRect(rx + rw - HOTSPOT_SIZE / 2, ry + rh - HOTSPOT_SIZE / 2, HOTSPOT_SIZE, HOTSPOT_SIZE);
    }

    public Rectangle getBounds() {
        return new Rectangle(x.getInt(), y.getInt(), w.getInt(), h.getInt());
    }

    public Rectangle getResizeBounds() {
        return new Rectangle(x.getInt() + w.getInt() - HOTSPOT_SIZE / 2, y.getInt() + h.getInt() - HOTSPOT_SIZE / 2, HOTSPOT_SIZE, HOTSPOT_SIZE);
    }

    public boolean click(int px, int py) {
        if (getResizeBounds().contains(px, py)) {
            resizing = true;
            return true;
        }

        resizing = false;
        xOffset = px - x.getInt();
        yOffset = py - y.getInt();
        return getBounds().contains(px, py);
    }

    public boolean isInside(int x, int y) {
        return getBounds().contains(x, y);
    }

    public void move(int px, int py) {
        if (resizing) {
            w.setInt(px - x.getInt());
            h.setInt(py - y.getInt());
        } else {
            x.setInt(px - xOffset);
            y.setInt(py - yOffset);
        }
    }

    public void handleAfterCreation(JComponent comp) {
        try {
            if (componentHandler != null) {
                componentHandler.handleAfterCreation(comp);
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }
}
