//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.ViewerFactory;

import javax.swing.*;
import java.awt.*;

public class LayoutPanelModel extends StructModel {

    public static final String TO_TOP_COMMAND = "toTop";

    BooleanModel glassPaneVisible;
    IntModel snapGrid;
    LayoutPanelComponentArrayModel componentArray;

    public LayoutPanelModel() {
        super("LayoutPanel");
        init();
    }

    private void init() {
        append(glassPaneVisible = new BooleanModel(false, "glassPaneVisible"));
        append(snapGrid = new IntModel(10, "snapGrid"));
        append(componentArray = new LayoutPanelComponentArrayModel());
        appendMethod(new DefaultDataModelCommand(TO_TOP_COMMAND));
        appendMethod(new DefaultDataModelCommand("edit"));

        glassPaneVisible.setTransient(true);
    }

    public int getHighlightedIndex() {
        return componentArray.getHighlightedItemIndex();
    }

    public void toTopHighlightedItem() {
        componentArray.setFireEvents(false);
        DataModel m = componentArray.removeHighlightedItem();
        componentArray.setFireEvents(true);
        componentArray.insertItemAt(m, 0);

        componentArray.setHighlightedItem(0);
    }

    public void addComponent(JComponent comp) {
        componentArray.insertItemAt(new LayoutPanelComponentModel(comp, componentArray.getSize()), 0);
    }

    public void layout(int index, JComponent component) {
        if (componentArray.getSize() > index) {
            componentArray.getLayoutPanelComponentModel(index).layout(component);
        }
    }

    public void paint(Graphics g, Point offset) {
        DataModelEnumeration e = componentArray.getEnumeration();
        LayoutPanelComponentModel h = (LayoutPanelComponentModel) componentArray.getHighlightedItem();

        if (h != null) {
            h.paint(g, offset);
        }
    }

    /**
     * @return
     */
    public LayoutPanelComponentModel getActiveComponentModel() {
        return (LayoutPanelComponentModel) componentArray.getHighlightedItem();
    }

    public void setHighlightedComponent(int x, int y) {
        DataModelEnumeration e = componentArray.getEnumeration();
        int i = 0;

        componentArray.setHighlightedItem(-1);

        while (e.hasMore()) {
            LayoutPanelComponentModel c = (LayoutPanelComponentModel) e.next();

            if (c.click(x, y)) {
                componentArray.setHighlightedItem(i);
                //break; do not break to make sure that all resising boolean are reset to false
            }

            ++i;
        }
    }

    public void moveHighlightedComponent(int x, int y) {
        LayoutPanelComponentModel h = (LayoutPanelComponentModel) componentArray.getHighlightedItem();

        if (h != null) {
            x /= snapGrid.getInt();
            x *= snapGrid.getInt();
            y /= snapGrid.getInt();
            y *= snapGrid.getInt();
            h.move(x, y);
        }
    }

    public void toggleGlass() {
        glassPaneVisible.setBoolean(!glassPaneVisible.getBoolean());
    }

    public void toTop() {
        // not implemented here
    }

    public void edit() throws Exception {
        LayoutPanelComponentModel comp = (LayoutPanelComponentModel) componentArray.getHighlightedItem();
        JFrame frame = new JFrame(comp.name.getText());
        DataViewer viewer = ViewerFactory.getInstance().createDefaultEditor(comp);

        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(viewer.getJComponent(), BorderLayout.CENTER);
        frame.pack();
        DialogManager.center(frame, true);
        frame.setVisible(true);
    }
}
