//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets;

import javax.swing.*;
import java.awt.*;

/**
 * Displays clock (analog)
 */
public class WatchPanel extends JPanel {

    int minutes = 0;
    boolean empty = false;
    public static final double PLENGH_H = 0.6;
    public static final double PLENGH_M = 0.9;

    public WatchPanel(int height) {
        setOpaque(true);
        setPreferredSize(new Dimension(height, height));
    }

    public int getMinutes() {
        return minutes;
    }

    public void setMinutes(int minutes) {
        this.minutes = minutes;
        repaint();
    }

    public boolean isEmpty() {
        return empty;
    }

    public void setEmpty(boolean empty) {
        this.empty = empty;
    }

    public void paint(Graphics g) {
        super.paint(g);

        if (empty) {
            return;
        }

        Dimension size = getSize();
        Point center = new Point(size.width / 2, size.height / 2);

        double h = (double) minutes / 60.0;
        double hangle = h / 12.0 * Math.PI * 2.0;
        double m = minutes % 60;
        double mangle = m / 60.0 * Math.PI * 2.0;

        g.setColor(Color.white);
        g.fillOval(0, 0, size.width, size.height);
        g.setColor(SystemColor.getColor("", SystemColor.CONTROL_TEXT));
        g.drawLine(center.x, center.y, center.x + getX(hangle, PLENGH_H), center.y + getY(hangle, PLENGH_H));
        g.drawLine(center.x, center.y, center.x + getX(mangle, PLENGH_M), center.y + getY(mangle, PLENGH_M));
    }

    private int getX(double hangle, double plengh) {
        return (int) (Math.sin(hangle) * getSize().width / 2 * plengh);
    }

    private int getY(double hangle, double plengh) {
        return -(int) (Math.cos(hangle) * getSize().height / 2 * plengh);
    }
}

