//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets.filechooser;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

public class FileChooser extends JPanel {

    File roots[];
    JComboBox comboBox;
    JButton button;
    JTextField text;
    JTreeTable treeTable;
    JScrollPane scrollpane;
    FileSystemModel fileSystemModel;
    boolean dirOnly;

    public FileChooser(File root, boolean dirOnly) {
        this.dirOnly = dirOnly;
        BorderLayout bl = new BorderLayout();

        fileSystemModel = new FileSystemModel(root, dirOnly);
        treeTable = new JTreeTable(fileSystemModel);
        text = new JTextField();

        roots = File.listRoots();

        MySelectionAdapter adapter = new MySelectionAdapter();
        treeTable.setSelectionAdapter(adapter);
        treeTable.addMouseListener(adapter);

        setLayout(bl);
        JPanel top = new JPanel(new BorderLayout(5, 5));
        top.setBorder(new EmptyBorder(5, 5, 5, 5));

        top.add(comboBox = new JComboBox(roots), BorderLayout.CENTER);
        top.add(button = new JButton("Rescan"), BorderLayout.EAST);

        add(top, BorderLayout.NORTH);
        add(scrollpane = new JScrollPane(treeTable), BorderLayout.CENTER);
        scrollpane.setFocusable(false);
        scrollpane.getViewport().setFocusable(false);

        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
        bottomPanel.add(text, BorderLayout.CENTER);
        add(bottomPanel, BorderLayout.SOUTH);

        comboBox.getModel().setSelectedItem(root);

        comboBox.addActionListener(new MyComboBoxListener());
        button.addActionListener(new RescanListener());
    }

    public void setEditable(boolean on) {
        text.setEnabled(on);
    }

    public TreePath expand(String fileName, boolean highlight) {
        TreePath ret = fileSystemModel.getPathFor(fileName);

        if (ret != null) {
            treeTable.expandPath(ret);

            if (highlight) {
                treeTable.selectPath(ret);
            }

            Rectangle rect = treeTable.getRectForPath(ret);

            if (rect != null) {
                JViewport viewport = scrollpane.getViewport();
                Point p = viewport.getViewPosition();

                p.y = Math.max(rect.y - 50, 0);

                viewport.setViewPosition(p);
            }
        }

        return ret;
    }

    public File getSelectedFile() {
        FileNode fileNode = (FileNode)treeTable.getSelectedTreeComponent();

        if (fileNode == null) {
            return null;
        }

        return fileNode.getFile();
    }

    public static void main(String[] args)
            throws Exception {
        javax.swing.UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");

        JFrame frame = new JFrame("FileChooser");

        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent we) {
                System.exit(0);
            }
        });

        final FileChooser chooser = new FileChooser(new File("c:\\"), false);

        Container contentPane = frame.getContentPane();

        contentPane.setLayout(new BorderLayout());
        contentPane.add(chooser, BorderLayout.CENTER);

        JButton okButton = new JButton("OK");
        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                System.out.println(chooser.getSelectedFile().getPath());
            }
        });

        contentPane.add(okButton, BorderLayout.SOUTH);

        frame.getRootPane().setDefaultButton(okButton);

        frame.pack();
//chooser.expand("c:\\TEMP\\software\\Palm", true);
        chooser.expand("c:\\temp", true);

        frame.show();
    }

    class MySelectionAdapter implements SelectionAdapter, MouseListener {
        @Override
        public void select(Object o) {
            FileNode f = (FileNode) o;
            //System.out.println(file.getAbsolutePath());

            selectFile(f.getFile());
        }

        private void selectFile(File file) {
            if (file == null) {
                text.setText("");
                return;
            }

            if (file.isDirectory()) {
                text.setText(file.getAbsolutePath() + File.separatorChar);
            } else {
                text.setText(file.getAbsolutePath());
            }
        }

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() > 1) {
                File selectedFile = getSelectedFile();
                if (selectedFile != null && selectedFile.isDirectory() && !dirOnly) {
                    return;
                }

                JButton defaultButton = getRootPane().getDefaultButton();

                if (defaultButton != null) {
                    defaultButton.doClick();
                }
            }
        }

        @Override
        public void mousePressed(MouseEvent e) {
        }

        @Override
        public void mouseReleased(MouseEvent e) {
        }

        @Override
        public void mouseEntered(MouseEvent e) {
        }

        @Override
        public void mouseExited(MouseEvent e) {
        }
    }

    public void updateAll() {
        treeTable = new JTreeTable(fileSystemModel = new FileSystemModel((File) comboBox.getModel().getSelectedItem(), dirOnly));

        treeTable.setSelectionAdapter(new MySelectionAdapter());

        remove(scrollpane);
        add(scrollpane = new JScrollPane(treeTable), BorderLayout.CENTER);
        scrollpane.setFocusable(false);
        scrollpane.getViewport().setFocusable(false);

        revalidate();
    }

    class MyComboBoxListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent e) {
            updateAll();
        }
    }

    class RescanListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent e) {
            updateAll();
        }
    }

    public String getText() {
        return text.getText();
    }
}
