//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview;

import com.nexirius.framework.datamodel.DataModel;

import java.io.Serializable;
import java.util.Hashtable;


/**
 * this class stores (and serves) HTML templates which are associated to data model classes
 */
public class HTMLTemplateMap implements Serializable {



    Hashtable editorMap = new Hashtable();
    Hashtable viewerMap = new Hashtable();

    /**
     * get associated template for a data model
     *
     * @param model    the model for which the template is accessed
     * @param isEditor if true then the template for input forms should be returned
     * @return the name of the associated template or null (if no template was mapped)
     */
    public String getTemplateFor(DataModel model, boolean isEditor) {
        if (model == null) {

            return null;
        }

        Hashtable templateMap = (isEditor ? editorMap : viewerMap);

        String ret = null;
        Class c = model.getClass();

        while (ret == null && c != null) {
            ret = (String) templateMap.get(c);
            c = c.getSuperclass();
        }

        return ret;
    }

    /**
     * registerTemplate a template for a data model class
     *
     * @param modelClass the class of the model which is associated with the template
     * @param template   the template
     * @param isEditor   a template can be registered for both (a viewer and an editor)
     */
    public void registerTemplate(Class modelClass, String template, boolean isEditor) {
        Hashtable templateMap = (isEditor ? editorMap : viewerMap);

        templateMap.put(modelClass, template);
    }

    public void registerTemplate(HTMLTemplateMapEntry entry) {
        Hashtable templateMap = (entry.isEditor() ? editorMap : viewerMap);

        templateMap.put(entry.getClazz(), entry.getTemplateName());
    }
}
