//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelEnumeration;
import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.VariableStore;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * Translates the $!array("fieldname", view|edit, "templatename") function.
 * <br>
 * The array function is replaced with a sequence of templates where each template is associated to
 * one array status model.
 */
public class ArrayHTMLFunction extends DefaultHTMLFunction {

    public String getFunctionName() {
        return "array";
    }

    public String getFieldName() {
        return getArgument(0);
    }

    public Boolean getIsEditor() {
        String arg = getArgument(1);

        if (arg == null) {
            return null;
        }

        if (arg.equalsIgnoreCase(HTMLParser.EDIT)) {
            return new Boolean(true);
        }

        if (arg.equalsIgnoreCase(HTMLParser.VIEW)) {
            return new Boolean(false);
        }

        return null;
    }

    public String getTemplate() {
        return getArgument(2);
    }

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String childName = getFieldName();
        DataModel child = parser.getModel().getChild(childName);

        if (!(child instanceof ArrayModel)) {

            if (childName == null) {
                throw new Exception(parser.getModel().getFieldName() + " is not an ArrayModel");
            }

            throw new Exception("Child " + childName + " of " + parser.getModel().getFieldName() + " is not an ArrayModel");
        }

        ArrayModel array = (ArrayModel) child;
        DataModelEnumeration e = array.getEnumeration();
        VariableStore variableStore = parser.getResolver().getVariableStore();
        int index = 0;
        String oldIndex = variableStore.getValueOf(VariableStore.INDEX);
        String oldCount = variableStore.getValueOf(VariableStore.COUNT);
        String template = getTemplate();
        boolean childIsEditor = parser.isEditor();
        Boolean type = getIsEditor();

        if (type != null) {
            childIsEditor = type.booleanValue();
        }

        variableStore.setVariable(VariableStore.COUNT, Integer.toString(array.getSize()));

        while (e.hasMore()) {
            DataModel item = e.next();

            variableStore.setVariable(VariableStore.INDEX, Integer.toString(index + 1));

            byte result[] = parser.getResolver().resolve(sessionVariable, item, template, childIsEditor);

            parser.getOut().write(result);

            ++index;
        }

        variableStore.setVariable(VariableStore.COUNT, oldCount);
        variableStore.setVariable(VariableStore.INDEX, oldIndex);
    }
}
