//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * represents a function which is embedded into HTML $!function(params)
 */
public abstract class DefaultHTMLFunction implements HTMLFunction {
    String[] arguments;

    public DefaultHTMLFunction() {
    }

    /**
     * access the parameter at index
     *
     * @param i index
     * @return the parameter at index or null
     */
    public String getArgument(int i) {
        if (i < arguments.length) {

            return arguments[i];
        }

        return null;
    }

    public abstract String getFunctionName();

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser, String[] parameters) throws Exception {
        arguments = parameters;
        translate(sessionVariable, parser);
    }

    public abstract void translate(HTMLSessionVariable sessionVariable, HTMLParser parser) throws Exception;

    public String getFullChildName(HTMLParser parser, String childName) throws Exception {
        DataModel model = parser.getModel();
        DataModel rootModel = parser.getRootModel();
        DataModel childModel = null;

        if (childName == null || childName.equals("this")) {
            childModel = model;
        } else if (rootModel != null) {
            childModel = model.getChild(childName);
        }

        childName = rootModel.getChildName(childModel);
        return childName;
    }

}
