//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.VariableStore;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * this class handles $!event("editEvent", "link text", "&moreParameter=value", attributes) functions
 * <A HREF='?state=overview&event=editEvent&index=4&moreParameter=value"' attributes>link text</A>
 */
public class EventHTMLFunction extends DefaultHTMLFunction {

    public String getFunctionName() {
        return "event";
    }

    public String getEvent() {
        return getArgument(0);
    }

    public String getLinkText() {
        return getArgument(1);
    }

    public String getMoreParameters() {
        return getArgument(2);
    }

    public String getAttributes() {
        return getArgument(3);
    }

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String event = getEvent();

        if (event == null) {
            throw new Exception("too few parameters for function event (no event parameter)");
        }

        String linkText = getLinkText();

        if (linkText == null) {
            linkText = parser.getResolver().translate(event);
        } else {
            linkText = parser.getResolver().translate(linkText);
        }

        String moreParameters = getMoreParameters();
        String attributes = getAttributes();

        StringBuffer buf = new StringBuffer();

        buf.append("<A HREF='$!encodeURL(\"$(");
        buf.append(VariableStore.REQUEST_URL);
        buf.append(")?");
        buf.append(HTMLFunction.PARAMETER_STATE);
        buf.append("=$(");
        buf.append(VariableStore.STATE);
        buf.append(')');
        buf.append('&');
        buf.append(HTMLFunction.PARAMETER_INDEX);
        buf.append("=$(");
        buf.append(VariableStore.INDEX);
        buf.append(')');
        buf.append('&');
        buf.append(HTMLFunction.PARAMETER_EVENT);
        buf.append('=');
        buf.append(event);
        if (moreParameters!= null) {
            buf.append(moreParameters);
        }
        buf.append("\")' ");
        if (attributes!= null) {
            buf.append(attributes);
        }
        buf.append(">");
        buf.append(linkText);
        buf.append("</A>");

        parser.getOut().write(buf.toString().getBytes(HTMLResolver.CHARSET));
    }
}
