//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * This class handles the $!FOR("counterVariable", "LOOPNUMBER")---block----$ENFOR$ function
 *
 * use the counterVariable like this: $counterVariable$
 * example: $!FOR(i, "$!getNumberOfPartners()") $!showPartner($i$) $ENDFOR$
 */
public class ForHTMLFunction extends DefaultHTMLFunction {
    public static final String FUNCTION_NAME = "FOR";
    public static final String ENDFOR = "$ENDFOR$";

    public String getFunctionName() {
        return FUNCTION_NAME;
    }

    /**
     * access the first parameter
     *
     * @return
     */
    public String getCounterVariable() {
        return getArgument(0);
    }

    /**
     * access the second parameter
     *
     * @return
     */
    public int getLoopNumber(HTMLParser parser) throws Exception {
        String argument = getArgument(1);

        try {
            return Integer.parseInt(argument);
        } catch (NumberFormatException ex) {
            DataModel child = parser.getModel().getChild(argument);

            return child.getChildren().size();
        }
    }

    /**
     * access the third parameter
     *
     * @return
     */
    public String getBlock() {
        return getArgument(2);
    }

    /**
     * translate the function into HTML and write the result back to the parser
     *
     * @param sessionVariable
     * @param parser the parser which translates the embedded commands
     * @throws Exception
     */
    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser) throws Exception {
        String counterVariable = getCounterVariable();
        int n = getLoopNumber(parser);

        for (int i = 0; i < n; ++i) {
            String block = getBlock();

            block = block.replace("$" + counterVariable + "$", Integer.toString(i));
            block = parser.resolve(sessionVariable, block);

            parser.getOut().write(block.getBytes(HTMLResolver.CHARSET));
        }
    }
}
