//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

import java.io.OutputStream;
import java.io.IOException;

/**
 * this class handles $!includeBody("template") functions
 */
public class IncludeBodyHTMLFunction extends DefaultHTMLFunction {
    static String startTag = "<BODY";
    static String startTagSmall = "<body";
    static String endTag = "</BODY>";
    static String endTagSmall = "</body>";

    public String getFunctionName() {
        return "includeBody";
    }

    public String getTemplate() {
        return getArgument(0);
    }

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String template = getTemplate();
        byte result[] = parser.getResolver().resolve(sessionVariable, parser.getModel(), template, parser.isEditor());
        int begin = 0;
        int end = result.length;

        OutputStream out = parser.getOut();

        extractBody(begin, end, result, out);
    }

    public static void extractBody(int begin, int end, byte[] result, OutputStream out) throws IOException {
        begin = writeOrSkip(false, begin, end, result, startTag.getBytes(HTMLResolver.CHARSET), startTagSmall.getBytes(HTMLResolver.CHARSET), out);

        if (begin >= 0) {
            while (begin < end) {
                if (result[begin] == '>') {
                    writeOrSkip(true, begin + 1, end, result, endTag.getBytes(HTMLResolver.CHARSET), endTagSmall.getBytes(HTMLResolver.CHARSET), out);
                    break;
                }
                ++begin;
            }
        }
    }

    private static int writeOrSkip(boolean write, int begin, int end, byte[] result, byte[] tag, byte[] tagSmall, OutputStream out) throws IOException {
        int state = 0;
        int wordStart = begin;

        for (int i = begin; i < end; i++) {
            byte b = result[i];

            if (state < tag.length) {
                if (b == tag[i - wordStart] || b == tagSmall[i - wordStart]) {
                    ++state;
                } else {
                    if (write) {
                        for (int j = wordStart; j <= i; ++j) {
                            out.write((char) result[j]);
                        }
                    }
                    state = 0;
                    wordStart = i + 1;
                }
            } else {
                return i;
            }
        }

        return -1;
    }

    public static void main(String[] argv) throws IOException {
        System.out.print("*");
        byte[] result = "123<Body>ABC</Body>XYZ".getBytes();
        int begin = writeOrSkip(false, 0, result.length, result, startTag.getBytes(HTMLResolver.CHARSET), startTagSmall.getBytes(HTMLResolver.CHARSET), System.out);
        if (begin >= 0) {
            while (begin < result.length) {
                if (result[begin] == '>') {
                    writeOrSkip(true, begin + 1, result.length, result, endTag.getBytes(HTMLResolver.CHARSET), endTagSmall.getBytes(HTMLResolver.CHARSET), System.out);
                    break;
                }
                ++begin;
            }
        }

        System.out.println("*");
    }
}


