//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.VariableStore;
import com.nexirius.framework.htmlview.HTMLSessionVariable;
import com.nexirius.framework.datamodel.DataModel;

/**
 * the default value of the event is HTMLFunction.EVENT_POPUP
 * this class handles $!popup("childname", view|edit, "template", "link text", "event", "true|false") functions
 * if parameter 6 is true, then the actual model is duplicated in the actual state with HTMLState.startDuplicatePopup()
 */
public class PopupHTMLFunction extends FieldHTMLFunction {

    public String getFunctionName() {
        return "popup";
    }

    public String getLinkText() {
        return getArgument(3);
    }

    public String getEvent() {
        return getArgument(4);
    }

    public boolean getDuplicate() {
        String arg = getArgument(5);

        if ("true".equalsIgnoreCase(arg)) {
            return true;
        }

        return false;
    }

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String childName = getChildName();
        String template = getTemplate();
        Boolean type = getIsEditor();
        boolean childIsEditor = parser.isEditor();
        String linkText = getLinkText();
        String event = getEvent();

        if (type != null) {
            childIsEditor = type.booleanValue();
        }

        childName = getFullChildName(parser, childName);

        if (linkText == null) {
            linkText = childName;
            linkText = parser.getResolver().translate(linkText);
        }

        if (event == null || event.equals("null")) {
            event = HTMLFunction.EVENT_POPUP;
        }

        StringBuffer buf = new StringBuffer();

        buf.append("<A HREF='$!encodeURL(\"$(");
        buf.append(VariableStore.REQUEST_URL);
        buf.append(")?");
        buf.append(HTMLFunction.PARAMETER_STATE);
        buf.append("=$(");
        buf.append(VariableStore.STATE);
        buf.append(")");
        buf.append('&');
        buf.append(HTMLFunction.PARAMETER_EVENT);
        buf.append('=');
        buf.append(event);
        buf.append('&');
        buf.append(HTMLFunction.PARAMETER_CHILD);
        buf.append('=');
        buf.append(childName);

        if (type != null) {
            buf.append('&');
            buf.append(HTMLFunction.PARAMETER_ISEDITOR);
            buf.append('=');
            buf.append(childIsEditor ? HTMLParser.EDIT : HTMLParser.VIEW);
        }

        if (template != null) {
            buf.append('&');
            buf.append(HTMLFunction.PARAMETER_TEMPLATE);
            buf.append('=');
            buf.append(template);
        }

        if (getDuplicate()) {
            buf.append('&');
            buf.append(HTMLFunction.PARAMETER_DUPLICATE);
            buf.append("=true");
        }

        buf.append("\")'>");
        buf.append(linkText);
        buf.append("</A>");

        parser.getOut().write(buf.toString().getBytes(HTMLResolver.CHARSET));
    }
}
