//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.jdbc;

import com.nexirius.framework.datamodel.*;

import java.util.Calendar;

public class TestModel extends StructModel {
    private IntModel intField;
    private StringModel stringField;
    private DateModel dateField;
    private ComboBoxModel comboField;
    private ArrayModel relatedModels;

    public TestModel() {
        this(null);
    }

    public TestModel(String name) {
        super(name);

        setInstanceName(name);

        init();
    }

    public ArrayModel getRelatedModels() {
        return relatedModels;
    }

    private void init() {
        intField = new IntModel(0, "intField");
        append(intField);
        stringField = new StringModel("", "stringField");
        append(stringField);
        dateField = new DateModel(new java.util.Date(), "dateField");
        append(dateField);
        comboField = new ComboBoxModel(0, new SimpleArrayModel(new Object[]{"a)", "b)", "c)"}), "comboField");
        append(comboField);
        relatedModels = new ArrayModel(new RelatedModel(), "RelatedModels");
        append(relatedModels);

        StructModel subStruct = new StructModel("subStruct");
        subStruct.append(new StringModel("Value of substruct substring", "subString"));
        append(subStruct);
    }

    public void set(int i, String s, int day, int month, int year, String comboValue) {
        Calendar cal = Calendar.getInstance();

        cal.set(Calendar.DAY_OF_MONTH, day);
        cal.set(Calendar.MONTH, month - 1);
        cal.set(Calendar.YEAR, year);

        dateField.setDate(cal.getTime());
        intField.setInt(i);
        stringField.setText(s);
        comboField.setChildText(null, comboValue);
    }

    public static void main(String[] argv) {
        try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver").newInstance();
        } catch (InstantiationException e) {
            throw new RuntimeException(e);  //TODO
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);  //TODO
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);  //TODO
        }

//        String filename = "c:/java/mdbTEST.mdb";
//        String database = "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=" + filename.trim() + ";DriverID=22;READONLY=false}";

         JdbcConnectionHandler.init(new JdbcConnectionHandler("jdbc:odbc:test", null, null));

        DatabaseTableMapping mapping = new DatabaseTableMapping("TEST_TABLE");

        mapping.registerFieldName("intField", "F_INT");
        mapping.registerFieldName("stringField", "F_STRING");
        mapping.registerFieldName("dateField", "F_DATE");
        mapping.registerTranslator(new MyComboFieldTranslator());

        mapping.setPrimaryKeyField("ID");
        mapping.setPrimaryKeyIsString(false);

        JdbcConnectionHandler.instance().registerDatabaseTableMapping(TestModel.class.getName(), mapping);

        int key = (int) (Math.random() * 1000.0);

        System.out.println("key = " + key);
        TestModel test = new TestModel("" + key);

        test.set(2345, "Jonah", 11, 4, 2001, "b)");

        DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(TestModel.class.getName());
        try {
            TestModel testModelTemplate = new TestModel();
            deleteAllEntries(testModelTemplate, databaseTableMapping);

            databaseTableMapping.create(test);
            databaseTableMapping.read(test);

            System.out.println("test.dragData() = " + test.dragData());

            test.set(111, "Jetzt funktioniert es", 3, 12, 1990, "c)");

            databaseTableMapping.update(test);

            databaseTableMapping.delete(test);

            ArrayModel arr = new ArrayModel(null);

            databaseTableMapping.readChildren("F_INT", "2345", new TestModel(""), arr);

            System.out.println("arr.dragData() = " + arr.dragData());

            JdbcConnectionHandler.instance().closeConnection();
        } catch (Exception e) {
            throw new RuntimeException(e);  //TODO
        }

        DatabaseTableMapping standardMapping = DatabaseTableMapping.createStandardMapping(new TestModel("STANDARD_TABLE"));
        standardMapping.setPrimaryKeyIsString(false);
        DatabaseTableMapping relatedMapping = JdbcConnectionHandler.instance().registerDatabaseTableMapping(RelatedModel.class.getName(), "TEST_RELATED_TABLE", "UID", new String[]{"Name", "Value"}, new String[]{"NAME", "DOUBLE_VALUE"});
        relatedMapping.setForeignKeyField("PARENT_ID");
        JdbcConnectionHandler.instance().registerDatabaseTableMapping(TestModel.class.getName(), standardMapping);

        try {
            TestModel testModelTemplate = new TestModel();
            RelatedModel relatedModelTemplate = new RelatedModel();
            deleteAllEntries(relatedModelTemplate, relatedMapping);
            databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(TestModel.class.getName());

            deleteAllEntries(testModelTemplate, databaseTableMapping);

            databaseTableMapping.create(test);

            JdbcConnectionHandler.instance().closeConnection();

            relatedMapping.readChildren(test.getRelatedModels());

            DataModelEnumeration en = test.getRelatedModels().getEnumeration();

            while (en.hasMore()) {
                DataModel model = en.next();
                relatedMapping.delete(model);
            }

            test.getRelatedModels().append(new RelatedModel("A", 1.1));
            test.getRelatedModels().append(new RelatedModel("B", 2.02));

            relatedMapping.updateChildren(test.getRelatedModels());

            test.getRelatedModels().getDataModel(0).setChildText("Value", "2234235.3245");
            test.getRelatedModels().append(new RelatedModel("C (new)", 5.3456));

            relatedMapping.updateChildren(test.getRelatedModels());

            relatedMapping.commit();

            test.getRelatedModels().clear();

            relatedMapping.readChildren(test.getRelatedModels());

            String s = test.getRelatedModels().dragData();

            System.out.println("s = " + s);
            JdbcConnectionHandler.instance().closeConnection();
        } catch (Exception e) {
            throw new RuntimeException(e);  //TODO
        }
    }

    private static void deleteAllEntries(DataModel template, DatabaseTableMapping mapping) throws Exception {
        ArrayModel result = new ArrayModel(template, "");

        mapping.search(null, template, result);

        mapping.deleteChildren(result);
    }

    static class MyComboFieldTranslator extends SingleFieldTranslator {
        public MyComboFieldTranslator() {
            super("F_COMBO");
        }

        @Override
        public String getModelAttribute(DataModel model) {
            return model.getChildText("comboField");
        }

        @Override
        public void setModelAttribute(DataModel model, String value) {
            model.setChildText("comboField", value);
        }
    }

    public static class RelatedModel extends StructModel {
        public RelatedModel() {
            this("", 0.0);
        }

        public RelatedModel(String name, double value) {
            super("RelatedModel");
            init(name, value);
        }

        private void init(String name, double value) {
            append(new StringModel(name, "Name"));
            append(new DoubleModel(value, "Value"));
        }
    }
}

