//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelCommand;
import com.nexirius.framework.dataviewer.DataViewer;

import javax.swing.*;

/**
 * This class is used to define the position and size of a CommandViewer component.
 *
 * @see com.nexirius.framework.dataviewer.CommandViewer
 */
public class ButtonLayoutItem extends DefaultLayoutItem {



    public ButtonLayoutItem() {
        super();
    }

    /**
     * @param field The associated method command name
     * @param x     the x position of the component
     * @param y     the y position of the component
     * @param w     the width of the component (if  0 then the preferred width is used)
     * @param h     the height of the component (if  0 then the preferred height is used)
     */
    public ButtonLayoutItem(String field, int x, int y, int w, int h) {
        super(field, x, y, w, h);
    }

    /**
     * @param field    The associated structure field name
     * @param x        the x position of the component
     * @param y        the y position of the component
     * @param w        the width of the component (if  0 then the preferred width is used)
     * @param h        the height of the component (if  0 then the preferred height is used)
     * @param viewerId Use the specified id as the component name (this name is used with DataViewer.getSubcomponent)
     * @see com.nexirius.framework.dataviewer.DataViewer#getSubcomponent
     */
    public ButtonLayoutItem(String field, int x, int y, int w, int h, String viewerId) {
        this(field, x, y, w, h);
        setViewerId(viewerId);
    }

    /**
     * @param field    The associated structure field name
     * @param x        the x position of the component
     * @param y        the y position of the component
     * @param w        the width of the component (if  0 then the preferred width is used)
     * @param h        the height of the component (if  0 then the preferred height is used)
     * @param viewerId Use the specified id as the component name (this name is used with DataViewer.getSubcomponent)
     * @param prop     Any property changer
     * @see com.nexirius.framework.dataviewer.DataViewer#getSubcomponent
     */
    public ButtonLayoutItem(String field, int x, int y, int w, int h, String viewerId, PropertyEntry prop) {
        super(field, x, y, w, h, viewerId);
        setProperties(prop);
    }

    /**
     * Create and layout the button component
     *
     * @param parent_viewer The parent viewer which displays the newly created component on its panel.
     */
    public void doLayout(DataViewer parent_viewer) {
        try {
            DataModel parent_model = parent_viewer.getDataModel();
            // look for the associated model
            DataModelCommand method = parent_model.getMethod(getModelFieldName());

            // create the viewer
            DataViewer viewer = parent_viewer.getFactory().createDefaultEditor(method);

            // create a JComponent from the associated viewer
            JComponent comp = viewer.getJComponent();

            parent_viewer.add(comp);

            // move and resize the JComponent
            defaultLayout(comp, parent_viewer.getFactory());
        } catch (Exception ex) {
            throw new RuntimeException(ex);

            // ignore
        }
    }
}



