//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.Viewable;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.LayoutItem;
import com.nexirius.framework.dataviewer.ViewerCreator;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.framework.swing.ClientResourceTitledBorder;

import javax.swing.*;

/**
 * This class defines the layout of a structure field.
 *
 * @author Marcel Baumann
 */
public class StructureFieldLayout extends DefaultLayoutItem {



    ViewerCreator viewerCreator = null;
    DefaultLayoutItem layout = null;
    boolean createLabel;

    public StructureFieldLayout() {
        super();
    }

    /**
     * @param field         The associated structure field name
     * @param x             the x position of the component
     * @param y             the y position of the component
     * @param w             the width of the component (if twidth and theight are both 0 then they are ignored)
     * @param h             the height of the component (if twidth and theight are both 0 then they are ignored)
     * @param viewerCreator the viewer creator which is used to create the component or null (default viewer from the model)
     * @param layout        the layout which is used to layout the component or null (default layout from the model)
     * @param createLabel   creates a frame with the label when true
     */
    public StructureFieldLayout(String field, int x, int y, int w, int h, ViewerCreator viewerCreator, DefaultLayoutItem layout, boolean createLabel) {
        super(field, x, y, w, h);

        this.viewerCreator = viewerCreator;

        setLayoutItem(layout);

        this.createLabel = createLabel;
    }

    /**
     * @param viewerId is used to assign a specific name to the created JComponent (The JComponent can be accessed with DataViewer.getSubcomponent())
     */
    public StructureFieldLayout(String field, int x, int y, int w, int h, ViewerCreator viewerCreator, DefaultLayoutItem layout, boolean createLabel, String viewerId) {
        this(field, x, y, w, h, viewerCreator, layout, createLabel);

        setViewerId(viewerId);
    }

    /**
     * @param viewerId is used to assign a specific name to the created JComponent (The JComponent can be accessed with DataViewer.getSubcomponent())
     * @param prop     a SinglePropertyEntry or a MultiplePropertyEntry
     * @see SinglePropertyEntry
     * @see MultiplePropertyEntry
     */
    public StructureFieldLayout(String field, int x, int y, int w, int h, ViewerCreator viewerCreator, DefaultLayoutItem layout, boolean createLabel, String viewerId, PropertyEntry prop) {
        this(field, x, y, w, h, viewerCreator, layout, createLabel, viewerId);

        setProperties(prop);
    }

    public StructureFieldLayout(DefaultLayoutItem item) {
        super();
        setLayoutItem(item);
    }

    public LayoutItem getLayoutItem() {
        return layout;
    }

    public void setLayoutItem(DefaultLayoutItem item) {
        layout = item;
    }

    public void removeLayout() {
        layout = null;
    }

    protected void createFieldName(ViewerFactory factory, String text, JComponent panel) {
        ClientResourceTitledBorder tb = new ClientResourceTitledBorder(factory.getClientResource(), text);

        panel.setBorder(tb);
    }

    public void doLayout(DataViewer parent_viewer) {
        try {
            DataModel parent_model = parent_viewer.getDataModel();
            // look for the associated model
            Viewable model = parent_model.getViewableChild(getModelFieldName());
            DataViewer viewer = null;

            if (viewerCreator == null) {
                viewer = parent_viewer.getFactory().createViewer(model, parent_viewer.isEditor());
            } else {
                viewer = parent_viewer.getFactory().createViewer(viewerCreator, model);
            }

            // layout the associated model

            viewer.setLayout(layout);

            // create a JComponent from the associated viewer
            JComponent comp = viewer.getJComponent();

            parent_viewer.add(comp);

            if (createLabel) {
                createFieldName(parent_viewer.getFactory(), model.getFieldName(), comp);
            }

            // move and resize the JComponent
            defaultLayout(comp, parent_viewer.getFactory());

            viewer.updateUI();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
            // ignore
        }
    }
}
