//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.remote;

import com.nexirius.framework.datamodel.*;

import java.io.PushbackInputStream;

/**
 * @author Marcel Baumann
 */
public class DataModelMethodEvent extends DataModelRemoteEvent {



    public DataModelMethodEvent(DataModelEvent event)
            throws Exception {
        if (event == null) {

            return;
        }

        DataModel transmitter = event.getTransmitter();

        if (transmitter == null) {

            transmitter = event.getDataModel();
        }

        if (transmitter == null || !(transmitter instanceof RemoteDataModel)) {

            return;
        }

        RemoteDataModel remote = (RemoteDataModel) transmitter;

        append(new IntModel(remote.getID(), ID));
        append(new StringModel(event.getChildName(), CHILD_NAME));
        append(new IntModel(event.getSubtype(), SUBTYPE));
        append(new StringModel(event.getData(), VALUE));
    }

    public DataModelMethodEvent(PushbackInputStream in)
            throws Exception {
        initMembers();
        readDataFrom(in);
    }

    private void initMembers() {
        append(new IntModel(-1, ID));
        append(new StringModel("", CHILD_NAME));
        append(new StringModel("", VALUE));
    }

    public String applyOn(DataModel model)
            throws Exception {
        DataModel target = model;
        String childName = getChildName();
        int subtype = getSubtype();

        if (childName.length() > 0) {
            target = model.getChild(childName);

            if (target == null) {
                throw new Exception("No child '" + childName + "' on " + model.getClass().getName());
            }
        }

        if (target != null) {
            DataModelCommand meth = target.getMethod(getNewValue());

            if (meth != null) {
                meth.doAction();

                return "OK";
            } else {
                throw new Exception("No method '" + getNewValue() + "' on " + model.getClass().getName());
            }
        }

        return "FAILED";
    }
}

