package com.nexirius.framework.rest;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.json.IJsonFormatMap;
import com.nexirius.framework.datamodel.xml.DataModelXmlGenerator;
import com.nexirius.framework.rest.interfaces.IJnexRestParameterExtractor;
import com.nexirius.framework.rest.interfaces.IJnexRestParameters;
import com.nexirius.framework.rest.interfaces.IJnexRestRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;

public class JnexRestRequest implements IJnexRestRequest {
    private HttpServletRequest request;
    private HttpServletResponse response;
    private static final String CHARSET = "UTF-8";

    public JnexRestRequest(HttpServletRequest request, HttpServletResponse response) {
        this.request = request;
        this.response = response;
    }

    public IJnexRestParameters getParameters(IJnexRestParameterExtractor parameterExtractor) {
        if (parameterExtractor != null) {
            return parameterExtractor.extractParameters(request);
        }

        if (isGet()) {
            String function = request.getParameter("function");
            String argument = request.getParameter("argument");

            return new JnexRestParameters(function, argument);
        } else if (isPost()) {
            throw new RuntimeException("cannot extract rest parameters from http post request");
        }

        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isPost() {
        return request.getMethod().equalsIgnoreCase("post");
    }

    public boolean isGet() {
        return request.getMethod().equalsIgnoreCase("get");
    }

    public OutputStream getOutputStream(String mimeType) throws IOException {
        response.setContentType(mimeType);

        return response.getOutputStream();
    }

    public void reply(String mimeType, Object data) throws IOException {
        if (data instanceof String) {
            OutputStream out = getOutputStream(mimeType);
            out.write(((String) data).getBytes(CHARSET));
        } else if (data instanceof InputStream) {
            InputStream in = (InputStream) data;
            OutputStream out = getOutputStream(mimeType);
            int bufSize = 102400;
            byte[] buf = new byte[bufSize];
            while (true) {
                int n = in.read(buf, 0, buf.length);

                if (n > 0) {
                    out.write(buf, 0, n);
                } else {
                    break;
                }
            }
        }
    }

    public void replyJson(DataModel model, IJsonFormatMap formatMap) throws Exception {
        OutputStream out = getOutputStream(MIME_JSON);

        formatMap.getJsonFormat(model).writeJson(model, out);
    }

    public void replyXml(DataModel model, DataModelXmlGenerator xmlGenerator) throws IOException {
        OutputStream out = getOutputStream(MIME_XML);

        xmlGenerator.generateXml(out, model);
    }

    public HttpServletRequest getHttpServletRequest() {
        return request;
    }

    public HttpServletResponse getHttpServletResponse() {
        return response;
    }
}
