//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.util.assertion.Assert;
import com.nexirius.util.resource.ClientResource;
import com.nexirius.util.resource.ResourceChangeEvent;

import javax.swing.*;
import javax.swing.text.DefaultCaret;

import java.awt.*;


/**
 * A JTextArea which responds to changes in client resources.
 * Changes can include tool tip text.
 * <p/>
 * Property name           Component Property
 * -------------           ------------------
 * resourceKey.tip         tooltiptext
 * <p/>
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1999.03.03  MB              Created
 */
public class CFJTextArea
        extends JTextArea
        implements CFJItem {



    private static Insets defaultInsets = new Insets(1, 3, 1, 3);

    // The key used to look up specific resources
    private String resourceKey;

    /**
     * Construct a CFJTextArea  (Client Resource JTextArea)
     * Need a null constructor so we can use this class in a GUI builder
     */
    public CFJTextArea() {
        super();
        setMargin(defaultInsets);
        initCaretBehaviour();
    }

    /**
     * Construct a CFJTextArea (Client Resource JTextArea)
     *
     * @param resource    The resource to be used to establish basic attributes
     *                    of the JTextArea
     * @param resourceKey The key to be used in establishing resources
     */
    public CFJTextArea(ClientResource resource, String resourceKey) {
        super(resourceKey);
        initCaretBehaviour();
        Assert.pre(resource != null, "Parameter resource is not null");
        Assert.pre(resourceKey != null, "Parameter resourceKey is not null");

        this.resourceKey = resourceKey;

        update(resource);
        resource.addResourceChangeListener(this);
/*
		Keymap map = getKeymap();

		KeyStroke tab = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_TAB, 0);
		KeyStroke ctrltab = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_TAB, java.awt.Event.CTRL_MASK);

		Action act = map.getAction(ctrltab);

		map.removeKeyStrokeBinding(tab);
		map.addActionForKeyStroke(tab, act);
*/
        setMargin(defaultInsets);
    }

    protected void initCaretBehaviour() {
        //DefaultCaret caret = (DefaultCaret) getCaret();
        //caret.setUpdatePolicy(DefaultCaret.NEVER_UPDATE);
    }
/*
	public boolean isManagingFocus()
	{
		return false;
	}
*/
/*
	class GoNextElement extends AbstractAction
	{
		public void actionPerformed(java.awt.event.ActionEvent e)
		{
			Component c = getNextFocusableComponent();

			if (c != null) {
				c.requestFocus();
			}
			//transferFocus();
		}
	}
*/
    /**
     * Get the resource key
     *
     * @return the resource key
     */
    public String getResourceKey() {
        return this.resourceKey;
    }

    /**
     * Set the resource key
     *
     * @param resourceKey the resource key
     */
    public void setResourceKey(String resourceKey) {
        Assert.pre(resourceKey != null, "Parameter resourceKey is not null");
        this.resourceKey = resourceKey;
    }


    /**
     * @see import com.nexirius.util.resource.ResourceChangeListener;
     */
    public void resourceChange(ResourceChangeEvent event) {
        update(event.getClientResource());
    }


    /**
     * Update properties based on client resource
     *
     * @param clientResource the resource to be used to establish properties
     */
    public void update(ClientResource clientResource) {
        if (resourceKey == null) {
            return;
        }

        final String tip = clientResource.getToolTipText(resourceKey);
        if (tip != null) {
            setToolTipText(tip);
        }

        /* Repaint is not required for this component, only for tool tip text */
/*      invalidate();
        validate();
        repaint();
*/
    }
    
    @Override
    public void setText(String t) {
        super.setText(t);
        if(!this.hasFocus()) {
            this.setCaretPosition(0);
        }
    }
}
