//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.textselector;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.DataModel;

public interface ModelSelector extends TextSelector {
    /**
     * redefine this method to represent different status models
     *
     * @return
     */
    public DataModel createModel();

    /**
     * access the fieldname of the field which holds the primary key
     *
     * @return
     */
    public String getIdFieldName();

    /**
     * registerTemplate new entry (must call setModel(model))
     */
    public void addModel(DataModel model);

    /**
     * get a possible completion for a given string
     *
     * @param fieldName the field name of the field which is used as a search pattern
     * @param s         the search pattern
     * @return the actual possible completion or null if none of the entries starts with the given string
     */
    public DataModel getCompletionFor(String fieldName, String s);

    /**
     * get all possible completions for a given string
     *
     * @param fieldName the field name of the field which is used as a search pattern
     * @param s         the search pattern
     * @param result    the resulting list of strings will be appended to this list
     * @return true if there are more than the returned number of possible completions
     */
    public boolean getCompletionListFor(String fieldName, String s, ArrayModel result);

    /**
     * access the model which has been selected
     */
    public DataModel getModel();

    /**
     * set the model which will be selected
     */
    public void setModel(DataModel model);
}
