//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import javax.swing.*;
import java.awt.*;
import java.awt.print.PageFormat;

public class ImagePrintable implements FixPrintable {



    Image image = null;
    Point location;
    int width;

    public ImagePrintable(Image image, Point location, int width) {
        this.image = image;
        this.location = location;
        this.width = width;
    }

    public ImagePrintable(String imagename, Point location, int width) {
        this(new ImageIcon(imagename).getImage(), location, width);
    }

    public int print(Graphics gc, PageFormat pageFormat, int pageIndex) {
        int xPage = (int) pageFormat.getImageableX();
        int yPage = (int) pageFormat.getImageableY();
        int wPage = (int) pageFormat.getImageableWidth();
        int hPage = (int) pageFormat.getImageableHeight();

        int wi = image.getWidth(null);
        int hi = image.getHeight(null);

        Point p = translate(location, xPage, yPage, wPage, hPage);
        int w = recalc(width, 0, wPage);
        int h = (w * hi) / wi;

        gc.drawImage(image, p.x, p.y, w, h, null);

        return PAGE_EXISTS;
    }

    public static Point translate(Point p, int x, int y, int w, int h) {
        return new Point(recalc(p.x, x, w), recalc(p.y, y, h));
    }

    public static int recalc(int v, int offset, int width) {
        return (v * width) / 100 + offset;
    }
}
