//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import java.awt.*;
import java.awt.print.Printable;
import java.util.Vector;

public class PrintTable {



    public static final int HEADER_ZONE = -1;
    public static final int DATA_ZONE = 0;
    public static final int FOOTER_ZONE = 1;

    protected PrintTableTemplate template = null;
    protected int width;
    protected PrintTableRowList rows;
    protected Point location;
    protected PrintTableDataAccessor data;
    protected int height = 0;

    public PrintTable(Point location, PrintTableTemplate t, PrintTableDataAccessor data, int width) {
        this.location = location;
        this.data = data;
        template = t;
        this.width = width;
    }

    public void set(PrintTableTemplate t, PrintTableDataAccessor data) {
        this.data = data;
        template = t;
    }

    public PrintTableDataAccessor getData() {
        return data;
    }

    public Printable[] preparePages(int minY, int maxY, int pageIndex) {
        int rowCount = 0;

//System.out.println("preparePages minY=" + minY + " maxY = " + maxY + " pageIndex=" + pageIndex);
        Vector pageList = new Vector();
        Point p = new Point(location);

        PrintTablePart page = new PrintTablePart(p, this, rowCount, pageIndex);

        rowCount = page.preparePage(rowCount, p, maxY);

        pageList.addElement(page);
        height = page.getHeight();

        ++pageIndex;
//System.out.println("Page " + v.size() + " row = " + rowCount + " location = " + p);

// continue on new page
        while (rowCount >= 0) {
            p = new Point(location);
            p.y = minY;

            page = new PrintTablePart(p, this, rowCount, pageIndex);

            rowCount = page.preparePage(rowCount, p, maxY);

            pageList.addElement(page);
            height = page.getHeight();
            ++pageIndex;
//System.out.println("Page " + v.size() + " row = " + rowCount + " location = " + p);
        }

        return (PrintTablePart[]) pageList.toArray(new PrintTablePart[pageList.size()]);
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }
}

