//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import javax.swing.*;
import java.awt.*;

public class PrintTableCell {



    protected PrintTableRow parent;
    protected PrintTableCellTemplate template;

    protected String text;
    protected int height = 0;
    protected int width = 0;
    protected JComponent component = null;

    public PrintTableCell(PrintTableRow p, PrintTableCellTemplate t) {
        parent = p;
        template = t;
    }

    public PrintTableDataAccessor getData() {
        return parent.getData();
    }

    public int preparePrint(int x, int y) {
        int rowWidth = parent.getWidth();
        int mtop = template.getMarginTop(rowWidth);
        int mbottom = template.getMarginBottom(rowWidth);
        int mleft = template.getMarginLeft(rowWidth);
        int mright = template.getMarginRight(rowWidth);

        width = template.getWidth(rowWidth);

        int areawidth = width - mleft - mright;

        if (areawidth < 0) {
            areawidth = 0;
        }

        text = getData().get(parent.getZone(), template.getColumnName(), y, parent.getPageIndex());

        int alignment = template.getAlignment();

        if (template.isHtml()) {
            component = new JEditorPane("text/html", text);
        } else {
            if (alignment == SwingConstants.LEFT) {
                JTextArea area = new JTextArea(text);

                area.setLineWrap(true);
                area.setWrapStyleWord(true);

                component = area;
            } else {
                JLabel label = new JLabel(text);

                label.setHorizontalAlignment(alignment);

                component = label;
            }
            component.setFont(template.getFont(rowWidth));
        }

        component.setSize(areawidth, 30);
        component.setVisible(true);

        component.setLocation(mleft, mtop);
        component.setSize(areawidth, component.getPreferredSize().height);

        height = component.getPreferredSize().height + mtop + mbottom;

        return height;
    }

    public void print(Point location, Graphics gc) {
        Color bg = template.getBackground();

        if (bg != null) {
            Color oldColor = gc.getColor();

            component.setBackground(bg);
            gc.setColor(bg);
            gc.fillRect(location.x, location.y, width, parent.getHeight());
            gc.setColor(oldColor);
        }

        if (template.getUsesPageIndex()) //FIX check for Property USES_PAGE_INDEX
        {
            if (component instanceof JLabel) {
                text = getData().get(parent.getZone(), template.getColumnName(), parent.getRow(), parent.getPageIndex());
                ((JLabel) component).setText(text);
            }
        }

        int compX = location.x + component.getLocation().x;
        int compY = location.y + component.getLocation().y;

        gc.translate(compX, compY);
        component.printAll(gc);
        gc.translate(-compX, -compY);

        if (!template.getNoFrame()) {
            gc.drawRect(location.x, location.y, width, parent.getHeight());
        }
    }

    public int getWidth() {
        return width;
    }
}
