//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import java.awt.*;
import java.util.Hashtable;

public class PrintTableCellProperties {



    public static final String NAME = "NAME";

    public static final String COLUMN_NAME = "COLUMN_NAME";
    public static final String FONTFACE = "FONTFACE";
    public static final String FONTSIZE = "FONTSIZE";
    public static final String FONTSTYLE = "FONTSTYLE";
    public static final String ALIGNMENT = "ALIGNMENT";
    public static final String LINE_SPACING = "LINE_SPACING";
    public static final String TEXTCOLOR = "TEXTCOLOR";
    public static final String USES_PAGE_INDEX = "USES_PAGE_INDEX";
    public static final String BGCOLOR = "BGCOLOR";
    public static final String SHADING = "SHADING";
    public static final String WIDTH = "WIDTH";
    public static final String WIDTH_PERCENT = "WIDTH_PERCENT";
    public static final String HTML = "HTML";

    public static final String MARGINTOP = "MARGINTOP";
    public static final String MARGINBOTTOM = "MARGINBOTTOM";
    public static final String MARGINLEFT = "MARGINTOP";
    public static final String MARGINRIGHT = "MARGINRIGHT";

    public static final String NO_FRAME = "NO_FRAME";
    public static final String FRAMETOP = "FRAMETOP";
    public static final String FRAMEBOTTOM = "FRAMEBOTTOM";
    public static final String FRAMELEFT = "FRAMETOP";
    public static final String FRAMERIGHT = "FRAMERIGHT";

    PrintTableCellProperties parent = null;
    Hashtable props = new Hashtable();

    public PrintTableCellProperties() {
    }

    public PrintTableCellProperties(PrintTableCellProperties parentProperties) {
        parent = parentProperties;
    }

    public void setParentProperties(PrintTableCellProperties parentProperties) {
        parent = parentProperties;
    }

    public PrintTableCellProperty getProperty(String name) {
        PrintTableCellProperty ret = (PrintTableCellProperty) props.get(name);

        if (ret == null) {

            if (parent != null) {

                return parent.getProperty(name);
            }
        }

        return ret;
    }

    public void setProperty(String name, PrintTableCellProperty prop) {
        props.put(name, prop);
    }

    public void setProperty(String name, String prop) {
        props.put(name, new PrintTableCellProperty(prop, 0));
    }

    public void setProperty(String name, double prop) {
        props.put(name, new PrintTableCellProperty(null, prop));
    }

    public void setProperty(String name) {
        props.put(name, new PrintTableCellProperty(null, 0));
    }

    public void setWidthPercent(int widthPercent) {
        setProperty(WIDTH_PERCENT, widthPercent);
    }

    public void setInsets(Insets insets) {
        setProperty(MARGINTOP, insets.top);
        setProperty(MARGINRIGHT, insets.right);
        setProperty(MARGINBOTTOM, insets.bottom);
        setProperty(MARGINLEFT, insets.left);
    }

    public void setFont(Font font) {
        setProperty(FONTFACE, font.getFamily());
        setProperty(FONTSIZE, font.getSize2D());
        setProperty(FONTSTYLE, font.getStyle());
    }

    public void setHtml() {
        setProperty(HTML, 1);
    }

    public void setNoFrame() {
        setProperty(NO_FRAME);
    }

    //public int getMaxColumns(int width);
    //public int getMaxRows(int height);
}

