//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import javax.swing.*;
import java.awt.*;

public class PrintTableCellTemplate {



    protected PrintTableCellProperties cellProperties;

    public PrintTableCellTemplate(PrintTableCellProperties cellProperties, PrintTableCellProperties parentProperties) {
        if (cellProperties == null) {
            this.cellProperties = new PrintTableCellProperties(parentProperties);
        } else {
            this.cellProperties = cellProperties;
            this.cellProperties.setParentProperties(parentProperties);
        }
    }

    public String getColumnName() {
        PrintTableCellProperty prop = cellProperties.getProperty(PrintTableCellProperties.COLUMN_NAME);

        if (prop == null) {

            return null;
        }

        return prop.s;
    }

    public int getWidth(int rowWidth) {
        PrintTableCellProperty prop = cellProperties.getProperty(PrintTableCellProperties.WIDTH);

        if (prop == null) {
            prop = cellProperties.getProperty(PrintTableCellProperties.WIDTH_PERCENT);

            if (prop == null) {
                return 0;
            }

            return (int) (rowWidth * prop.d / 100.0);
        }

        return 0;
    }

    public Font getFont(int rowWidth) {
        PrintTableCellProperty pface = cellProperties.getProperty(PrintTableCellProperties.FONTFACE);
        PrintTableCellProperty psize = cellProperties.getProperty(PrintTableCellProperties.FONTSIZE);
        PrintTableCellProperty pstyle = cellProperties.getProperty(PrintTableCellProperties.FONTSTYLE);

        String face = "Default";
        int size = getIntProperty(rowWidth, PrintTableCellProperties.FONTSIZE, 10);
        int style = Font.PLAIN;

        if (pface != null) {
            face = pface.s;
        }

        if (pstyle != null) {
            style = (int) pstyle.d;
        }

        return new Font("Default", style, size);
    }

    public Color getBackground() {
        int shade = getIntProperty(100, PrintTableCellProperties.SHADING, -1);

        if (shade < 0) {

            return null;
        }

        int rgb = ((100 - shade) * 255) / 100;

        return new Color(rgb, rgb, rgb);
    }

    public int getMarginTop(int rowWidth) {
        return getIntProperty(rowWidth, PrintTableCellProperties.MARGINTOP, 0);
    }

    public int getMarginBottom(int rowWidth) {
        return getIntProperty(rowWidth, PrintTableCellProperties.MARGINBOTTOM, 0);
    }

    public int getMarginLeft(int rowWidth) {
        return getIntProperty(rowWidth, PrintTableCellProperties.MARGINLEFT, 0);
    }

    public int getMarginRight(int rowWidth) {
        return getIntProperty(rowWidth, PrintTableCellProperties.MARGINRIGHT, 0);
    }

    public int getAlignment() {
        PrintTableCellProperty p = cellProperties.getProperty(PrintTableCellProperties.ALIGNMENT);

        if (p != null) {
            return (int) p.d;
        }

        return SwingConstants.LEFT;
    }

    public int getIntProperty(int rowWidth, String property, int def) {
        PrintTableCellProperty p = cellProperties.getProperty(property);

        if (p != null) {
            return (int) (rowWidth * p.d / 100.0);
        }

        return def;
    }

    public boolean getNoFrame() {
        return null != cellProperties.getProperty(PrintTableCellProperties.NO_FRAME);
    }

    public boolean getUsesPageIndex() {
        return null != cellProperties.getProperty(PrintTableCellProperties.USES_PAGE_INDEX);
    }

    public boolean isHtml() {
        return null != cellProperties.getProperty(PrintTableCellProperties.HTML);
    }
}
