//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import java.awt.*;

public class PrintTableRow {



    protected boolean pageBrake = false;
    protected PrintTable parent;
    protected PrintTableRowTemplate template;
    protected int zone;
    protected int maxHeight = 0;
    protected PrintTableCellList cells = new PrintTableCellList();
    protected int row = 0;
    protected int pageIndex = 0;

    public PrintTableRow(PrintTable parent, PrintTableRowTemplate template, int zone) {
        this.parent = parent;
        this.template = template;
        this.zone = zone;
    }

    public PrintTableDataAccessor getData() {
        return parent.getData();
    }

    public int getWidth() {
        return parent.getWidth();
    }

    public int getHeight() {
        return maxHeight;
    }

    public int getPageIndex() {
        return pageIndex;
    }

    public int getRow() {
        return row;
    }

    public int preparePrint(int y, int pageIndex) {
        this.pageIndex = pageIndex;
        row = y;
        PrintTableCellTemplate cellTemplate = template.getFirstCellTemplate();

        for (int x = 0; x < template.getColumns() && cellTemplate != null; ++x) {
            PrintTableCell cell = new PrintTableCell(this, cellTemplate);

            int h = cell.preparePrint(x, y);

            maxHeight = Math.max(h, maxHeight);

            cells.append(cell);

            cellTemplate = template.getNextCellTemplate();
        }

        return getHeight();
    }

    public void print(Point location, Graphics gc) {
        int x = 0;
        Point p = new Point(location);

        for (PrintTableCell c = cells.first(); c != null; c = cells.next()) {
            c.print(p, gc);

            p.translate(c.getWidth(), 0);
        }
    }

    public int getZone() {
        return zone;
    }
}
