//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;



public class PrintTableTemplate {
    protected PrintTableCellProperties defaultCellProperties;
    protected PrintTableRowTemplate header;
    protected PrintTableRowTemplate row;
    protected PrintTableRowTemplate footer;

    public PrintTableTemplate() {
        this(new PrintTableCellProperties());
    }

    public PrintTableCellProperties getDefaultCellProperties() {
        return defaultCellProperties;
    }

    public PrintTableTemplate(PrintTableCellProperties properties) {
        defaultCellProperties = properties;
    }

    public void setDefaultCellProperties(PrintTableCellProperties defaultCellProperties) {
        this.defaultCellProperties = defaultCellProperties;
    }

    public void setRow(PrintTableCellProperties cellProperties) {
        if (cellProperties == null) {
            cellProperties = new PrintTableCellProperties(cellProperties);
        } else {
            cellProperties.setParentProperties(defaultCellProperties);
        }

        row = new PrintTableRowTemplate(cellProperties);
    }

    public void appendCellToRow(PrintTableCellProperties cellProperties) {
        if (row == null) {
            row = new PrintTableRowTemplate(defaultCellProperties);
        }

        row.appendCell(cellProperties);
    }

    public void setHeader(PrintTableCellProperties cellProperties) {
        if (cellProperties == null) {
            cellProperties = new PrintTableCellProperties(cellProperties);
        } else {
            cellProperties.setParentProperties(defaultCellProperties);
        }

        header = new PrintTableRowTemplate(cellProperties);
    }

    public void appendCellToHeader(PrintTableCellProperties cellProperties) {
        if (header == null) {
            header = new PrintTableRowTemplate(defaultCellProperties);
        }

        header.appendCell(cellProperties);
    }

    public PrintTableRow createHeader(PrintTable parent) {
        if (header != null) {

            return new PrintTableRow(parent, header, PrintTable.HEADER_ZONE);
        }

        return null;
    }

    public PrintTableRow createRow(PrintTable parent) {
        if (row != null) {

            return new PrintTableRow(parent, row, PrintTable.DATA_ZONE);
        }

        return null;
    }

    public PrintTableRow createFooter(PrintTable parent) {
        if (footer != null) {

            return new PrintTableRow(parent, footer, PrintTable.FOOTER_ZONE);
        }

        return null;
    }
}
