//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print.example;

import com.nexirius.print.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;

public class PrintExample {



    static class CloseListener extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    public static void main(String argv[]) {
/*
		String fonts[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();

		for (int n = 0; n<fonts.length; ++n)
		{
			System.out.println(fonts[n]);
		}
*/
        JFrame frame = new JFrame("TEST");

        frame.addWindowListener(new CloseListener());

        String fields[] = {"Row1", "Row2"};
        PrintTableDataAccessor data = new PrintTableDataAccessor(new ExampleData(), fields);
        PrintPreview printPreview = new PrintPreview();

        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(BorderLayout.CENTER, printPreview);
        frame.setVisible(true);

        PrinterJob prnJob = PrinterJob.getPrinterJob();

        if (!prnJob.printDialog()) {

            return;
        }

        PageFormat pf = prnJob.defaultPage();

        MultiPage multiPage = new MultiPage();
        PageTemplate pageTemplate = new PageTemplate(new Rectangle(0, 12, 100, 88), pf);

        //pageTemplate.add(new FixPrintableTable(new Rectangle(0, 0, 100, 10), pf, new Example2Template(), new PrintTableDataAccessor(new Example2Data(), null)));
        pageTemplate.add(new DefaultFixPrintable(pf, "LINKS", "ZENTRUM", "RECHTS"));
        pageTemplate.add(new ImagePrintable("NexiriusPaper.gif", new Point(80, 0), 20));


        Point location = new Point(pageTemplate.getFlowBounds().x, pageTemplate.getFlowBounds().y);
        PrintTable table = new PrintTable(location, new ExampleTemplate(), data, pageTemplate.getFlowBounds().width);

        multiPage.append(table, pageTemplate);
        PrintTable table2 = new PrintTable(new Point(location.x + pageTemplate.getFlowBounds().width / 2, multiPage.getCurrentFlowY()), new ExampleTemplate(), data, pageTemplate.getFlowBounds().width / 2);
        multiPage.append(table2, pageTemplate);

        printPreview.setPageable(multiPage, prnJob, pf);
        prnJob.setPageable(multiPage);
        frame.pack();
    }
}

class ExampleTemplate extends PrintTableTemplate {
    public ExampleTemplate() {
        super(new PrintTableCellProperties());

        defaultCellProperties.setProperty(PrintTableCellProperties.FONTFACE, "Courier");
        defaultCellProperties.setProperty(PrintTableCellProperties.FONTSIZE, 2);

        PrintTableCellProperties cp1 = new PrintTableCellProperties();
        PrintTableCellProperties cp2 = new PrintTableCellProperties();

        cp1.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 40);
        cp1.setProperty(PrintTableCellProperties.MARGINTOP, 3);
        cp1.setProperty(PrintTableCellProperties.MARGINBOTTOM, 5);
        cp1.setProperty(PrintTableCellProperties.MARGINLEFT, 5);
        cp1.setProperty(PrintTableCellProperties.MARGINRIGHT, 5);
        cp1.setProperty(PrintTableCellProperties.COLUMN_NAME, "Name");
        cp1.setProperty(PrintTableCellProperties.SHADING, 10);
        cp1.setProperty(PrintTableCellProperties.FONTFACE, "Arial");
        cp1.setProperty(PrintTableCellProperties.FONTSIZE, 4);
        cp2.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 60);
        cp2.setProperty(PrintTableCellProperties.COLUMN_NAME, "Email");
        cp2.setProperty(PrintTableCellProperties.SHADING, 20);
        //cp2.setProperty(PrintTableCellProperties.NO_FRAME);
        cp2.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.RIGHT);

        appendCellToRow(cp1);
        appendCellToRow(cp2);

        PrintTableCellProperties cpt1 = new PrintTableCellProperties();
        PrintTableCellProperties cpt2 = new PrintTableCellProperties();

        cpt1.setProperty(PrintTableCellProperties.FONTFACE, "Default");
        cpt1.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 40);
        cpt1.setProperty(PrintTableCellProperties.COLUMN_NAME, "Name");
        cpt1.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.CENTER);
        cpt2.setProperty(PrintTableCellProperties.FONTFACE, "Default");
        cpt2.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 60);
        cpt2.setProperty(PrintTableCellProperties.COLUMN_NAME, "Email");
        cpt2.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.CENTER);
        appendCellToHeader(cpt1);
        appendCellToHeader(cpt2);
    }
}

class Example2Template extends PrintTableTemplate {
    public Example2Template() {
        super(new PrintTableCellProperties());

        defaultCellProperties.setProperty(PrintTableCellProperties.FONTFACE, "Default");
        defaultCellProperties.setProperty(PrintTableCellProperties.FONTSIZE, 2);
        defaultCellProperties.setProperty(PrintTableCellProperties.MARGINLEFT, 1);
        defaultCellProperties.setProperty(PrintTableCellProperties.MARGINRIGHT, 1);

        PrintTableCellProperties cp1 = new PrintTableCellProperties();
        PrintTableCellProperties cp2 = new PrintTableCellProperties();
        PrintTableCellProperties cp3 = new PrintTableCellProperties();

        cp1.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 20);
        cp1.setProperty(PrintTableCellProperties.COLUMN_NAME, "FIX1");
        cp1.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.LEFT);

        cp2.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 60);
        cp2.setProperty(PrintTableCellProperties.COLUMN_NAME, "FIX2");
        cp2.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.CENTER);

        cp3.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 20);
        cp3.setProperty(PrintTableCellProperties.COLUMN_NAME, "FIX31");
        cp3.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.RIGHT);

        appendCellToRow(cp1);
        appendCellToRow(cp2);
        appendCellToRow(cp3);
    }
}

class ExampleData implements PrintTableData {
    public int getRowCount() {
        return 15;
    }

    public String getHeader(String colName) {
        return "HEADER: " + colName;
    }

    public String getData(String colName, int row, int pageIndex) {
        if (row % 4 == 0) {
            return "The quick brown fox\njumps\nover the lazy tree";
        }

        if (colName.equals("Name") && row % 5 == 0) {
            return "My name is not important.\n\txxxx";
        }

        return colName + row + " Page:" + (pageIndex + 1);
    }

    public String getFooter(String colName) {
        return "Footer";
    }
}

class Example2Data implements PrintTableData {
    public int getRowCount() {
        return 1;
    }

    public String getHeader(String colName) {
        return "HEADER: " + colName;
    }

    public String getData(String colName, int row, int pageIndex) {
        return "Example2Data:" + colName + row + " Page:" + (pageIndex + 1);
    }

    public String getFooter(String colName) {
        return "Footer";
    }
}

