//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.tcp;

import com.nexirius.framework.FWLog;

import java.net.ServerSocket;
import java.net.Socket;

// This class implements a simple TCP server that listens
// for incoming connections. It spawns a Client object
// to handle the actual connection.

public class Server implements Runnable {



    // serverSocket is the socket we are listening on
    ServerSocket serverSocket;
    Thread myThread;
    ClientFactory clientFactory;

    public Server(int port, ClientFactory clientFactory) {
        try {
            serverSocket = new ServerSocket(port);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
            // System.exit(1);
        }

        this.clientFactory = clientFactory;
    }

    public void run() {
        int activeThreads = 0;

        while (true) {
            if (activeThreads != Thread.activeCount()) {
                activeThreads = Thread.activeCount();
                System.out.println("THREADS = " + activeThreads);
            }

            try {
                // Accept a new connection
                Socket newConn = serverSocket.accept();

                // Create a client to handle the connection
                Client newClient = clientFactory.createClient(newConn);

                // Start the client (it's runnable)
                newClient.start();

            } catch (Exception e) {
                e.printStackTrace(System.err);
                System.err.println("While accepting connctions");
                try {
                    Thread.currentThread().sleep(1000);
                } catch (Exception e1) {
                }
            }
        }
    }

    public void start() {
        myThread = new Thread(this);
        myThread.start();
    }

    public static void main(String argv[]) {
        int port = 0;

        try {
            if (argv.length != 1) {
                throw new Exception("USAGE: java com.nexirius.tcp.Server port_number");
            }

            port = Integer.parseInt(argv[0]);
        } catch (Exception ex) {
            FWLog.error(ex);
            System.exit(1);
        }

        new Server(port, new ClientFactoryImpl()).start();
    }
}
