package com.nexirius.tools;

import com.nexirius.util.XFile;
import com.nexirius.util.StringVector;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.File;

public class DateFolderMaker extends JFrame {
    private BoundedRangeModel progressModel;
    private JTextField textField;
    private boolean stopped = false;
    private static final SimpleDateFormat SIMPLE_DATE_FORMAT = new SimpleDateFormat("yyyy_MM_dd");

    public DateFolderMaker() throws HeadlessException {
        super("jnex DateFolderMaker");

        getContentPane().setLayout(new BorderLayout());

        init();
    }

    private void init() {
        textField = new JTextField();
        getContentPane().add(textField, BorderLayout.CENTER);
        JProgressBar progressBar = new JProgressBar();
        progressModel = progressBar.getModel();
        getContentPane().add(progressBar, BorderLayout.NORTH);
        JButton start = new JButton("Start");
        JButton stop = new JButton("Stop");
        JPanel buttonPanel = new JPanel(new GridLayout(1, 0));
        buttonPanel.add(stop);
        buttonPanel.add(start);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

        start.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        createDirs();                        
                    }
                }).start();
            }
        });

        stop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stopped = true;
            }
        });
    }

    private void createDirs() {
        stopped = false;
        String dirname = textField.getText();
        XFile dir = new XFile(dirname);

        if (dir.exists() && dir.isDirectory()) {
            StringVector files = dir.getFiles(true);

            if (stopped) {
                return;
            }
            //Calendar calendar = Calendar.getInstance();
            int count = files.size();
            int n = 0;

            for (Iterator iterator = files.iterator(); iterator.hasNext();) {
                String fileName = (String) iterator.next();

                XFile sourceFile = new XFile(dirname, fileName);

                if (sourceFile.canRead()) {
                    long lastModified = sourceFile.lastModified();
                    //calendar.setTime();

                    Date date = new Date(lastModified);

                    File targetDir = new File(dirname, SIMPLE_DATE_FORMAT.format(date));

                    targetDir.mkdirs();

                    try {
                        if (stopped) {
                            return;
                        }
                        ++n;
                        progressModel.setRangeProperties(n, 1, 0, count,  false);
                        XFile targetFile = new XFile(targetDir.getPath(), sourceFile.getName());
                        targetFile.createFrom(sourceFile, false);
                        targetFile.setLastModified(lastModified);
                    } catch (Exception e1) {
                        new JOptionPane(e1.getMessage(), JOptionPane.ERROR);
                    }
                }
            }
        }
    }


    public static void main(String[] args) {
        DateFolderMaker dateFolderMaker = new DateFolderMaker();

        dateFolderMaker.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        dateFolderMaker.setLocation(200, 200);
        dateFolderMaker.pack();
        dateFolderMaker.setVisible(true);
    }
}
