//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.tools.dirsync;

import java.io.File;
import java.net.URL;

public class DirSync {
    /**
     * Copy directory information to a info directory which is used to sync the source directory over http
     *
     * @param sourceDirectory
     * @param infoDirectory
     * @param directoryFilter
     * @param fileFilter
     */
    public static void writeInfoDirectory(String sourceDirectory, String infoDirectory, String directoryFilter, String fileFilter) {
        File infoDir = new File(infoDirectory);

        if (infoDir.exists()) {
            infoDir.delete();
        }

        DirectoryInfoModel sourceModel = new DirectoryInfoModel(sourceDirectory, directoryFilter, fileFilter);

        sourceModel.writeInfoToDirectory(infoDirectory);
    }

    /**
     * Synchronize a local dircetory against a remote directory which is discribed in sourceInfo (used for dir synch over http)
     *
     * @param sourceDirectory URL to the source directory (containing all files which need to be in sync)
     * @param sourceInfo      this URL holds information about the files in the source directory
     * @param targetDirectory the path name of the target directory
     * @throws Exception
     */
    public static void syncDirectory(URL sourceDirectory, URL sourceInfo, String targetDirectory) throws Exception {
        DirectoryInfoModel infoModel = new DirectoryInfoModel(sourceInfo);
        DirectoryInfoModel targetModel = new DirectoryInfoModel(targetDirectory, "+\"*\"", "+\"*\"");

        targetModel.compareTo(infoModel);
        targetModel.sync(sourceDirectory.toString(), targetDirectory, new DefaultDirSyncManager());
    }

    public static void main(String[] argv) throws Exception {
        DirSync.writeInfoDirectory(argv[0], argv[1], "+\"*\"", "+\"*\"");
    }
}
