//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.tools.properties.model;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.DataModelEnumeration;
import com.nexirius.util.StringVector;
import com.nexirius.util.XFile;

public class EntryArrayModel extends ArrayModel {

    public static final String FIELD_NAME = "EntryArray";

    public EntryArrayModel() {
        super(new EntryModel(), FIELD_NAME);
    }

    public void readEntries(String filename) throws Exception {
        XFile file = new XFile(filename);

        StringVector lines = file.getTextLines();

        for (String line = lines.firstItem(); line != null; line = lines.nextItem()) {
            EntryModel entry = new EntryModel();

            if (entry.parse(line)) {
                sortInsert(entry);
            }
        }
    }

    public void saveTo(String filename, String locale) throws Exception {
        XFile file = new XFile(filename);
        DataModelEnumeration e = getEnumeration();
        StringVector lines = new StringVector();

        while (e.hasMore()) {
            EntryModel entry = (EntryModel) e.next();

            String line = entry.getLine(locale);

            if (line != null) {
                lines.append(line);
            }
        }

        file.writeTextLines(lines);
    }

    public void addLocaleEntry(String locale, EntryModel newEntry) {
        DataModelEnumeration e = getEnumeration();

        while (e.hasMore()) {
            EntryModel entry = (EntryModel)e.next();

            if (entry.getKey().equals(newEntry.getKey())) {
                entry.setLocaleText(locale, newEntry.getValueText());
                return ;
            }
        }

        sortInsert(newEntry);
    }
}
