//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.tools.properties.model;

import com.nexirius.framework.datamodel.*;
import com.nexirius.tools.properties.EditProperties;

public class EntryModel extends StructModel {

    StringModel key;
    StringModel value;
    ExtensionComboBoxModel extension;

    public EntryModel() {
        super("Entry");
        init();
    }

    public String getKey() {
        return getChildText("key");
    }

    public static FieldName getTableHeaders() {
        FullFieldName fn = new FullFieldName("Entry");

        fn.append(new BasicFieldName("key"));
        fn.append(new BasicFieldName("value"));

        return fn;
    }

    public boolean parse(String line) {
        int index = line.indexOf('=');

        if (index > 0) {
            key.setText(line.substring(0, index));
            value.setText(line.substring(index + 1));

            return true;
        }

        return false;
    }

    public String getLine(String locale) {
        StringBuffer ret = new StringBuffer(255);

        ret.append(key.getText());
        ret.append('=');

        String localeText = getLocaleText(locale);

        if (locale == null) {
            ret.append(getDefaultValue());
        } else {

            if (localeText.trim().length() == 0 || localeText.trim().equals(value.getText().trim())) {
                return null;
            } else {
                ret.append(localeText);
            }
        }
        
        return ret.toString();
    }

    private String getDefaultValue() {
        String text = value.getText();

        if (text.trim().length() != 0) {
            return text;
        }

        DataModelEnumeration en = getlocales().getEnumeration();

        while (en.hasMore()) {
            LocaleModel lm = (LocaleModel) en.next();

            if (lm.isDefault()) {
                return getLocaleText(lm.getFileExtension());
            }
        }

        return "";
    }

    public int compare(Object o1, Object o2) {
        EntryModel e1 = (EntryModel) o1;
        EntryModel e2 = (EntryModel) o2;

        return -e1.key.getText().compareToIgnoreCase(e2.key.getText());
    }

    private void init() {
        append(key = new StringModel("", "key"));
        append(value = new StringModel("", "value"));
        append(extension = new ExtensionComboBoxModel());

        LocaleArrayModel localeArray = getlocales();

        DataModelEnumeration en = localeArray.getEnumeration();

        while (en.hasMore()) {
            LocaleModel lm = (LocaleModel) en.next();

            append(new StringModel("", lm.getFileExtension()));
        }
    }

    private LocaleArrayModel getlocales() {
        return EditProperties.getInstance().getSettingsModel().getLocaleArrayModel();
    }

    public String getValueText() {
        return value.getText();
    }

    public String getLocaleText(String locale) {
        if (locale == null) {
            return value.getText();
        }

        return getChildText(locale);
    }

    public void setLocaleText(String locale, String text) {
        setChildText(locale, text);
    }

    public String toString() {
        return getLine(null);
    }
}
