//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * ActionServer
 * <p/>
 * a class that calls CommandTartget.execute() on the registered client
 * if an action event (actionPerformed(Event)) occurs.
 * ->client.execute(client, ActionEvent.getActionCommand(), null)
 * ->clinet.execute(client, message, null)
 *
 * @author Marcel Baumann
 */
public class ActionServer implements ActionListener {



    private CommandTarget m_client;
    private String m_message = null;

    /**
     * creates an action server which sends event.getActionCommand to the client if an
     * actionPerformed event arrives
     */
    public ActionServer(CommandTarget client) {
        m_client = client;
    }

    /**
     * creates an action server which sends the given message to the client if an
     * actionPerformed event arrives
     */
    public ActionServer(CommandTarget client, String message) {
        m_client = client;
        m_message = message;
    }

    public void actionPerformed(ActionEvent e) {
        if (m_message == null) {
            m_client.execute(m_client, e.getActionCommand(), null);
        } else {
            m_client.execute(m_client, m_message, null);
        }
    }
}

