//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * Static class utility methods.
 *
 * @author Marcel Baumann
 */
public class ClassUtil {

    public static Object call(String className, String methodName) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        return call(className, methodName, null);
    }

    /**
     * call a static method
     * @param className
     * @param methodName
     * @param args use Classname.class to pass null values as arguments
     * @return
     * @throws ClassNotFoundException
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    public static Object call(String className, String methodName, Object[] args) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        Class aClass = ClassLoader.getSystemClassLoader().loadClass(className);
        Method method = getMethod(aClass, methodName, args);
        return method.invoke(null, args);
    }

    private static Method getMethod(Class aClass, String methodName, Object[] args) throws NoSuchMethodException {
        Class[] parameterTypes = new Class[args == null ? 0 : args.length];
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            parameterTypes[i] = arg instanceof Class ? (Class)arg : arg.getClass();
        }
        return aClass.getMethod(methodName, parameterTypes);
    }

    /**
     * call an instance method (no parameter)
     * @param instance
     * @param methodName
     * @return
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    public static Object call(Object instance, String methodName) throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        return call(instance, methodName, null);
    }

    /**
     * call an instance method (use Classname.class to pass null values as arguments)
     * @param instance
     * @param methodName
     * @param args use Classname.class to pass null values as arguments
     * @return
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    public static Object call(Object instance, String methodName, Object[] args) throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        Class aClass = instance.getClass();
        Method method = getMethod(aClass, methodName, args);
        return method.invoke(instance, args);
    }

    /**
     * strip off path and parent class names
     */
    public static String getShortClassName(Object o) {
        if (o == null) return null;
        String shortName = o.getClass().getName();
        shortName = shortName.substring(shortName.lastIndexOf('.') + 1);
        shortName = shortName.substring(shortName.lastIndexOf('$') + 1);
        return shortName;
    }

    public static void main(String[] args) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException, ClassNotFoundException {
        call(System.out, "println", new Object[] {"klappt"});
        System.out.println("user.home=" + call("java.lang.System", "getProperty", new Object[] {"user.home"}));
    }
}
