//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

/**
 * This interface is used to create a deep copy of a class instance. The instance wich is
 * duplicated uses the parameter instance to find out whether duplicate() was called from a subclass (instance != null)
 * or whether it has to create a copy of its own type (instance = new MyClass()).
 * Subclass intances have to call super.duplicate(new_intance, copyPairs) before they fill in their own data.
 * <code>
 * public Object duplicate(Object instance, CopyPairs copyPairs)
 * {
 * DataModelCommandVector ret;
 * <p/>
 * if (instance == null) {
 * ret = new DataModelCommandVector();
 * } else if (!(instance instanceof DataModelCommandVector) {
 * throw new RuntimeException("Can't duplicate DataModelCommandVector into a: " + instance.getClass().getName());
 * }
 * <p/>
 * Enumeration e = elements();
 * <p/>
 * while (e.hasMoreElements()) {
 * DataModelCommand orig = (DataModelCommand)e.nextElement();
 * DataModelCommand copy = (DataModelCommand)copyPairs.get(orig);
 * <p/>
 * if (copy == null) {
 * // create a new instance
 * copy = (DataModelCommand)orig.duplicate(null, copyPairs);
 * copyPairs.put(orig, copy);
 * }
 * <p/>
 * ret.append(copy);
 * }
 * <p/>
 * return ret;
 * }
 * <code/>
 */
public interface DeepCopy {

    /**
     * @param instance  This parameter is null if the current has to create a new intance, otherwise it fills its attributes into the given instance (which is a subclass instance)
     * @param copyPairs is used to find out whether a certain instance reference has already a related copy registered
     */
    public Object duplicate(Object instance, CopyPairs copyPairs);
}
