//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import com.nexirius.util.unittest.UnitTestable;
import com.nexirius.util.unittest.UnitTestReport;

import java.io.File;
import java.io.FileOutputStream;

public class DirPathUnitTest implements UnitTestable {

    String foo_path = "C:\\;.;P:;X:\\hello\\world";
    String foo_dir = "X:\\hello\\world";
    String dir_temp = "C:\\temp";
    String dir_root = dir_temp + "\\DirPathUnitTest";
    String dir1 = dir_root + "\\sub1";
    String dir2 = dir_root + "\\sub2";
    String dir3 = dir_root + "\\sub1\\sub";

    DirPath path1 = new DirPath(foo_path);

    String file1 = dir1 + "\\file1";
    String file2 = dir2 + "\\file2";
    String file3 = dir3 + "\\file3";

    public DirPathUnitTest() {
    }

    public void createDir(String name)
            throws Exception {
        File dir = new File(name);

        if (!dir.mkdir()) {
            throw new Exception("Could not create " + name);
        }
    }

    public void createFile(String name)
            throws Exception {
        FileOutputStream s = new FileOutputStream(new File(name));

        s.write('x');

        s.close();
/*		TextFile file = new TextFile(name);

		file.openNewFileForWrite("Test", 1);

		file.close();
*/
    }

    public void prepare(UnitTestReport report)
            throws Exception {
        report.message("PREPARE SOME FILES AND DIRECTORIES UNDER " + dir_root);

        File temp = new File(dir_temp);

        if (!temp.exists()) {
            throw new Exception("Could not prepare test directories and files (" + dir_temp + " does not exist)");
        }

        XFile root = new XFile(dir_root);

        if (root.exists()) {
            report.message("Try to delete existing " + dir_root);

            if (!root.delete()) {
                throw new Exception("Could not remove existing " + dir_root);
            }
        }

        createDir(dir_root);
        createDir(dir1);
        createDir(dir2);
        createDir(dir3);

        createFile(file1);
        createFile(file2);
        createFile(file3);

        report.assertTrue(true);
    }

    public void cleanup(UnitTestReport report)
            throws Exception {
        report.message("CLEANUP");

        XFile root = new XFile(dir_root);

        if (root.exists()) {
            if (!root.delete()) {
                throw new Exception("Could not remove existing " + dir_root);
            }
        }

        report.assertTrue(true);
    }

    public void performUnitTest(UnitTestReport report) {
        try {
            prepare(report);

            report.message("Check if the constructor builds up the same path as getPath returns");
            report.assertTrue(foo_path.equals(path1.getPath()));

            report.message("Is a specific directory part of the path");
            report.assertTrue(path1.contains(foo_dir));

            report.message("Set path to " + dir1 + ";" + dir2 + ";" + dir3);
            path1.set(dir1);
            path1.append(dir2);
            path1.append(dir3);
            report.assertTrue(path1.getPath().equals(dir1 + ";" + dir2 + ";" + dir3));

            report.message("Find " + file1);
            report.assertTrue(dir1.equals(path1.getDirectory("file1")));
            report.message("Find " + file2);
            report.assertTrue(path1.getFile("file2").exists());
            report.message("Find " + file3);
            report.assertTrue(dir3.equals(path1.getDirectory("file3")));

            report.message("Look for XXX (must return null)");
            report.assertTrue(null == path1.getDirectory("XXX"));

            cleanup(report);
        } catch (Exception ex) {
            report.assertTrue(false);
            report.setFailureReason(ex.getMessage());
        }
    }
}
