//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER

package com.nexirius.util;


import com.nexirius.framework.FWLog;

import java.awt.*;
import java.awt.event.ActionListener;
import java.io.InputStream;


class ExecutorUnitTest implements CompletionListener {
    Executor m_currentExecutor = null;

    public void processCompleted(ExecutionStatus processStatus) {
        System.out.println("\nA process ended - there are "
                + String.valueOf(m_currentExecutor.getNumStarted() - m_currentExecutor.getNumCompleted())
                + " still running.");
        System.out.println("Exit Status was : " + String.valueOf(processStatus.getExitValue())
                + " for command :" + processStatus.getCommand());
        System.out.println(processStatus.getResult());
    }

    ExecutorUnitTest() {
        String targetBatch = System.getProperty("user.home", "")
                + "execute.bat";

        java.io.File batch = new java.io.File(targetBatch);
        try {
            java.io.DataOutputStream str = new java.io.DataOutputStream(new java.io.FileOutputStream(batch));
            str.writeBytes("@echo Test Process %1 is running...\n");
            str.writeBytes("@time /t\n");
            str.writeBytes("@cd\n");
            str.writeBytes("@echo Completed Test Process %1\n\n");
            str.close();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        ;

        System.out.println("Unit test for " + Executor.class);


        runTest(build(targetBatch, 1), 1, false);
        runTest(build("Must Fail", 1), 1, false);
        runTest(build(targetBatch, 3), 1, false);
        runTest(build(targetBatch, 1), 3, false);
        runTest(build(targetBatch, 10), 1, false);
        runTest(build(targetBatch, 10), 4, false);
        runTest(build(targetBatch, 20), 20, true);
        runTest(build(targetBatch, 10), 3, true);
    }

    private static int s_processId = 0;

    private String[] build(String path, int num) {
        String[] cmd1 = new String[num];
        int i = 0;
        for (i = 0; i < cmd1.length; i++) {
            cmd1[i] = path + " " + String.valueOf(s_processId++);
        }
        return cmd1;
    }

    private void runTest(String[] what, int num, boolean kill) {
        if (num == 1)
            System.out.println("\n\nExecuting serially.");
        else
            System.out.println("\n\nExecuting " + num + " at once.");

        for (int i1 = 0; i1 < what.length; i1++)
            System.out.println("    " + what[i1]);
        m_currentExecutor = new Executor(what);
        m_currentExecutor.setMaxConcurrent(num);
        m_currentExecutor.addCompletionListener(this);
        m_currentExecutor.execute();
        if (kill) {
            System.out.println("Waiting for half to complete....");
            while (m_currentExecutor.getNumCompleted() < what.length / 2)
                ;
            System.out.println("Aborting now....");
            System.out.println("At this point " + String.valueOf(m_currentExecutor.getNumCompleted()) +
                    " processes have already ended, at most " +
                    String.valueOf(m_currentExecutor.getNumStarted() - m_currentExecutor.getNumCompleted()) + " processes will be killed.");
            m_currentExecutor.abort();
            System.out.println("Waiting for aborted processes to complete....");
            while (m_currentExecutor.getNumStarted() != m_currentExecutor.getNumCompleted())
                ;
        } else {
            System.out.println("Waiting for completion....");
            while (m_currentExecutor.getNumCompleted() != what.length)
                ;
        }
    }
}

/**
 * Executes a list of external commands on the underlying OS. This will run
 * in a separate thread so that other processing can continue to, hopefully,
 * issue other commands before it needs to wait for exec completion.
 */

public class Executor {




    /**
     * main runs the unit test for Executor.
     */
    public static void main(String[] args) {
        ExecutorUnitTest ut = new ExecutorUnitTest();
    }

    private static int s_inst = 0;

    private java.util.Vector m_runners = new java.util.Vector();
    private String[] m_command;
    private int m_maxConcurrent = 1;
    private int m_completed = 0;
    private int m_started = 0;
    private ActionListener m_informer = null;
    private volatile boolean m_abort = false;

    /**
     * Create a new Executor by passing a list of commands to be executed.
     */
    public Executor(String[] command) {
        m_command = command;
    }

    /**
     * Create a new Executor by passing a vector of commands to be executed.
     */
    public Executor(StringVector command_list) {
        m_command = command_list.getArray();
    }

    /**
     * Set the maximum number of processes which this instance of Executor will
     * start. Whenever a currently executiong process completes then the next
     * command in the list will be started.
     */
    public void setMaxConcurrent(int num) {
        m_maxConcurrent = num;
    }

    public int getMaxConcurrent() {
        return m_maxConcurrent;
    }

    /**
     * CompletionListeners are called whenever a process completes. It passes the result of
     * the execution to the CompletionListener as well as completion status.
     */

    class CompletionListenerImpl implements java.awt.event.ActionListener {
        private CompletionListener m_cl;

        CompletionListenerImpl(CompletionListener cl) {
            m_cl = cl;
        }

        public void actionPerformed(java.awt.event.ActionEvent e) {
            if (e instanceof TerminationEvent) {
                m_cl.processCompleted((ExecutionStatus) e.getSource());
            } else {
                System.out.println("Internal error - I got back: " + e);
            }
        }

    }

    public ActionListener addCompletionListener(CompletionListener listener) {
        return m_informer = AWTEventMulticaster.add(m_informer, new CompletionListenerImpl(listener));
    }

    public void removeCompletionListener(CompletionListener listener) {
        m_informer = AWTEventMulticaster.remove(m_informer, new CompletionListenerImpl(listener));
    }

    /**
     * Start executing the commands given when the object was created. This method will create
     * up to the currently set value of MaxConcurrent processes and start them before returning.
     * The processes will run asynchronously with this process. The control of these processes is
     * managed in another thread.
     * <p/>
     * A client cannot interact with the underlying threads and processes.
     */
    public synchronized void execute() {
        while (!m_abort
                &&
                (m_started < m_command.length)
                &&
                ((m_started - m_completed) < m_maxConcurrent)) {
            // Create a new runner and add it to the list of those
            // currently running
            Runner aRunner = new Runner(this, m_started++);
            m_runners.addElement(aRunner);
            aRunner.start();
        }
    }

    /**
     * Abort will destroy any processes which are currently executing and
     * prevent any more from being started by abort to be true.
     * Note that the information about the number of processes started, completed
     * and the total to be run will be unaffected by this flag. Thatis, you can find out how many
     * processes were actually run and how many we are still waiting to complete using the usual
     * methods.
     */
    public synchronized void abort() {
        m_abort = true;

        java.util.Enumeration e = m_runners.elements();
        while (e.hasMoreElements())
            ((Runner) e.nextElement()).killProcess();
    }

    /**
     * Returns the value of the abort flag. This is set when an abort request is made.
     */
    public boolean getAbort() {
        return m_abort;
    }

    /**
     * Get the total number of processes started so far. This number includes
     * completed processes. getNumStarted() - getNumCompleted() gives the number
     * of currently running processes.
     */
    public int getNumStarted() {
        return m_started;
    }

    /**
     * Get the number of process which have completed so far.
     * Subtracting this value from the number of commands submitted gives
     * the number of processes waiting to be started.
     * <p/>
     * This is useful value to display in a scroll bar to show progress of the complete
     * execution.
     */
    public int getNumCompleted() {
        return m_completed;
    }

    public int getTotalCommands() {
        return m_command.length;
    }

    /**
     * Method called whenever a Runner instance detects that a process has completed.
     * It returns
     */
    synchronized private void runnerFinished(Runner who) {
        m_completed++;
        m_runners.removeElement(who);

        if (m_informer != null) {
            m_informer.actionPerformed(new TerminationEvent(who));
        }

        // Start any more jobs still waiting
        execute();
    }

    private String getCommand(int which) {
        return m_command[which];
    }


    /**
     * This is a inner class which performs the real execution.
     * It runs as a separate thread rather than hanging the current thread.
     * Once the execution is complete the Runner calls its Executor
     * parent to tell it that the execution has finished and delivers back
     * the results of the execution.
     */
    class Runner extends Thread implements ExecutionStatus, com.nexirius.util.CommandTarget {
        private Executor m_parent;
        private int m_commandId;
        private StringBuffer m_result = new StringBuffer("");
        private StringBuffer m_error = new StringBuffer("");
        private int m_exitvalue = -1;
        private Process m_process = null;
        private boolean m_mayStart = true;
        private int m_seconds = 10;

        private com.nexirius.util.Timer m_timer = null;

        Runner(Executor parent, int commandId) {
            super("Executor thread for command : " + parent.getCommand(commandId));
            m_parent = parent;
            m_commandId = commandId;
            m_timer = new com.nexirius.util.Timer(null, this, true);
        }

        public boolean execute(com.nexirius.util.CommandTarget caller, String function, Object parameters[]) {
            if (m_timer != null) {
                System.out.println("-------------------------Still Waiting-----------------------------");
                System.out.println("Waiting for : " + String.valueOf(m_seconds) + " seconds.");
                System.out.println("For command : " + getCommand());
                System.out.println("StdOut      : " + getResult());
                System.out.println("StdErr      : " + getError());
                System.out.println("-------------------------------------------------------------------");
                m_timer = new com.nexirius.util.Timer(null, this, true);
                m_timer.alertAfter(5000);
                m_seconds += 5;
            }
            return true;
        }

        public String getCommand() {
            return m_parent.getCommand(m_commandId);
        }

        public int getID() {
            return m_commandId;
        }

        public String getResult() {
            return m_result.toString();
        }

        public String getError() {
            return m_error.toString();
        }

        public int getExitValue() {
            return m_exitvalue;
        }

        public synchronized void killProcess() {
            m_mayStart = false;
            if (m_process != null) {
                m_process.destroy();
            } else {
            }
        }

        public void run() {
            Runtime rt = Runtime.getRuntime();
            int b = 0;
            int e = 0;
            InputStream in = null;
            InputStream err = null;
            synchronized (this) {
                try {
                    if (m_mayStart) {
                        m_process = rt.exec(m_parent.getCommand(m_commandId));
                        in = m_process.getInputStream();
                        err = m_process.getErrorStream();
                        m_timer.alertAfter(10000);
                    } else {
                        m_result.append("Process was not started - abort requested.");
                        m_exitvalue = -2;
                    }
                } catch (Exception ex) {
                    FWLog.error("Command execution failed", ex);

                    if (m_result.length() > 0) {
                        m_result.append(ex.getMessage());
                    }
                    m_result.append(ex.getMessage());
                }
            } // End of synchronized section

            Reader stdIn = null;
            Reader stdErr = null;
            if (in != null) {
                stdIn = new Reader(in, m_result);
                stdErr = new Reader(err, m_error);
                stdIn.start();
                stdErr.start();
            }
            try {
                m_process.waitFor();
                stdIn.join();
                stdErr.join();
            } catch (Exception ewf) {
                System.out.println("InterruptedException whilst wait for the process to end: " + ewf);
            }

            if (m_process != null)
                m_exitvalue = m_process.exitValue();
            m_parent.runnerFinished(this);
            m_timer.stopTimeout();
            m_timer = null;
        }


        public String toString() {
            return m_parent.getCommand(m_commandId) + "\n" +
                    m_result;
        }
    }

    class Reader extends Thread {
        InputStream m_stream;
        StringBuffer m_buff;

        Reader(InputStream stream, StringBuffer buff) {
            m_stream = stream;
            m_buff = buff;
        }

        public void run() {
            int b = 0;
            while (true) {
                try {
                    b = m_stream.read();
                } catch (Exception ex) {
                    FWLog.error("Read failed", ex);
                    m_buff.append(ex.getMessage());
                }
                if (b == -1)
                    break;
                m_buff.append((char) b);
            }
            // Clean up unwanted references
            m_buff = null;
            m_stream = null;
        }
    }
}
