//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;


import java.io.*;


public class MagicNumberTextFile extends TextFile {



    long magicNumber = 0;
    MagicNumberOutputStream mn_out;
    MagicNumberInputStream mn_in;

    /**
     * Creates a File instance that represents the file whose pathname is
     * the given path argument.
     */
    public MagicNumberTextFile(String path) {
        super(path);
    }

    /**
     * Creates a File instance whose pathname is the pathname of the specified
     * directory, followed by the separator character, followed by the
     * name argument.
     */
    public MagicNumberTextFile(File dir, String name) {
        super(dir, name);
    }

    public MagicNumberOutputStream getOut() {
        return mn_out;
    }

    public MagicNumberInputStream getIn() {
        return mn_in;
    }

    /**
     * closes the file (this function is automatically called if all the contents
     * of a file a read)
     */
    public void close()
            throws IOException, MagicNumberException {
        if (mn_out != null) {
            mn_out.close();
            magicNumber = mn_out.getMagicNumber();
        }

        boolean magicNumberError = false;

        if (mn_in != null) {
            mn_in.close();

            if (!mn_in.checkMagic()) {
                magicNumberError = true;
            }

            if (mn_in.hasMagic()) {
                magicNumber = mn_in.getMagicNumber();
            }
        }

        mn_out = null;
        mn_in = null;

        super.close();

        if (magicNumberError) {
            throw new MagicNumberException("WrongMagicNumber");
        }
    }

    public long getMagicNumber() {
        return magicNumber;
    }

    public OutputStream createOutputStream(File f)
            throws IOException {
        return mn_out = new MagicNumberOutputStream(new FileOutputStream(f));
    }

    public PushbackInputStream createPushbackInputStream(File f)
            throws IOException {
        return mn_in = new MagicNumberInputStream(new FileInputStream(f));
    }
}
