//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.util.Hashtable;
import java.util.MissingResourceException;

public class ParameterResources {



    public static final String PROPERTY_EXTENSIONS = "PropertyExtensions";

    StringResource m_resource = null;
    Hashtable m_extensions = new Hashtable();

    public ParameterResources(StringResource resource)
            throws ResourceFileException {
        m_resource = resource;
        init();
    }

    public void init()
            throws ResourceFileException {
        try {
            String ext = m_resource.getString(PROPERTY_EXTENSIONS);
            StringVector v = new StringVector(ext, " \t,");

            for (String str = v.firstItem(); str != null; str = v.nextItem()) {
                m_extensions.put(str, str);
            }
        } catch (MissingResourceException ex) {
            throw new ResourceFileException("Resource '" + PROPERTY_EXTENSIONS + "' was not defined in " + m_resource);
        }
    }

    public boolean hasExtension(String s) {
        return null != m_extensions.get(prepareExtension(s));
    }

    public StringVector getAllExtensions() {
        return new StringVector(/*FIX m_extensions.elements()*/);
    }

    public String prepareExtension(String s) {
        XString xs = new XString(s);

        return xs.removeCharacter(' ');
    }

    public String getKey(String key, String extension)
            throws ResourceFileException {
        String ext = prepareExtension(extension);

        if (!hasExtension(ext)) {
            throw new ResourceFileException("The resource file '" + m_resource + "' does not support resource extension '" + ext);
        }

        return key + "_" + ext;
    }

    public String getString(String key)
            throws ResourceFileException {
        return m_resource.getString(key);
    }

    public StringVector getAllKeys(String key)
            throws ResourceFileException {
        StringVector sv = getAllExtensions();
        StringVector ret = new StringVector();

        for (String s = sv.firstItem(); s != null; s = sv.nextItem()) {
            ret.append(getKey(key, s));
        }

        return ret;
    }

    public String getOSNameKey(String key)
            throws ResourceFileException {
        return getKey(key, (String) System.getProperty("os.name"));
    }

    public String getOSNameProperty(String key)
            throws ResourceFileException {
        return m_resource.getString(getOSNameKey(key));
    }
}
