//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.util.*;

/**
 * A class that implements a Vector with a sort() function (bubble sort)
 */
public class SortedVector<E> extends Vector<E> {
    private static final long serialVersionUID = -6041098402986341356L;

    Enumeration<E> m_enumeration;
    int m_index = 0;

    public SortedVector() {
        super();
    }

    public SortedVector(E array[]) {
        super();

        if (array == null) {

            return;
        }

        for (int i = 0; i < array.length; ++i) {
            addElement(array[i]);
        }
    }

    public SortedVector(Vector<E> v) {
        super();

        Enumeration<E> e = v.elements();

        while (e.hasMoreElements()) {
            addElement(e.nextElement());
        }
    }

    public SortedVector(int initialCapacity) {
        super(initialCapacity);
    }

    public SortedVector(int initialCapacity, int capacityIncrement) {
        super(initialCapacity, capacityIncrement);
    }

    /**
     * Sorts the elements using Collections.sort(Comparator).
     * @param comparator
     */
    public void sort(Comparator<E> comparator)
    {
        Collections.sort(this, comparator);
    }
    
    /**
     * sorts the elements of the vector (bubble sort)
     */
    public boolean sort(Sorter sorter) {
        boolean ret = false;

        if (sorter == null) {
            return false;
        }

        List<E> o = new ArrayList<E>(size());

        o.addAll(this);

        E temp;
        int i = 0;
        int end = o.size();
        boolean did_swap = true;

        while (did_swap) {
            did_swap = false;

            for (i = 1; i < end; ++i) {
                if (sorter.compare(o.get(i - 1), o.get(i)) < 0) {
                    did_swap = true;
                    temp = o.get(i);
                    o.set(i, o.get(i - 1));
                    o.set(i - 1, temp);
                    ret = true;
                }
            }

            --end;
        }

        if (ret) {
            removeAllElements();

            for (i = 0; i < o.size(); ++i) {
                addElement(o.get(i));
            }
        }

        return ret;        
    }

    /**
     * this function is already defined in Java 2.0!
     */
    public E set(int index, E newObject) {
        E ret = elementAt(index);

        setElementAt(newObject, index);

        return ret;
    }

    /**
     * inserts the object at a sorted position
     * (the vector must be sorted before calling this function)
     *
     * @return the actual position
     */
    public int sortInsert(E o, Sorter sorter) {
        if (sorter == null) {
            addElement(o);

            return size() - 1;
        }

        for (int i = size() - 1; i >= 0; --i) {
            Object o2 = get(i);

            if (sorter.compare(o, o2) <= 0) {
                insertElementAt(o, i + 1);

                return i + 1;
            }
        }

        insertElementAt(o, 0);

        return 0;
    }

    /**
     * @return an array of all the objects in the list
     */
    public Object[] getObjectArray() {
        Object ret[] = null;
        int i = 0;

        if (size() > 0) {
            Enumeration<E> e = elements();

            ret = new Object[size()];

            while (e.hasMoreElements()) {
                ret[i++] = e.nextElement();
            }
        }

        return ret;
    }

/*
	public synchronized int indexOf(Object o)
	{
		if (size() > 0) {
			Enumeration e = elements();

			for (int index = 0; e.hasMoreElements(); ++index)
			{
				if (o == e.nextElement()) {

					return index;
				}
			}
		}

		return -1;
	}
*/
    /**
     * @return an array of some objects in the list (if to < 0 all remaining objects are appended)
     */
    public Object[] getObjectArray(int from, int to) {
        Object ret[] = null;

        if (from < 0) {
            from = 0;
        }

        if (to < 0 || to >= size()) {
            to = size() - 1;
        }

        if (from <= to && from < size()) {
            Enumeration<E> e = elements();

            ret = new Object[to - from + 1];

            while (e.hasMoreElements()) {
                ret[from] = e.nextElement();

                if (from >= to) {
                    break;
                }

                ++from;
            }
        }

        return ret;
    }

    /**
     * with firstObject nextObject the list may be walked in simple loops like this:
     * <CODE>
     * for (MyClass mc = (MyClass)v.firstObject(); mc != null;  mc = (MyClass)v.nextObject()) {
     * <BR>
     * // do something with mc
     * <BR>
     * }
     * <BR>
     * </CODE>
     * Do not use this function in recursive calls or inner loops
     *
     * @return the first Object in the list or nul (empty list)
     */
    public Object firstObject() {
        m_enumeration = elements();

        if (m_enumeration.hasMoreElements()) {
            return m_enumeration.nextElement();
        }

        return null;
    }

    /**
     * with firstObject nextObject the list may be walked in simple loops like this:
     * <CODE>
     * for (MyClass mc = (MyClass)v.firstObject(); mc != null;  mc = (MyClass)v.nextObject()) {
     * <BR>
     * // do something with mc
     * <BR>
     * }
     * <BR>
     * </CODE>
     * Do not use this function in recursive calls or inner loops
     *
     * @return the next Object in the list or null
     */
    public Object nextObject() {
        if (m_enumeration != null && m_enumeration.hasMoreElements()) {
            return m_enumeration.nextElement();
        }

        m_enumeration = null;

        return null;
    }

    public Object getObject(int index) {
        if (index >= 0 && index < size()) {

            return elementAt(index);
        }

        return null;
    }

    /**
     * with lastObject previousObject the list may be walked backwards in simple loops like this:
     * <CODE>
     * for (MyClass mc = (MyClass)v.lastObject(); mc != null;  mc = (MyClass)v.previousObject()) {
     * <BR>
     * // do something with mc
     * <BR>
     * }
     * <BR>
     * </CODE>
     * Do not use this function in recursive calls or inner loops
     *
     * @return the last Object in the list or null (empty list)
     */
    public Object lastObject() {
        m_index = size();

        return previousObject();
    }

    /**
     * with lastObject previousObject the list may be walked backwards in simple loops like this:
     * <CODE>
     * for (MyClass mc = (MyClass)v.lastObject(); mc != null;  mc = (MyClass)v.previousObject()) {
     * <BR>
     * // do something with mc
     * <BR>
     * }
     * <BR>
     * </CODE>
     * Do not use this function in recursive calls or inner loops
     *
     * @return the previous Object in the list or null
     */
    public Object previousObject() {
        if (m_index > 0) {

            return elementAt(--m_index);
        }

        return null;
    }

    public synchronized boolean equals(Object other) {
        // FIX with Java 2.0 already implemented

        if (other != null && other instanceof SortedVector) {
            SortedVector<E> o = (SortedVector<E>) other;

            if (size() == o.size()) {
                Enumeration<E> e1 = elements();
                Enumeration<E> e2 = o.elements();

                while (e1.hasMoreElements()) {
                    Object o1 = e1.nextElement();
                    Object o2 = e2.nextElement();

                    if ((o1 == null ? o2 == null : o1.equals(o2)) == false) {
                        return false;
                    }
                }

                return true;
            }
        }

        return false;
    }

    public boolean isFirst(Object o) {
        if (o != null) {
            return o == firstElement();
        }

        return false;
    }

    public boolean isLast(Object o) {
        if (o != null) {
            return o == lastElement();
        }

        return false;
    }
}
