//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;

public class StringVector extends SortedVector<String> implements Sorter {
    private static final long serialVersionUID = 8369005965329446859L;

    boolean ignoreCase = true;

    public StringVector() {
        super();
    }

    public StringVector(String s[]) {
        append(s);
    }

    /**
     * creates a StringVector from a String which is separated by delimiter characters.
     * see java.util.StringTokenizer
     */
    public StringVector(String s, String delimiters) {
        StringTokenizer st = new StringTokenizer(s, delimiters);

        while (st.hasMoreElements()) {
            append(st.nextToken());
        }
    }

    /**
     * creates a StringVector from a String, which is separated by delimiter character
     */
    public StringVector(String s, char delimiter) {
        this(s, delimiter, false);
    }

    /**
     * Creates a StringVector from a String, which is separated by delimiter character. If the separator character is
     * inside a quoted string then it is ignored
     */
    public StringVector(String s, char delimiter, boolean withQotedStrings) {
        if (withQotedStrings) {
            StringBuffer sb = new StringBuffer();
            int state = 0;
            for (int i = 0; i < s.length(); ++i) {
                char c = s.charAt(i);

                switch (state) {
                    case 0: // beginning
                        if (c == '\"') {
                            state = 1;
                        } else if (c == delimiter) {
                            append(sb.toString());
                            sb = new StringBuffer();
                        } else {
                            state = 4;
                            sb.append(c);
                        }
                        break;
                    case 1: // in string
                        if (c == '\\') {
                            state = 2;
                        } else if (c == '\"') {
                            state = 3;
                        } else {
                            sb.append(c);
                        }
                        break;
                    case 2: // in string after \
                        sb.append(c);
                        state = 1;
                        break;
                    case 3: // end of string reached
                        if (c == delimiter) {
                            append(sb.toString());
                            sb = new StringBuffer();
                            state = 0;
                        }
                        break;
                    case 4: // no quoted string
                        if (c == delimiter) {
                            append(sb.toString());
                            sb = new StringBuffer();
                            state = 0;
                        } else {
                            sb.append(c);
                        }
                        break;
                }
            }

            append(sb.toString());
        } else {
            while (true) {
                int index = s.indexOf(delimiter);

                if (index >= 0) {
                    append(s.substring(0, index));
                    s = s.substring(index + 1);
                } else {
                    append(s);
                    break;
                }
            }
        }
    }

    public boolean isIgnoreCase() {
        return ignoreCase;
    }

    public void setIgnoreCase(boolean ignoreCase) {
        this.ignoreCase = ignoreCase;
    }

    /**
     * returns the String at the specified index (or null)
     */
    public String getItem(int index) {
        return (String) getObject(index);
    }

    /**
     * returns the first status and sets the cursor to it
     */
    public String firstItem() {
        return (String) firstObject();
    }

    /**
     * returns the next status (only makes sense after calling firstItem())
     */
    public String nextItem() {
        return (String) nextObject();
    }

    /**
     * returns the last status and sets the cursor to it
     */
    public String lastItem() {
        return (String) lastObject();
    }

    /**
     * get the next status to the left of the previously read status (makes
     * sense only after calling lastItem())
     */
    public String previousItem() {
        return (String) previousObject();
    }

    /**
     * @return a String where all items are listed (with separator character between items)
     */
    public String getAll(String separator) {
        StringBuffer ret = new StringBuffer();
        Enumeration<String> e = elements();

        while (e.hasMoreElements()) {
            ret.append((String) e.nextElement());

            if (e.hasMoreElements() && separator != null) {
                ret.append(separator);
            }
        }

        return ret.toString();
    }

    /**
     * alphabetically sort this vector
     */
    public void sort() {
        super.sort(this);
    }

    /**
     * alphabetically insert s into this
     */
    public void sortInsert(String s) {
        super.sortInsert(s, this);
    }

    /**
     * alphabetically insert s elements into this
     */
    public void sortInsert(String s[]) {
        if (s == null) {
            return;
        }

        for (int i = 0; i < s.length; ++i) {
            sortInsert(s[i]);
        }
    }

    /**
     * alphabetically insert s elements into this (only if the match the pattern)
     */
    public void sortInsert(String s[], String match_pattern) {
        if (s == null) {
            return;
        }

        for (int i = 0; i < s.length; ++i) {
            if (XString.match(match_pattern, s[i])) {
                sortInsert(s[i]);
            }
        }
    }

    /**
     * alphabetically insert sv elements into this
     */
    public void sortInsert(StringVector sv) {
        for (String s = sv.firstItem(); s != null; s = sv.nextItem()) {
            sortInsert(s);
        }
    }

    /**
     * alphabetically insert sv elements into this (only if the match the pattern)
     */
    public void sortInsert(StringVector sv, String match_pattern) {
        for (String s = sv.firstItem(); s != null; s = sv.nextItem()) {
            if (XString.match(match_pattern, s)) {
                sortInsert(s);
            }
        }
    }

    /**
     * append s to the end of this vector
     */
    public void append(String s) {
        addElement(s);
    }

    /**
     * append all the elements of s to the end of this vector
     */
    public void append(String s[]) {
        for (int i = 0; i < s.length; ++i) {
            addElement(s[i]);
        }
    }

    /**
     * append all the members of sv to the end of this vector
     */
    public void append(StringVector sv) {
        for (String s = sv.firstItem(); s != null; s = sv.nextItem()) {
            addElement(s);
        }
    }

    /**
     * @return a string array with all the members of this
     */
    public String[] getArray() {
        String ret[] = new String[size()];

        copyInto(ret);

        return ret;
    }

    /**
     * @return true if vector contains s
     */
    public boolean contains(String s) {
        for (String str = firstItem(); str != null; str = nextItem()) {
            if (ignoreCase) {
                if (str.equalsIgnoreCase(s)) {
                    return true;
                }
            } else {
                if (str.equals(s)) {
                    return true;
                }
            }
        }

        return false;
    }

    public int compare(Object o1, Object o2) {
        String item1 = (String) o1;
        String item2 = (String) o2;
        String s1 = item1.toUpperCase();
        String s2 = item2.toUpperCase();

        if (ignoreCase) {
            return s2.compareToIgnoreCase(s1);
        }

        return s2.compareTo(s1);
    }

    public boolean equals(Object other) {
        if (other != null) {
            if (other instanceof StringVector) {
                StringVector sv = (StringVector) other;

                if (size() != sv.size()) {
                    return false;
                }

                String s1 = firstItem();
                String s2 = sv.firstItem();

                while (s1 != null && s2 != null) {

                    if (ignoreCase) {
                        if (!s1.equalsIgnoreCase(s2)) {
                            return false;
                        }
                    } else {
                        if (!s1.equals(s2)) {
                            return false;
                        }
                    }

                    s1 = nextItem();
                    s2 = sv.nextItem();
                }

                return true;
            }
        }

        return false;
    }

    public void print() {
        Enumeration<String> e = elements();

        while (e.hasMoreElements()) {
            System.out.println((String) e.nextElement());
        }
    }

    /**
     * replace every occurence of 'from' with 'to'
     *
     * @return the number of replacements
     */
    public int replace(String from, String to) {
        int ret = 0;
        StringVector new_list = new StringVector();
        Enumeration<String> e = elements();

        while (e.hasMoreElements()) {
            XString xs = new XString((String) e.nextElement());

            ret += xs.replace(from, to);

            new_list.append(xs.toString());
        }

        removeAllElements();

        append(new_list);

        return ret;
    }

    /**
     * @return true if the string was added
     */
    public boolean sortInsertNoDuplicates(String string) {
        if (!contains(string)) {
            sortInsert(string);
            return true;
        }

        return false;
    }

    public void reduceDuplicates() {
        HashSet<String> set = new HashSet<String>(this);

        clear();

        Iterator<String> it = set.iterator();

        while (it.hasNext()) {
            String s = it.next();

            sortInsert(s);
        }
    }

    public synchronized String toString() {
        return toString(',');
    }

    public String toString(char delimiter) {
        StringBuffer ret = new StringBuffer();

        for (Iterator<String> iterator = this.iterator(); iterator.hasNext();) {
            String s = (String) iterator.next();
            ret.append(s);

            if (iterator.hasNext()) {
                ret.append(delimiter);
            }
        }

        return ret.toString();
    }

    public String toString(String delimiter) {
        StringBuffer ret = new StringBuffer();

        for (Iterator<String> iterator = this.iterator(); iterator.hasNext();) {
            String s = (String) iterator.next();
            ret.append(s);

            if (iterator.hasNext()) {
                ret.append(delimiter);
            }
        }

        return ret.toString();
    }

    public static void main(String[] argv) {
        StringVector sortedVector = new StringVector();

        sortedVector.sortInsert("D");
        sortedVector.sortInsert("A");
        sortedVector.sortInsert("B");
        sortedVector.sortInsert("C");
        sortedVector.sortInsert("A");
        sortedVector.sortInsert("C");


        System.out.println(sortedVector);

        try {
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update("Mineral".getBytes());
            byte[] bytes = md5.digest("hallo1jaksdhfskajdhfk".getBytes());

            for (int i = 0; i < bytes.length; i++) {
                byte aByte = bytes[i];
                System.out.println(i + ":" + Integer.toHexString(aByte & 0xff));
            }



            try {
                ObjectOutputStream out = new ObjectOutputStream(System.out);

                out.write(bytes);
            } catch (IOException e) {
                throw new RuntimeException(e);  //TODO
            }

        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }


}

