//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

/**
 * a class that calls its parent execute function after a given number of milliseconds
 */
public class Timer extends Thread implements CommandTarget {



    private String m_message;
    private CommandTarget m_client;
    private long m_timeout;
    private boolean m_stopped;
    private boolean m_die_after_timeout;
    private boolean m_inCallback = false;

    public Timer(String message, CommandTarget client, boolean die_after_timeout) {
        setDaemon(true);
        m_message = message;
        m_client = client;
        m_die_after_timeout = die_after_timeout;
    }

    /**
     * call this function to start a thread that sleeps for the
     * given number of millis. It then calls execute(this, message, null)
     * on its client.
     * If this function is called twice before the previous sleep elapsed, it
     * interrupts its previous sleep without alert and restarts the new sleep.
     */
    public synchronized void alertAfter(long millis) {
        m_timeout = millis;

        if (!isAlive()) {
            start();		// in case its not running yet
        } else {
            m_stopped = false;

            if (!m_inCallback) {
                interrupt();	// in case its sleeping
            }
        }
    }

    /**
     * stops the current timer.
     *
     * @return true if an active timer was stopped
     */
    public synchronized boolean stopTimeout() {
        boolean ret = false;

        if (isAlive()) {
            m_stopped = true;

            if (!m_inCallback) {
                interrupt();
            }

            ret = !m_stopped;
        }

        return ret;
    }

    public void run() {
        while (true) {
            try {
                interrupted(); // resets the interrupted flag
                sleep(m_timeout);

                m_inCallback = true;
                m_client.execute(this, m_message, null);
                m_inCallback = false;

                if (m_die_after_timeout) {
                    break;
                }
            } catch (InterruptedException ex) {
                if (m_stopped) {
                    m_stopped = false;

                    if (m_die_after_timeout) {
                        break;
                    }
                }
            }
        }
    }

    public boolean execute(CommandTarget caller, String command, Object par[]) {
        return true;
    }
}
