package com.nexirius.util;

import com.nexirius.framework.htmlview.HTMLResolver;

import java.io.*;

public class UnicodeHelper {
    public static final byte[] BYTES_ZERO = new byte[]{'\\', 'u', '0', '0', '0', '0'};

    public static byte[] translateStringToUnicode(String s) {
        StringReader reader = new StringReader(s);
        ByteArrayOutputStream out = new ByteArrayOutputStream(10240);
        translateToUnicode(reader, out);

        return out.toByteArray();
    }

    public static void translateToUnicode(Reader reader, OutputStream out) {
        while (true) {
            try {
                int c = reader.read();

                if (c < 0) {
                    out.close();
                    break;
                }
                if (c < '\u007F' && c > '\u0000') {
                    out.write(c);
                } else if (c == '\u0000') {
                    out.write(BYTES_ZERO);
                } else {
                    if (c <= '\u00FF') {
                        out.write(BYTES_ZERO, 0, 4);
                    } else if (c <= '\u0FFF') {
                        out.write(BYTES_ZERO, 0, 3);
                    } else {
                        out.write(BYTES_ZERO, 0, 2);
                    }

                    out.write(Integer.toHexString(c).getBytes(HTMLResolver.CHARSET));
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * translate ascii string containing backslash u escape sequences into UTF-8 String
     * @param in
     * @return
     * @throws UnsupportedEncodingException
     */
    public static String translateToUTF8(String in) throws UnsupportedEncodingException {
        ByteArrayInputStream inStream = new ByteArrayInputStream(in.getBytes("UTF-8"));
        ByteArrayOutputStream outStream = new ByteArrayOutputStream(10240);

        translateToUTF8(inStream, outStream);

        System.out.println("new String(outStream.toByteArray()) = " + new String(outStream.toByteArray()));

        byte[] bytes = outStream.toByteArray();
        
        return new String(bytes, "UTF-8");
    }

    public static void translateToUTF8(InputStream in, OutputStream out) {
        int state = 0;
        byte[] unicode = new byte[4];

        while (true) {
            try {
                int c = in.read();

                if (c < 0) {
                    break;
                }
                state = parseByte(state, (byte) c, out, unicode);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    private static int parseByte(int state, byte c, OutputStream out, byte[] unicode) throws IOException {
        switch (state) {
            case 0: // no escape
                switch (c) {
                    case '\\':
                        state = 1;
                        break;
                    default:
                        out.write(c);
                }
                break;

            case 1: // after escape
                switch (c) {
                    case '\\':
                        state = 0;
                        out.write('\\');
                        break;

                    case 'u':
                        state = 2;
                        break;

                    default: // unexpected escape character
                        state = 0;
                        out.write('\\');
                        out.write(c);
                }
                break;

            case 2: // first unicode character
            case 3: // second unicode character
            case 4: // third unicode character
            case 5: // forth unicode character
                unicode[state - 2] = c;

                if (state < 5) {
                    ++state;
                } else {
                    // append unicode character

                    try {
                        char ch = (char) Integer.parseInt(new String(unicode), 16);
                        byte[] bytes = String.valueOf(ch).getBytes("UTF-8");
                        out.write(bytes);
                    } catch (Exception ex) {
                        out.write('\\');
                        out.write('u');
                        out.write(unicode);
                    }
                    state = 0;
                }

                break;
        }

        return state;
    }

//    public static void main(String[] args) throws IOException {
//        String test = "aäoöuü\u2030\u0007\u0000\u0231\u0099";
//        byte[] bytes = translateStringToUnicode(test);
//        String str = new String(bytes);
//        System.out.println(test + " = " + str);
//
//        str = "\"" + str + "\"";
//
//        ByteArrayInputStream in = new ByteArrayInputStream(str.getBytes());
//        String tokenStr = TextToken.nextToken(new PushbackInputStream(in)).getString();
//        System.out.println("tokenStr = " + tokenStr);
//
//        System.out.println("translateToUTF8(str); = " + translateToUTF8(str));
//    }
}
