//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.*;

public class VariableResolver {
    public static String DEFAULT_CHARSET = "UTF-8";

    public static String translate(VariableAccessor acc, String inString) throws IOException {
        StringWriter out = new StringWriter();
        translate(acc, new StringReader(inString), out);

        return out.toString();
    }

    /**
     * translate variables like $(variable) from in into a resolved out.
     *
     * @param acc
     * @param in
     * @param out
     * @return the number of translated variables
     * @throws IOException
     */
    public static int translate(VariableAccessor acc, Reader in, Writer out) throws IOException {
        int number = 0;
        int c;
        int state = 0;
        StringBuffer var = null;

        while (true) {
            c = in.read();

            if (c < 0) {
                break;
            }

            switch (state) {
                case 0:
                    if (c == '$') {
                        state = 1;
                    } else {
                        out.write(c);
                    }
                    break;

                case 1:
                    if (c == '(') {
                        var = new StringBuffer();
                        state = 2;
                    } else {
                        out.write('$');
                        out.write(c);
                        state = 0;
                    }
                    break;

                case 2:
                    if (c == ')') {
                        String value = acc.getValueOf(var.toString());

                        if (value != null) {
                            out.write(value);
                        }
                        state = 0;
                        ++number;
                    } else {
                        var.append((char) c);
                    }
                    break;
            }
        }

        return number;
    }

    /**
     * translate variables like $(variable) from in into a resolved out.
     *
     * @param acc
     * @param in
     * @param out
     * @return the number of translated variables
     * @throws IOException
     */
    public static int translate(VariableAccessor acc, InputStream in, OutputStream out) throws IOException {
        return translate(acc, in, out, DEFAULT_CHARSET);
    }
    /**
     * translate variables like $(variable) from in into a resolved out.
     *
     * @param acc
     * @param in
     * @param out
     * @return the number of translated variables
     * @throws IOException
     */
    public static int translate(VariableAccessor acc, InputStream in, OutputStream out, String charset)
            throws IOException {
        int number = 0;
        int c;
        int state = 0;
        StringBuffer var = null;

        while (true) {
            c = in.read();

            if (c < 0) {
                break;
            }

            switch (state) {
                case 0:
                    if (c == '$') {
                        state = 1;
                    } else {
                        out.write(c);
                    }
                    break;

                case 1:
                    if (c == '(') {
                        var = new StringBuffer();
                        state = 2;
                    } else {
                        out.write('$');
                        out.write(c);
                        state = 0;
                    }
                    break;

                case 2:
                    if (c == ')') {
                        String value = acc.getValueOf(var.toString());

                        if (value != null) {
                            out.write(value.getBytes(charset));
                        }
                        state = 0;
                        ++number;
                    } else {
                        var.append((char) c);
                    }
                    break;
            }
        }

        return number;
    }

    public static void main(String argv[])
            throws IOException {
        String a = "$(xxx)lkjdsfnlg$sdlfvsjdk--$(Marcel)--jkhfdl$(end)";
        InputStream in = new ByteArrayInputStream(a.getBytes());
        OutputStream out = new ByteArrayOutputStream();

        int n = VariableResolver.translate(new MyVariableAccessor(), in, out);

        System.out.println(out.toString() + "   translated " + n + " variables");
    }

    static class MyVariableAccessor implements VariableAccessor {
        public String getValueOf(String var) {
            if (var.equals("Marcel")) {

                return "Baumann";
            } else if (var.equals("end")) {

                return "ENDE";
            } else {

                return "not found";
            }

        }
    }
}
