//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.PushbackInputStream;
import java.util.Enumeration;
import java.util.Vector;

/**
 * The XFileTree class holds all the files from a directory
 */
public class XFileTree {



    String name;
    XFileTree parent = null;
    Vector children = null;
    Vector files = null;
    Enumeration enumeration = null;
    Enumeration fenumeration = null;

    public XFileTree(String name, XFileTree parent) {
        this.name = name;
        this.parent = parent;
    }

    public XFileTree(XFile dir) {
        name = dir.getAbsolutePath();

        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();

            dir.writeDirectory(out);

            PushbackInputStream in = new PushbackInputStream(new ByteArrayInputStream(out.toString().getBytes()));

            readFrom(in);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public XFileTree(String path) {
        this(new XFile(path));
    }

    public XFileTree(String path, String name) {
        this(new XFile(path, name));
    }

    public int readFrom(PushbackInputStream in)
            throws Exception {
        int ret = 0;

        TextToken t = TextToken.nextToken(in);

        while (t != null) {
            if (t.isChar('}')) {

                return ret;
            } else if (t.isString()) {
                String n = t.getString();
                t = TextToken.nextToken(in);

                if (t.isChar('{')) {
                    XFileTree child = new XFileTree(n, this);

                    ret += child.readFrom(in);

                    appendChild(child);
                } else if (t.isLong()) {
                    FileChild file = new FileChild(n);

                    file.date = t.getLong();

                    t = TextToken.nextToken(in);

                    if (t != null) {
                        file.size = t.getLong();
                    }

                    appendFile(file);
                }
            } else {
                System.out.println("Unexpected token " + t);
                break;
            }

            t = TextToken.nextToken(in);
        }

        return ret;
    }

    public int writeDirectory(OutputStream out)
            throws Exception {
        int ret = 0;

        TextToken t = new TextToken(name);

        t.writeTo(out);
        out.write('{');
        out.write('\n');

        while (hasMoreChildren()) {
            ret += nextChild().writeDirectory(out);
        }

        while (hasMoreFiles()) {
            nextFile().writeDirectory(out);
            ++ret;
        }

        out.write('}');
        out.write('\n');

        return ret;
    }

    private void appendChild(XFileTree child) {
        if (children == null) {
            children = new Vector();
        }

        children.addElement(child);
    }

    public XFileTree nextChild() {
        return (XFileTree) enumeration.nextElement();
    }

    public boolean hasMoreChildren() {
        if (children == null) {

            return false;
        }

        if (enumeration == null) {
            enumeration = children.elements();
        }

        boolean ret = enumeration.hasMoreElements();

        if (!ret) {
            enumeration = null;
        }

        return ret;
    }

    private void appendFile(FileChild child) {
        if (files == null) {
            files = new Vector();
        }

        files.addElement(child);
    }

    public FileChild nextFile() {
        return (FileChild) fenumeration.nextElement();
    }

    public boolean hasMoreFiles() {
        if (files == null) {

            return false;
        }

        if (fenumeration == null) {
            fenumeration = files.elements();
        }

        boolean ret = fenumeration.hasMoreElements();

        if (!ret) {
            fenumeration = null;
        }

        return ret;
    }

    public class FileChild {
        public String name;
        public long size = 0;
        public long date = 0;

        public FileChild(String n) {
            name = n;
        }

        public void writeDirectory(OutputStream out)
                throws Exception {
            TextToken t = new TextToken(name);

            t.writeTo(out);
            out.write(' ');
            t = new TextToken(date);
            t.writeTo(out);
            out.write(' ');
            t = new TextToken(size);
            t.writeTo(out);
            out.write('\n');
        }
    }

    public static void main(String argv[])
            throws Exception {
        XFileTree f = new XFileTree("C:\\marcel\\projects\\src\\jshell");

        f.writeDirectory(System.out);
    }
}
