//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.assertion;

import java.io.Serializable;

/**
 * This type was created in VisualAge.
 *
 * @author Marcel Baumann
 * @invariant
 */
public class Assert implements Serializable {



    private static LogImpl logger = null;

    // Flags indicate whether or not assertions are made at runtime
    private static boolean preconditionOn = true;
    private static boolean postconditionOn = true;
    private static boolean assertOn = true;
    private static boolean invariantOn = true;

    //printModes:
    public static final int SILENT = 0;
    public static final int PRINT_FAILURES = 1;
    public static final int PRINT_ALL = 2;
    public static int printMode = SILENT;

    //!! These classes are temporary and should be removed
    //!! once diagnostic services are in place.
    //!! MB 30.11.1998
    private static class LogImpl {
    }

    private static class Log {
        public static LogImpl getImpl() {
            return null;
        }
    }

    /**
     * There are three print modes:
     * SILENT - no assertions are printed
     * PRINT_FAILURES - only failed assertions are printed
     * PRINT_ALL - all assertions are printed
     *
     * @param logMode int
     * @author Marcel Baumann
     */
    public static void setPrintMode(int printMode) {
        Assert.printMode = printMode;
    }

    /**
     * This method was created in VisualAge.
     *
     * @param cond    boolean
     * @param message java.lang.String
     * @author Marcel Baumann
     */
    public static void assertion(boolean cond, String message) {
        //Assert.getLogger().trace(LevelOfDetail.Constants.LOW, message, e);
        if (assertOn && !cond) {
            RuntimeException e = new AssertionException(message);
            if (printMode == PRINT_FAILURES) System.out.println(message);
            throw e;
        }
        if (printMode == PRINT_ALL) System.out.println(message);
    }

    /**
     * This method was created for jnex
     *
     * @return com.nexirius.util.log.LogImpl
     * @author Marcel Baumann
     */
    private static LogImpl getLogger() {
        if (logger == null) {
            Assert.logger = Log.getImpl();
        }
        return Assert.logger;
    }

    /**
     * This method was created in VisualAge.
     *
     * @param cond    boolean
     * @param message java.lang.String
     * @author Marcel Baumann
     */
    public static void inv(boolean cond, String message) {
        //Assert.getLogger().trace(LevelOfDetail.Constants.LOW, message, e);
        if (invariantOn && !cond) {
            if (printMode == PRINT_FAILURES) System.out.println(message);
            RuntimeException e = new InvariantException(message);
            throw e;
        }
        if (printMode == PRINT_ALL) System.out.println(message);
    }

    /**
     * This method was created in VisualAge.
     *
     * @param cond    boolean
     * @param message java.lang.String
     * @author Marcel Baumann
     */
    public static void post(boolean cond, String message) {
        // Assert.getLogger().trace(LevelOfDetail.Constants.LOW, message, e);
        if (postconditionOn && !cond) {
            if (printMode == PRINT_FAILURES) System.out.println(message);
            RuntimeException e = new PostconditionException(message);
            throw e;
        }
        if (printMode == PRINT_ALL) System.out.println(message);
    }

    /**
     * This method was created in VisualAge.
     *
     * @param cond    boolean
     * @param message java.lang.String
     * @author Marcel Baumann
     */
    public static void pre(boolean cond, String message) {
        // Assert.getLogger().trace(LevelOfDetail.Constants.LOW, message, e);
        if (preconditionOn && !cond) {
            if (printMode == PRINT_FAILURES) System.out.println(message);
            RuntimeException e = new PreconditionException(message);
            throw e;
        }
        if (printMode == PRINT_ALL) System.out.println(message);
    }

    /**
     * This method was created in VisualAge.
     *
     * @param logger com.nexirius.util.log.LogImpl
     * @author Marcel Baumann
     */
    public static void setLogger(LogImpl logger) {
        Assert.pre(logger != null, "Parameter logger non-null");
        Assert.logger = logger;
    }
}
