//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.resource;

import com.nexirius.framework.FWLog;
import com.nexirius.util.EventMulticaster;
import com.nexirius.util.assertion.Assert;
import com.nexirius.util.textresource.TextResourceImpl;
import javax.swing.*;
import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * See com.nexirius.util.resource.ClientResource for documentation.
 *
 * @author Marcel Baumann
 */
public class ClientResourceImpl extends TextResourceImpl implements ClientResource {

    protected static DateFormat DEFAULT_DATE_FORMAT = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    protected static Color DEFAULT_COLOR = Color.black;
    protected static int DEFAULT_FONT_SIZE = 12;
    protected static String DEFAULT_FONT_FAMILY = "SansSerif";
    protected static Font DEFAULT_FONT = new Font(DEFAULT_FONT_FAMILY, Font.PLAIN, DEFAULT_FONT_SIZE);

    Object resourceChangeListenerList = null;

    @Override
    public void addResourceChangeListener(ResourceChangeListener listener) {
        //FIX (memory blocking) resourceChangeListenerList = EventMulticaster.addEventListener(resourceChangeListenerList, listener);
    }

    @Override
    public void removeResourceChangeListener(ResourceChangeListener listener) {
        resourceChangeListenerList = EventMulticaster.removeEventListener(resourceChangeListenerList, listener);
    }

    public void fireResourceChange(ResourceChangeEvent event) {
        Object listeners[] = EventMulticaster.getArray(resourceChangeListenerList);

        for (int i = 0; listeners != null && i < listeners.length; ++i) {
            ((ResourceChangeListener) listeners[i]).resourceChange(event);
        }
    }



    //protected transient com.nexirius.util.resource.ResourceChangeListener listener = null;
//	protected transient com.nexirius.util.resource.ResourceChangeListener aResourceChangeListener = null;
    /**
     * ClientResourceImpl constructor comment.
     */
    public ClientResourceImpl(String resourceName) {
        super(resourceName);
    }
    /*
     *
     * @param newListener com.nexirius.util.resource.ResourceChangeListener
     public void addResourceChangeListener(com.nexirius.util.resource.ResourceChangeListener newListener)
     {
     aResourceChangeListener = com.nexirius.util.resource.ResourceChangeEventMulticaster.add(aResourceChangeListener, newListener);
     return;
     }
     */

    /*
     * Method to support listener events.
     protected void fireResourceChange(com.nexirius.util.resource.ResourceChangeEvent event)
     {
     if (aResourceChangeListener == null)
     {
     return;
     };
     aResourceChangeListener.resourceChange(event);
     }
     */

    @Override
    public void setParent(ClientResource clientResource) {
        super.setParent(clientResource);
    }

    /**
     * Checks wheter the resourceKey is defined
     */
    @Override
    public boolean isDefined(String resourceKey) {
        return getText(resourceKey) != null;
    }

    /**
     * Gets the resource value as text and translates it into an int
     */
    @Override
    public int getInt(String resourceKey) {
        String v = getText(resourceKey);

        Assert.pre(v != null, "Integer Resource '" + resourceKey + "' not defined");

        try {
            return Integer.parseInt(v);
        } catch (Exception ex) {
            throw new RuntimeException("Integer Resource '" + resourceKey + "' not a valid integer (value = '" + v + "')");
        }
    }

    /**
     * Gets the resource value as text and translates it into a double
     */
    public double getDouble(String resourceKey) {
        String v = getText(resourceKey);

        Assert.pre(v != null, "Double Resource '" + resourceKey + "' not defined");

        try {
            return Double.valueOf(v).doubleValue();
        } catch (Exception ex) {
            throw new RuntimeException("Integer Resource '" + resourceKey + "' not a valid integer (value = '" + v + "')");
        }
    }

    /**
     * Returns the text displayed for a button in the current language.
     *
     * @return java.lang.String
     */
    @Override
    public String getButtonText(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".buttonText");

        if (debugging && text != null) {
            return "[" + resourceKey + "]" + text;
        }

        return text;
    }

    @Override
    public java.net.URL getHelpURL(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".help");
        if (text == null) return null;
        try {
            return new URL(text);
        } catch (MalformedURLException e) {
            return null;
        }
    }

    @Override
    public String getHelpString(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");

        return getText(resourceKey + ".help");
    }

    /**
     * getIcon returns an Icon.
     * !!TODO this implementation is a tempoorary HACK!!!  It should really use getClass().getResource()
     * so it can run from applications and applets.
     */
    @Override
    public javax.swing.Icon getIcon(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".icon");
        javax.swing.Icon icon = null; //text==null ? null : new ImageIcon(getClass().getResource(text));

        if (text != null && text.length() > 0) {
            URL url = Util.class.getClassLoader().getResource(text);
            //URL url = java.lang.ClassLoader.getSystemResource(text);

            if (url != null) {
                icon = new ImageIcon(url);
            }
        }
        return icon;
    }

    /**
     * Same as getText() but returns the key itself instead of null when not defined
     */
    @Override
    public String getLabel(String resourceKey) {
        String ret = getText(resourceKey);

        if (ret == null) {
            return resourceKey;
        }

        if (debugging) {
            return "[" + resourceKey + "]" + ret;
        }

        return ret;
    }

    /**
     * An accelerator key is a shortcut to a command or event.  Examples
     * are cut Ctrl+X, copy Ctrl+C, and paste Ctrl+V.
     * Note this is not the same as a mnemonic key, see getMnemonic().
     */
    @Override
    public String getKeyAcceleratorText(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".accelerator");
        return text;
    }

    /**
     * Return the text that should appear on a menu status in the current language.
     */
    @Override
    public String getMenuText(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".menu");

        if (debugging && text != null) {
            return "[" + resourceKey + "]" + text;
        }

        if (text == null) {
            return null;
        }

        int pos = text.indexOf("$");
        if (pos >= 0) {
            text = text.substring(0, pos) + text.substring(pos + 1);
        }
        return text;
    }

    /**
     * A mnemonic is "hot" key you can type to activate a button or menu status.  It is
     * indicated by an underlined letter (in windows anyway).  It is not necisaraly the
     * same as a accelerator key, because an accelerator key can fire an event even when
     * its menu is not visible for example.
     * getMnemonic calls getMenuText() and uses the character that directly preceeds $ as
     * the mnemonic key.  If no $ marker is found returns -1 and no mnemonic is set.
     * TODO!! probably shouldn't use $ for the mnemonic because then the bank can't put
     * dallor amounts in the menus!!
     * TODO!! Clarify accelerator key and mnemonic relationship and naming standards.
     */
    @Override
    public int getMnemonic(String resourceKey, String ext) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ext);
        if (text == null) return -1;
        int pos = text.indexOf("$");

        if (pos >= text.length()) {
            pos = -1;
        }

        return pos;
    }

    /**
     * Tool tips are help text that appears if you hold the cursor over a widget for a few seconds.
     * Returns the tool tip in the current language.
     */
    @Override
    public String getToolTipText(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".tip");
        text = text != null && text.length() == 0 ? null : text;
        return text;
    }

    /**
     * Appends .dateformat to the resourceKey and returns a Date Format Instance from the resource bundel.
     * Return DEFAULT_DATE_FORMAT if resource not found.
     *
     * @return java.text.DateFormat
     */
    @Override
    public DateFormat getDateFormat(String resourceKey) {
        Assert.pre(resourceKey != null, "resourceKey is not null");
        String text = getText(resourceKey + ".dateformat");

        if (text == null) {
            return DEFAULT_DATE_FORMAT;
        }

        return new SimpleDateFormat(text);
    }

    /**
     * Checks if resourceKey is one of the predefined color strings or appends .color to the
     * resourceKey and returns a Color Instance from the resource bundel.
     * Return DEFAULT_COLOR if resource not found.
     *
     * @return java.awt.Color
     */
    @Override
    public Color getColor(String resourceKey) {
        //System.out.println("getColor " + resourceKey);

        if (DEFAULT.equals(resourceKey)) {
            return DEFAULT_COLOR;
        }

        if (resourceKey.startsWith(SYSTEMCOLOR)) {
            // try to extract the associated system color
            PredefinedColor c = predefinedColors.get(resourceKey);

            if (c != null) {
                return c.color;
            }
        }

        String value = getText(resourceKey + ".color");

        if (value != null) {
            if (value.equals("null")) {
            		return null;
            }

            // try to translate the String into a Color intance
            return parseColor(value, DEFAULT_COLOR);
        }

        PredefinedColor c = predefinedColors.get(resourceKey);

        if (c != null) {
            return c.color;
        }

        Color ret = parseColor(resourceKey, null);

        if (ret != null) {
            return ret;
        }

        // try to translate the String into a Color intance
        return Color.getColor(resourceKey, DEFAULT_COLOR);
    }

    public static Color parseColor(String value, Color def) {
        Color ret = def;
        StringTokenizer st = new StringTokenizer(value, " \t,");

        if (st.countTokens() == 3) {
            try {
                String r = st.nextToken();
                String g = st.nextToken();
                String b = st.nextToken();
                int red = Integer.parseInt(r);
                int green = Integer.parseInt(g);
                int blue = Integer.parseInt(b);

                ret = new Color(Math.min(255, red), Math.min(255, green), Math.min(255, blue));

            } catch (Exception ex) {
                // ignore
                throw new RuntimeException(ex);
            }
        } else {
            try {
                ret = new Color(Integer.parseInt(value, 16));
            } catch (NumberFormatException ex) {
                throw new RuntimeException(ex);
            }
        }

        return ret;
    }

    /**
     * Appends .font to the resourceKey and returns a Font instance from the resource bundel.
     * </pre>
     * The property value should be one of the following forms:
     * fontname-style-pointsize
     * fontname-pointsize
     * fontname-style
     * fontname
     * where style is one of the three strings "BOLD", "BOLDITALIC", or "ITALIC", and point size is a decimal representation of the point size.
     * <p/>
     * The default style is PLAIN. The default point size is 12.
     * <\pre>
     * Return a default font if resource not found.
     *
     * @return java.awt.Font
     */
    @Override
    public Font getFont(String resourceKey) {
        if (DEFAULT.equals(resourceKey)) {
            return DEFAULT_FONT;
        }

        String descr = getText(resourceKey + ".font");

        if (descr == null) {
            descr = resourceKey;
        }

        try {
            //return Font.getFont(descr, DEFAULT_FONT);
            //FIX (Font.getFont(descr, DEFAULT_FONT); crashes

            String family = DEFAULT_FONT_FAMILY;
            int style = Font.PLAIN;
            int size = DEFAULT_FONT_SIZE;

            StringTokenizer st = new StringTokenizer(descr, "-");

            switch (st.countTokens()) {
                case 1:
                    family = st.nextToken();
                    break;

                case 2:
                    family = st.nextToken();
                    String t2 = st.nextToken();

                    try {
                        size = Integer.parseInt(t2);
                    } catch (Exception ex) {
                        style = getStyle(t2);
                    }
                    break;

                case 3:
                    family = st.nextToken();
                    style = getStyle(st.nextToken());
                    size = Integer.parseInt(st.nextToken());
                    break;

            }

            return new Font(family, style, size);
        } catch (Exception ex) {
// FIX
            FWLog.error(ex);
        }

        return DEFAULT_FONT;
    }

    private int getStyle(String token) {
//System.out.println("getStyle for " + token);

        if (token.equals("BOLD")) {
            return Font.BOLD;
        } else if (token.equals("ITALIC")) {
            return Font.ITALIC;
        } else if (token.equals("BOLDITALIC")) {
            return Font.ITALIC | Font.BOLD;
        }

        return Font.PLAIN;
    }

    /**
     * Appends .fontFamily to the resourceKey and returns a font family name from the resource bundel.
     * Return a default string if resource not found.
     *
     * @return font family identifier
     */
    @Override
    public String getFontFamily(String resourceKey) {
        if (DEFAULT.equals(resourceKey)) {
            return DEFAULT_FONT_FAMILY;
        }
        try {
            return getText(resourceKey + ".fontFamily");
        } catch (Exception ex) {
            FWLog.error(ex);
            // ignore
        }

        return DEFAULT_FONT_FAMILY;
    }

    /**
     * Appends .fontSize to the resourceKey and returns an int from the resource bundel.
     * Return a default size if resource not found.
     *
     * @return a font size
     */
    @Override
    public int getFontSize(String resourceKey) {
        if (DEFAULT.equals(resourceKey)) {
            return DEFAULT_FONT_SIZE;
        }
        try {
            return getInt(resourceKey + ".fontSize");
        } catch (Exception ex) {
            FWLog.error(ex);
            // fix
        }

        return DEFAULT_FONT_SIZE;
    }

    /**
     * Appends .currencyNumberFormat to the currencyCode and returns a format string (e.g. 'Fr.' #,##0.00;'Fr.' -#,##0.00)
     * Returns null if resource not found.
     *
     * @return a format description string
     */
    @Override
    public String getCurrencyNumberFormat(String currencyCode) {
        String ret = getText(currencyCode + ".currencyNumberFormat");

        if (ret == null) {
            FWLog.debug(currencyCode + ".currencyNumberFormat" + " is not defined");

            return null;
        }

        return ret;
    }

    /**
     * Appends .currencyRoundingDenominator to the currencyCode and returns a double (e.g. 0.05)
     * Returns 0.0 if resource not found.
     *
     * @return the smallest denominator
     */
    @Override
    public double getCurrencyRoundingDenominator(String currencyCode) {
        try {
            return getDouble(currencyCode + ".currencyRoundingDenominator");
        } catch (Exception ex) {
            FWLog.debug(currencyCode + ".currencyRoundingDenominator" + " is not defined");
        }

        return 0.0;
    }


//     public void removeResourceChangeListener(com.nexirius.util.resource.ResourceChangeListener newListener) {
//     aResourceChangeListener = com.nexirius.util.resource.ResourceChangeEventMulticaster.remove(aResourceChangeListener, newListener);
//     return;
//     }

    @Override
    public void updateListeners() {
        fireResourceChange(new ResourceChangeEvent(this, this));
    }

    private static PredefinedColor[] systemColors = {
        new PredefinedColor(COLOR_DESKTOP, SystemColor.desktop),
        new PredefinedColor(COLOR_ACTIVE_CAPTION, SystemColor.activeCaption),
        new PredefinedColor(COLOR_ACTIVE_CAPTION_TEXT, SystemColor.activeCaptionText),
        new PredefinedColor(COLOR_ACTIVE_CAPTION_BORDER, SystemColor.activeCaptionBorder),
        new PredefinedColor(COLOR_INACTIVE_CAPTION, SystemColor.inactiveCaption),
        new PredefinedColor(COLOR_INACTIVE_CAPTION_TEXT, SystemColor.inactiveCaptionText),
        new PredefinedColor(COLOR_INACTIVE_CAPTION_BORDER, SystemColor.inactiveCaptionBorder),
        new PredefinedColor(COLOR_WINDOW, SystemColor.window),
        new PredefinedColor(COLOR_WINDOW_BORDER, SystemColor.windowBorder),
        new PredefinedColor(COLOR_WINDOW_TEXT, SystemColor.windowText),
        new PredefinedColor(COLOR_MENU, SystemColor.menu),
        new PredefinedColor(COLOR_MENU_TEXT, SystemColor.menuText),
        new PredefinedColor(COLOR_TEXT, SystemColor.text),
        new PredefinedColor(COLOR_TEXT_TEXT, SystemColor.textText),
        new PredefinedColor(COLOR_TEXT_HIGHLIGHT, SystemColor.textHighlight),
        new PredefinedColor(COLOR_TEXT_HIGHLIGHT_TEXT, SystemColor.textHighlightText),
        new PredefinedColor(COLOR_TEXT_INACTIVE_TEXT, SystemColor.textInactiveText),
        new PredefinedColor(COLOR_CONTROL, SystemColor.control),
        new PredefinedColor(COLOR_CONTROL_TEXT, SystemColor.controlText),
        new PredefinedColor(COLOR_CONTROL_HIGHLIGHT, SystemColor.controlHighlight),
        new PredefinedColor(COLOR_CONTROL_LT_HIGHLIGHT, SystemColor.controlLtHighlight),
        new PredefinedColor(COLOR_CONTROL_SHADOW, SystemColor.controlShadow),
        new PredefinedColor(COLOR_CONTROL_DK_SHADOW, SystemColor.controlDkShadow),
        new PredefinedColor(COLOR_SCROLLBAR, SystemColor.scrollbar),
        new PredefinedColor(COLOR_INFO, SystemColor.info),
        new PredefinedColor(COLOR_INFO_TEXT, SystemColor.infoText),
        new PredefinedColor("black", Color.black),
        new PredefinedColor("blue", Color.blue),
        new PredefinedColor("cyan", Color.cyan),
        new PredefinedColor("darkGray", Color.darkGray),
        new PredefinedColor("gray", Color.gray),
        new PredefinedColor("green", Color.green),
        new PredefinedColor("lightGray", Color.lightGray),
        new PredefinedColor("magenta", Color.magenta),
        new PredefinedColor("orange", Color.orange),
        new PredefinedColor("pink", Color.pink),
        new PredefinedColor("red", Color.red),
        new PredefinedColor("white", Color.white),
        new PredefinedColor("yellow", Color.yellow),
    };


    private static Hashtable<String, PredefinedColor> predefinedColors = null;

    private static void updateSystemColors() {

        predefinedColors = new Hashtable<String, PredefinedColor>();

        for (int i = 0; i < systemColors.length; ++i) {
            predefinedColors.put(systemColors[i].name, systemColors[i]);
        }

    }


    private static class PredefinedColor {

        String name;
        Color color;

        PredefinedColor(String name, Color color) {
            this.name = name;
            this.color = color;
        }

/* Not accesible because this is not an override of equals(Object)!      public boolean equals(PredefinedColor other) {
            return other.name.equals(this.name);
        }*/

        @Override
        public int hashCode() {
            return name.hashCode();
        }
    }

    static {
        updateSystemColors();
    }
}
