//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.unittest;

public abstract class UnitTest implements UnitTestReport {
    private UnitTestProgress m_utp = null;
    private StringBuffer progress = new StringBuffer();
    private StringBuffer tests = new StringBuffer();
    private boolean m_passed = true;
    private boolean m_moreProgress = false;
    private int m_testsRun = 0;
    private int m_testsPassed = 0;
    private int m_testStep = 1;

    public void assertSame(String message, Object expect, Object have) {
        message(message + " expect[" + expect + "] have [" + have + "]");
        assertTrue(expect == have);
    }

    public void assertEquals(String message, Object expect, Object have) {
        message(message + " expect[" + expect + "] have [" + have + "]");
        if (expect == null) {
            assertTrue(expect == have);
        } else {
            assertTrue(expect.equals(have));
        }
    }

    public void assertEquals(String message, int expect, int have) {
        message(message + " expect[" + expect + "] have [" + have + "]");
        assertTrue(expect == have);
    }

    public void assertTrue(String message, boolean b) {
        message(message);
        assertTrue(b);
    }

    public String getUnitTestName() {
        return getClass().getName();
    }

    public void addProgress(String p) {
        progress.append(p);
        if (m_utp != null)
            m_utp.progress(p);
        else
            System.out.print(p);
    }

    public String getProgress() {
        return progress.toString();
    }

    public void addTest(String t) {
        tests.append(t + "\n");

        if (m_utp == null) {
            System.out.print(t + "\n");
        } else {
            m_utp.reportFailure(t + "\n");
        }
    }

    public String getTest() {
        return tests.toString();
    }


    protected void setUnitTestProgress(UnitTestProgress utp) {
        m_utp = utp;
    }

    final protected boolean passed() {
        return m_passed;
    }

    public final void assertTrue(boolean passed) {
        String pf;
        if (passed) {
            pf = " - passed.\n";
        } else {
            pf = " - failed.\n";
        }

        m_passed &= passed;

        m_testsRun++;
        m_testsPassed++;
        addProgress(pf);
        m_testStep++;
        m_moreProgress = false;
    }

    public final void setFailureReason(String why) {
        addTest(String.valueOf(m_testStep) + ": " + why);
    }

    public abstract void performUnitTest();

    public final void message(String msg) {
        if (m_moreProgress)
            addProgress("\n");
        addProgress(String.valueOf(m_testStep) + ": " + msg);
        m_moreProgress = true;
    }
}
